
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createProdDetLin.c
* Description:  Contains routines for the product files handling
*
* History:      
* 14-Jun-04     (csabet)
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/stat.h>
#include <stdio.h>
#include <cpl.h>
#include <time.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "fileHandling.h"
#include "createProdDetLin.h"
#include "cpl_table.h"
#include "qfits.h"

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createDetLinProd
* Input/Output: See function arguments to avoid duplication
* Description:  Creates MIDI product files
*
* History:      
* 15-Jun-04     (csabet) Created
******************************************************************************/
void createDetLinProd (
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat        *format,    // In: Image format
    int                numOfFiles,    // In: Number of files
    DetLinearity    *linearity,    // Ou: Pointer to the detector linearity data structure
    int                *error)        // Ou: Error status
{
 
    //  Local Declarations
    //    ------------------
    const char  routine[] = "createDetLinProd";
    char        *classification, *stringTemp;
    int         i, j, k;
    FILE        *inFitsBatchPtr;
    float        *detLinImage, max;
    int            *select, arraySize, min;
    qfitsdumper    qdDetLin;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Product files for DET_LIN from batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"--------------------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Product files for DET_LIN from batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "--------------------------------------------- \n");

    // Reset status
    *error = 0;
    arraySize = 10 * numOfFiles;
    
    //    Create the primary header extension
    createDetLinPrimHead (fileNames, format, arraySize, linearity, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Primary Header extension");
        return;
    }

    //    Allocate memory
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    detLinImage = (float *) calloc (arraySize * arraySize, sizeof (float));
    select = (int *) calloc (arraySize, sizeof (int));

    //    Convert Linearity vector into an image and write 
    //    into the FITS file. Rather tedious and long-handed method!
    max = linearity->mean[0];
    for (i = 0; i < numOfFiles; i++)
        if (linearity->mean[i] > max) max = linearity->mean[i];

    for (i = 0; i < arraySize; i++)
        select[i] = (int) ((100 * linearity->mean[i/10]) / max);
        
    min = select[0];
    for (i = 0; i < arraySize; i++)
        if (select[i] < min) min = select[i];
    for (i = 0; i < arraySize; i++)
        select[i] -= min;

    for (i = 0; i < arraySize; i++)
    {
        for (j = 0; j < arraySize; j++)
        {
            k = i * arraySize + j;
            if (i+1 == select[j])    // i+1 in order to include last element
            {
                detLinImage[k] = 1.0;
                detLinImage[k+1] = 1.0;    // This is safe since arraySize > 2 * numOfFiles
                break;
            }
        }
    }
    qdDetLin.filename  = fileNames->outFitsName;
    qdDetLin.npix      = arraySize * arraySize;
    qdDetLin.ptype     = PTYPE_FLOAT;
    qdDetLin.fbuf      = detLinImage;
    qdDetLin.out_ptype = BPP_IEEE_FLOAT;
    qfits_pixdump (&qdDetLin);

    free (select);
    free (detLinImage);

    //  Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (stringTemp);
        free (classification);
        *error = 1;
        return;
    }
        
    //  Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    // Copy keywords from the raw FITS file into the QC log
    createDetLinQcLog (fileNames->inFitsName, error);
    if (*error)
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot copy keywords to QC log");

    //    Add Product info to QC log
    addProdInfoToDetLinQcLog (format, fileNames, linearity, error);

    //  Release memory
    fclose (inFitsBatchPtr);
    free (classification);
    free (stringTemp);
 
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createDetLinPrimHead
* Input/Output: See function arguments to avoid duplication
* Description:  Creates the primary header
*
*
* History:
* 08-Jun-05     (csabet) Created
******************************************************************************/
void createDetLinPrimHead (
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat        *format,    // In: Image format
    int                size,        // In: Number of files
    DetLinearity    *linearity,    // Ou: Pointer to the detector linearity data structure
    int                *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "createDetLinPrimHead";
    qfits_header    *outFitsHeader;
    FILE            *inFitsBatchPtr=NULL, *outFitsPtr;
    int             i;
    char            *textBuff, *stringQfits, *messageBuffer, *currentTime, *cleanString,
                    *stringTemp, *classification, *sWidthX, *sWidthY, *tech;
    time_t          now;
    struct tm       *newTime;
    struct stat     buf;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset parameters
    *error = 0;

    //    Open list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        *error = 1;
        return;
    }

    //    Allocate memory
    tech = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthX = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthY = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    //    Copy primary header from the raw file to the output header
    outFitsHeader = qfits_header_read (fileNames->inFitsName);
    if (outFitsHeader == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load header from the input FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);
        return;
    }

    //    Write all NAXIS values as character strings
    sprintf (sWidthY, "%d", size);
    sprintf (sWidthX, "%d", size);

    //    Updates header for the DetRon output file
    qfits_header_mod (outFitsHeader, "BITPIX", "-32", "number of bits per pixel"); 
    qfits_header_mod (outFitsHeader, "NAXIS", "2", "number of data axes"); 
    qfits_header_add (outFitsHeader, "NAXIS1", sWidthX, "", NULL);
    qfits_header_add (outFitsHeader, "NAXIS2", sWidthY, "", NULL);
    qfits_header_mod (outFitsHeader, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );  

    //    Now write all the required product keaywords to the output header
    //    -----------------------------------------------------------------
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TYPE", format->obsType, "MIDI pipeline product type", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "REDUCED_DETLIN", "Pipeline product category", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", NULL);
    qfits_header_add (outFitsHeader, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO DID", MIDI_QC_DIC_VERSION, "QC dictionary version", NULL);

    //    Rewind the list of files
    rewind (inFitsBatchPtr);
    
    //    Loop through the list of files and write into the primary header
    i = 0;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    Get Technique
        stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR TECH");
        if (stringQfits == NULL)
        {
            sprintf (tech, "%s", "UNKNOWN");
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Technique");
        }
        else
        {
            cleanUpString (stringQfits, cleanString);
            sprintf (tech, "%s", cleanString);
        }
        
        //    If classification is not given get it from the raw file
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d NAME", i+1);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "FITS file name", "");
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d CATG", i+1);
        qfits_header_add (outFitsHeader, textBuff, classification, "Observation Categoty", "");

        if (strcmp (tech, "OTHER") != 0)
        {
            sprintf (messageBuffer, "%d", linearity->saturated[i]);
            sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d STATUS", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "1 means saturated", "");
            sprintf (messageBuffer, "%f", linearity->integTime[i]);
            sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d DIT", i+1);
            qfits_header_add (outFitsHeader, textBuff, messageBuffer, "Integration time", "");
        }
        i++;
    }
    fclose (inFitsBatchPtr);

    //    Add Image parameters
    if (strcmp (linearity->grismId, "OPEN") == 0)
    {
        sprintf (messageBuffer, "%s", linearity->grismId);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM NAME", messageBuffer, "Grism Name", "");
        sprintf (messageBuffer, "%d", linearity->winx);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM WINX", messageBuffer, "x coordinate", "");
        sprintf (messageBuffer, "%d", linearity->windx);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM WINDX", messageBuffer, "dx coordinate", "");
        sprintf (messageBuffer, "%d", linearity->winy);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM WINY", messageBuffer, "y coordinate", "");
        sprintf (messageBuffer, "%d", linearity->windy);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM WINDX", messageBuffer, "dx coordinate", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA0);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM A0", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM A1", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM A2", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM A3", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanSigma);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN IM SIG", messageBuffer, "Mean linearity standard deviation", "");
    }
    else if (strcmp (linearity->grismId, "PRISM") == 0)
    {
        sprintf (messageBuffer, "%s", linearity->grismId);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR NAME", messageBuffer, "Grism Name", "");
        sprintf (messageBuffer, "%d", linearity->winx);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR WINX", messageBuffer, "x coordinate", "");
        sprintf (messageBuffer, "%d", linearity->windx);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR WINDX", messageBuffer, "dx coordinate", "");
        sprintf (messageBuffer, "%d", linearity->winy);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR WINY", messageBuffer, "y coordinate", "");
        sprintf (messageBuffer, "%d", linearity->windy);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR WINDX", messageBuffer, "dx coordinate", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA0);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR A0", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR A1", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR A2", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR A3", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanSigma);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN PR SIG", messageBuffer, "Mean linearity standard deviation", "");
    }
    else if (strcmp (linearity->grismId, "GRISM") == 0)
    {
        sprintf (messageBuffer, "%s", linearity->grismId);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR NAME", messageBuffer, "Grism Name", "");
        sprintf (messageBuffer, "%d", linearity->winx);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR WINX", messageBuffer, "x coordinate", "");
        sprintf (messageBuffer, "%d", linearity->windx);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR WINDX", messageBuffer, "dx coordinate", "");
        sprintf (messageBuffer, "%d", linearity->winy);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR WINY", messageBuffer, "y coordinate", "");
        sprintf (messageBuffer, "%d", linearity->windy);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR WINDX", messageBuffer, "dx coordinate", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA0);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR A0", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA1);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR A1", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA2);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR A2", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanCoeffA3);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR A3", messageBuffer, "Linearity Coefficient", "");
        sprintf (messageBuffer, "%f", linearity->meanSigma);
        qfits_header_add (outFitsHeader, "HIERARCH ESO QC DETLIN GR SIG", messageBuffer, "Mean linearity standard deviation", "");
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Dispersive element");
        *error = 1;
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }


    //    Place Holders
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH1", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH2", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH3", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH4", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH5", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH6", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH7", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH8", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH9", "TBD", "TBD", "");
    
    //    Create the output fits file. If the file exist delete it
    if (stat (fileNames->outFitsName, &buf) == 0) remove (fileNames->outFitsName);

    outFitsPtr = fopen (fileNames->outFitsName, "w");
    if (!outFitsPtr)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create output FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);        
        return;
    }

   cpl_msg_info(cpl_func,"Created Product FITS file: %s \n", fileNames->outFitsName);
    fprintf (midiReportPtr, "Created Product FITS file: %s \n", fileNames->outFitsName);

    //    Write header into the output file
    qfits_header_sort (&outFitsHeader);
    qfits_header_dump (outFitsHeader, outFitsPtr);
    fclose (outFitsPtr);
    qfits_header_destroy (outFitsHeader);

    //    release memory
    free (messageBuffer);
    free (currentTime);
    free (cleanString);
    free (stringTemp);
    free (classification);
    free (tech);
    free (textBuff);
    free (sWidthX);    
    free (sWidthY);

    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    createDetLinQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Copies keywords from the raw input FITS primary header to the 
*               QC log and adds additional product keywords
*
* History:        
* 30-Aug-04        (csabet) Created
******************************************************************************/
void createDetLinQcLog (
    char    *inFitsName,//    In:    Name of the raw FITS file
    int        *error)        //    Ou:    Status
    
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "createDetLinQcLog";
    char            *qfitsString, *emptyString, *cleanString;

    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset status */
    *error = 0;

    /* Allocate memory */
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*    Initialise empty string */
    strcpy (emptyString, "\"UNKNOWN\"");

    /* Get all the keywords */
    qfitsString = qfits_query_hdr (inFitsName, "ARCFILE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ARCFILE             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ARCFILE             \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "TELESCOP");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TELESCOP            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TELESCOP            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "INSTRUME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INSTRUME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INSTRUME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "MJD-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "MJD-OBS             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "MJD-OBS             \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "DATE-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DATE-OBS            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DATE-OBS            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "UTC");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "UTC                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "UTC                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "LST");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "LST                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "LST                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DITDELAY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET INT MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.INT.MODE        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.INT.MODE        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NDIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NRTS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR CATG");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.CATG            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.CATG            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TECH");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TECH            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TECH            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TYPE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TYPE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS CAM NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS FILT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS GRIS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       \"%s\" \n", cleanString);}
        
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.MODE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.MODE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SHUT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SHUT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SLIT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.NAME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.NAME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PROG ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.START           \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS TARG NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OCS EXPO1 FNAME1");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.START           \"%s\" \n", cleanString);}
    
    /* Release memory */
    free (emptyString);
    free (cleanString);
    
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    addProdInfoToDetLinQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Adds product information to QC log
*
*
* History:        
* 16-Jan-04        (csabet) Created
******************************************************************************/
void addProdInfoToDetLinQcLog (
    ImageFormat        *format,    // In: Image format
    MidiFiles        *fileNames,
    DetLinearity    *linearity,    // Ou: Pointer to the detector linearity data structure
    int                *error)
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "addProdInfoToDetLinQcLog";
    int             i;
    FILE            *inFitsBatchPtr;
    char            *stringQfits, *messageBuffer, *currentTime, *textBuff, *cleanString,
                    *stringTemp, *classification, *tech;
    time_t          now;
    struct tm        *newTime;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset parameters */
    *error = 0;

    /*    Allocate memory */
    tech = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    
    /*    Get current time/date */
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Add keyword to QC log file
    fprintf (midiQcLogPtr, "PRO.TYPE            \"%s\" \n", format->obsType);
    fprintf (midiQcLogPtr, "PRO.CATG            \"REDUCED_DETLIN\" \n");
    fprintf (midiQcLogPtr, "PRO.ARCFILE         \"%s\" \n", fileNames->archFileName);
    fprintf (midiQcLogPtr, "PRO.PIPEDATE        \"%s\" \n", currentTime);
    fprintf (midiQcLogPtr, "PRO.VERSION         \"%s\" \n", MIDI_PIPE_VERSION);
    fprintf (midiQcLogPtr, "PRO.PIPEFILE        \"%s\" \n", fileNames->pipeFileName);
    fprintf (midiQcLogPtr, "PRO.DID             \"%s\" \n", MIDI_QC_DIC_VERSION);

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /* Loop through the list of files and write into the QC log */
    i = 0;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    Get Technique
        stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR TECH");
        if (stringQfits == NULL)
        {
            sprintf (tech, "%s", "UNKNOWN");
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Technique");
        }
        else
        {
            cleanUpString (stringQfits, cleanString);
            sprintf (tech, "%s", cleanString);
        }
        
        /* If classification is not given get it from the raw file */
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL) 
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.NAME        \"%s\" \n", i+1, messageBuffer);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.CATG        \"%s\" \n", i+1, classification);
        if (strcmp (tech, "OTHER") != 0)
        {
            fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.STATUS      %d \n", i+1, linearity->saturated[i]);
            fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.DIT         %f \n", i+1, linearity->integTime[i]);
        }
        i++;
    }
    fclose (inFitsBatchPtr);

    if (strcmp (linearity->grismId, "OPEN") == 0)
    {

        fprintf (midiQcLogPtr, "QC.DETLIN.IM.NAME  \"%s\" \n", linearity->grismId);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.WINX  %d \n", linearity->winx);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.WINDX %d \n", linearity->windx);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.WINY  %d \n", linearity->winy);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.WINDY %d \n", linearity->windy);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.A0    %f \n", linearity->meanCoeffA0);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.A1    %f \n", linearity->meanCoeffA1);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.A2    %f \n", linearity->meanCoeffA2);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.A3    %f \n", linearity->meanCoeffA3);
        fprintf (midiQcLogPtr, "QC.DETLIN.IM.SIG   %f \n", linearity->meanSigma);
    }
    else if (strcmp (linearity->grismId, "PRISM") == 0)
    {
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.NAME  \"%s\" \n", linearity->grismId);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.WINX  %d \n", linearity->winx);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.WINDX %d \n", linearity->windx);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.WINY  %d \n", linearity->winy);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.WINDY %d \n", linearity->windy);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.A0    %f \n", linearity->meanCoeffA0);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.A1    %f \n", linearity->meanCoeffA1);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.A2    %f \n", linearity->meanCoeffA2);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.A3    %f \n", linearity->meanCoeffA3);
        fprintf (midiQcLogPtr, "QC.DETLIN.PR.SIG   %f \n", linearity->meanSigma);
    }
    else if (strcmp (linearity->grismId, "GRISM") == 0)
    {
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.NAME  \"%s\" \n", linearity->grismId);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.WINX  %d \n", linearity->winx);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.WINDX %d \n", linearity->windx);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.WINY  %d \n", linearity->winy);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.WINDY %d \n", linearity->windy);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.A0    %f \n", linearity->meanCoeffA0);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.A1    %f \n", linearity->meanCoeffA1);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.A2    %f \n", linearity->meanCoeffA2);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.A3    %f \n", linearity->meanCoeffA3);
        fprintf (midiQcLogPtr, "QC.DETLIN.GR.SIG   %f \n", linearity->meanSigma);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Dispersive element");
        *error = 1;
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /*    release memory */
    free (tech);
    free (messageBuffer);
    free (currentTime);
    free (textBuff);
    free (cleanString);
    free (stringTemp);
    free (classification);
        
    return;
}
/*****************************************************************************/

