
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createProdRefPix.c
* Description:  Contains routines for product files handling
*
* History:      
* 14-Jun-04     (csabet)
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/stat.h>
#include <stdio.h>
#include <cpl.h>
#include <time.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "createProdRefPix.h"
#include "fileHandling.h"
#include "qfits.h"

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/




/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createRefPixProd
* Input/Output: See function arguments to avoid duplication
* Description:  Creates MIDI product files
*
* History:      
* 31-Aug-04     (csabet) Created
******************************************************************************/
void createRefPixProd (
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat        *format,    // In: Image format
    ReferencePixel    *refPix,    // In: Refrenece Pixel data structure
    int                *error)        // Ou: Error status
{
  
    //  Local Declarations
    //    ------------------
    const char  routine[] = "createRefPixProd";
    char        *tempFileName, *stringTemp, *classification;
    int         i, j;
    FILE        *tempFilePtr=NULL, *inFitsBatchPtr=NULL;
    float        *refPixImage;
    qfitsdumper    qdRefPix;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Product files for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Product for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "-------------------------- \n");

    // Reset status
    *error = 0;

    //    Create the primary header extension
    createRefPixPrimHead (fileNames, format, refPix, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Primary Header extension");
        return;
    }

    //    Allocate memory
    tempFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    refPixImage = (float *) calloc (format->iXWidth * format->iYWidth, sizeof (float));
    
    //    Create a dummy image
    qdRefPix.filename  = fileNames->outFitsName;
    qdRefPix.npix      = format->iXWidth * format->iYWidth;
    qdRefPix.ptype     = PTYPE_FLOAT;
    qdRefPix.fbuf      = refPixImage;
    qdRefPix.out_ptype = BPP_IEEE_FLOAT;
    qfits_pixdump (&qdRefPix);

    free (refPixImage);

    //    Open list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        *error = 1;
        return;
    }

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    //    Copy keywords from the raw FITS file into the QC log
    createRefPixQcLog (fileNames->inFitsName, error);
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot copy keywords to QC log");

    //    Add Product info to QC log
    addProdInfoToRefPixQcLog (format, fileNames, refPix, error);

    //    Create a centroid QC log
    sprintf (tempFileName, "%s%s.Centroid.txt", outFileDir, outRootName);
    tempFilePtr = fopen(tempFileName, "w");
    for (i = 0; i < refPix->numOfExposures; i++)
    {
        fprintf (tempFilePtr, "Exposure %d \n", i+1);
        for (j = 0; j < refPix->exposure[i].numOfBeams; j++)
        {
            fprintf (tempFilePtr, "%f %f %f \n", 
                refPix->exposure[i].centroid[j].xCoord,  
                refPix->exposure[i].centroid[j].yCoord, 
                refPix->exposure[i].centroid[j].size);
        }
    }
    
    //  Close files
    fclose (tempFilePtr);
    fclose (inFitsBatchPtr);

    //  Release memory
    free (tempFileName);
    free (stringTemp);
    free (classification);
 
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  createRefPixPrimHead
* Input/Output: See function arguments to avoid duplication
* Description:  Creates the primary header
*
*
* History:
* 25-Aug-06     (csabet) Created
******************************************************************************/
void createRefPixPrimHead (
    MidiFiles       *fileNames,    // In: Pointer to the file structure
    ImageFormat        *format,    // In: Image format
    ReferencePixel    *refPix,    // In: Refrenece Pixel data structure
    int                *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "createRefPixPrimHead";
    qfits_header    *outFitsHeader;
    FILE            *inFitsBatchPtr=NULL, *outFitsPtr;
    int             i;
    char            *textBuff, *stringQfits, *messageBuffer, *currentTime, *cleanString,
                    *stringTemp, *classification, *sWidthX, *sWidthY, *tech;
    time_t          now;
    struct tm       *newTime;
    struct stat     buf;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset parameters
    *error = 0;

    //    Open list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        *error = 1;
        return;
    }

    //    Allocate memory
    tech = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthX = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sWidthY = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    //    Copy primary header from the raw file to the output header
    outFitsHeader = qfits_header_read (fileNames->inFitsName);
    if (outFitsHeader == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load header from the input FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);
        return;
    }

    //    Write all NAXIS values as character strings
    sprintf (sWidthY, "%d", format->iYWidth);
    sprintf (sWidthX, "%d", format->iXWidth);

    //    Updates header for the DetRon output file
    qfits_header_mod (outFitsHeader, "BITPIX", "-32", "number of bits per pixel"); 
    qfits_header_mod (outFitsHeader, "NAXIS", "2", "number of data axes"); 
    qfits_header_add (outFitsHeader, "NAXIS1", sWidthX, "", NULL);
    qfits_header_add (outFitsHeader, "NAXIS2", sWidthY, "", NULL);
    qfits_header_mod (outFitsHeader, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );  

    //    Now write all the required product keaywords to the output header
    //    -----------------------------------------------------------------
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO TYPE", format->obsType, "MIDI pipeline product type", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO CATG", "REDUCED_REFPIX", "Pipeline product category", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", NULL);
    qfits_header_add (outFitsHeader, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", NULL);
    qfits_header_add (outFitsHeader, "HIERARCH ESO PRO DID", MIDI_QC_DIC_VERSION, "QC dictionary version", NULL);

//    qfits_header_consoledump(outFitsHeader);

    //    Rewind the list of files
    rewind (inFitsBatchPtr);
    
    //    Loop through the list of files and write into the primary header
    i = 0;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);

        //    Get Technique
        stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR TECH");
        if (stringQfits == NULL)
        {
            sprintf (tech, "%s", "UNKNOWN");
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Technique");
        }
        else
        {
            cleanUpString (stringQfits, cleanString);
            sprintf (tech, "%s", cleanString);
        }
        
        //    If classification is not given get it from the raw file
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL)
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d NAME", i+1);
        qfits_header_add (outFitsHeader, textBuff, messageBuffer, "FITS file name", "");
        sprintf (textBuff, "HIERARCH ESO PRO REC1 RAW%d CATG", i+1);
        qfits_header_add (outFitsHeader, textBuff, classification, "Observation Categoty", "");
        i++;
    }
    fclose (inFitsBatchPtr);

    //    Add Image parameters
    for (i = 0; i < refPix->numOfExposures; i++)
    {
        if (strcmp (refPix->exposure[i].beamCombiner, "OPEN") == 0)
        {
            if (strcmp (refPix->exposure[i].shutterId, "ABOPEN") == 0)
            {
                sprintf (messageBuffer, "%f", X1_REF_PIX_ABOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP A EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y1_REF_PIX_ABOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP A EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP A ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP A X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP A Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP A SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X2_REF_PIX_ABOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP B EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y2_REF_PIX_ABOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP B EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP B ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP B X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP B Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX OP B SIZ", messageBuffer, "size", "");
            }
        }
        else if (strcmp (refPix->exposure[i].beamCombiner, "HIGH_SENS") == 0)
        {
            if (strcmp (refPix->exposure[i].shutterId, "AOPEN") == 0)
            {
                sprintf (messageBuffer, "%f", X1_REF_PIX_HS_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A1 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y1_REF_PIX_HS_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A1 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A1 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A1 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A1 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A1 SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X2_REF_PIX_HS_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A2 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y2_REF_PIX_HS_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A2 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A2 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A2 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A2 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS A2 SIZ", messageBuffer, "size", "");
            }
            else if (strcmp (refPix->exposure[i].shutterId, "BOPEN") == 0)
            {
                sprintf (messageBuffer, "%f", X1_REF_PIX_HS_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B1 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y1_REF_PIX_HS_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B1 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B1 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B1 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B1 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B1 SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X2_REF_PIX_HS_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B2 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y2_REF_PIX_HS_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B2 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B2 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B2 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B2 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX HS B2 SIZ", messageBuffer, "size", "");
            }
        }
        else if (strcmp (refPix->exposure[i].beamCombiner, "SCI_PHOT") == 0)
        {
            if (strcmp (refPix->exposure[i].shutterId, "AOPEN") == 0)
            {
                sprintf (messageBuffer, "%f", X1_REF_PIX_SP_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A1 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y1_REF_PIX_SP_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A1 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A1 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A1 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A1 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A1 SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X2_REF_PIX_SP_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A2 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y2_REF_PIX_SP_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A2 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A2 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A2 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A2 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP A2 SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X3_REF_PIX_SP_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PA EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y3_REF_PIX_SP_AOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PA EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PA ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[2].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PA X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[2].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PA Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[2].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PA SIZ", messageBuffer, "size", "");
            }
            else if (strcmp (refPix->exposure[i].shutterId, "BOPEN") == 0)
            {
                sprintf (messageBuffer, "%f", X1_REF_PIX_SP_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B1 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y1_REF_PIX_SP_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B1 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B1 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B1 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B1 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[0].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B1 SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X2_REF_PIX_SP_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B2 EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y2_REF_PIX_SP_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B2 EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B2 ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B2 X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B2 Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[1].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP B2 SIZ", messageBuffer, "size", "");

                sprintf (messageBuffer, "%f", X3_REF_PIX_SP_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PB EX", messageBuffer, "Expected x", "");
                sprintf (messageBuffer, "%f", Y3_REF_PIX_SP_BOPEN);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PB EY", messageBuffer, "Expected y", "");
                sprintf (messageBuffer, "%f", SIZE_SEARCH_REF_PIX);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PB ESI", messageBuffer, "Search size", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[2].xCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PB X", messageBuffer, "x coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[2].yCoord);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PB Y", messageBuffer, "y coordinate", "");
                sprintf (messageBuffer, "%f", refPix->exposure[i].centroid[2].size);
                qfits_header_add (outFitsHeader, "HIERARCH ESO QC PIX SP PB SIZ", messageBuffer, "size", "");
            }
        }
    }

//    qfits_header_consoledump(outFitsHeader);

    //    Place Holders
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH1", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH2", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH3", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH4", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH5", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH6", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH7", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH8", "TBD", "TBD", "");
    qfits_header_add (outFitsHeader, "HIERARCH ESO QC PLACEH9", "TBD", "TBD", "");
    
    //    Create the output fits file. If the file exist delete it
    if (stat (fileNames->outFitsName, &buf) == 0) remove (fileNames->outFitsName);

    outFitsPtr = fopen (fileNames->outFitsName, "w");
    if (!outFitsPtr)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create output FITS file");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        free (sWidthX);
        free (sWidthY);        
        return;
    }

   cpl_msg_info(cpl_func,"Created Product FITS file: %s \n", fileNames->outFitsName);
    fprintf (midiReportPtr, "Created Product FITS file: %s \n", fileNames->outFitsName);

//    qfits_header_consoledump(outFitsHeader);

    //    Write header into the output file
    qfits_header_sort (&outFitsHeader);
    qfits_header_dump (outFitsHeader, outFitsPtr);
    fclose (outFitsPtr);
    qfits_header_destroy (outFitsHeader);

    //    release memory
    free (messageBuffer);
    free (currentTime);
    free (cleanString);
    free (stringTemp);
    free (classification);
    free (tech);
    free (textBuff);
    free (sWidthX);    
    free (sWidthY);

    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    createRefPixQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Copies keywords from the raw input FITS primary header to the 
*               QC log and adds additional product keywords
*
* History:        
* 30-Aug-04        (csabet) Created
******************************************************************************/
void createRefPixQcLog (
    char    *inFitsName,//    In:    Name of the raw FITS file
    int        *error)        //    Ou:    Status
    
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "createRefPixQcLog";
    char            *qfitsString, *emptyString, *cleanString;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset status */
    *error = 0;

    /* Allocate memory */
    emptyString = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    /*    Initialise empty string */
    strcpy (emptyString, "\"UNKNOWN\"");

    /* Get all the keywords */
    qfitsString = qfits_query_hdr (inFitsName, "ARCFILE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "ARCFILE             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "ARCFILE             \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "TELESCOP");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TELESCOP            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TELESCOP            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "INSTRUME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INSTRUME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INSTRUME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "MJD-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "MJD-OBS             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "MJD-OBS             \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "DATE-OBS");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DATE-OBS            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DATE-OBS            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "UTC");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "UTC                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "UTC                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "LST");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "LST                 %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "LST                 %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET CHIP NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.CHIP.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DIT             %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET DITDELAY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.DITDELAY        %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET INT MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.INT.MODE        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.INT.MODE        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NDIT");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NDIT            %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET NRTS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.NRTS.MODE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN1 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN1.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NX");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NX         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DET WIN2 NY");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DET.WIN2.NY         %s \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR CATG");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.CATG            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.CATG            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TECH");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TECH            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TECH            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO DPR TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "DPR.TYPE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "DPR.TYPE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS CAM NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.CAM.NAME        \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS FILT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.FILT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS GRIS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.GRIS.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS MODE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.MODE            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.MODE            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.NAME       \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.ID         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.ID         \"%s\" \n", cleanString);}

    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS OPT1 TYPE");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.OPT1.TYPE       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO INS SLIT NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "INS.SLIT.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.NAME            %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.NAME            \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.ID       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PI-COI NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PI-COI.NAME     \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS PROG ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.PROG.ID         \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.START           \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OBS TARG NAME");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OBS.TARG.NAME       \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO OCS EXPO1 FNAME1");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "OCS.EXPO1.FNAME1    \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL ID");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.ID              %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.ID              \"%s\" \n", cleanString);}
    
    qfitsString = qfits_query_hdr (inFitsName, "HIERARCH ESO TPL START");
    if (qfitsString == NULL) 
        fprintf (midiQcLogPtr, "TPL.START           %s \n", emptyString);
    else {cleanUpString (qfitsString, cleanString); 
        fprintf (midiQcLogPtr, "TPL.START           \"%s\" \n", cleanString);}
    
    /* Release memory */
    free (emptyString);
    free (cleanString);
    
    return;
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software 
*
* Module name:    addProdInfoToRefPixQcLog
* Input/Output:    See function arguments to avoid duplication
* Description:    Adds product information to QC log
*
*
* History:        
* 16-Jan-04        (csabet) Created
******************************************************************************/
void addProdInfoToRefPixQcLog (
    ImageFormat        *format,    // In: Image format
    MidiFiles        *fileNames,
    ReferencePixel     *refPix,    //    In:    Refrenece Pixel data structure
    int                *error)
{

    /*    Local Declarations
    --------------------*/
    const char         routine[] = "addProdInfoToRefPixQcLog";
    int             i;
    FILE            *inFitsBatchPtr;
    char            *stringQfits, *messageBuffer, *currentTime, *textBuff, *cleanString,
                    *stringTemp, *classification;
    time_t          now;
    struct tm        *newTime;
    
    /*    Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    /* Reset parameters */
    *error = 0;

    /*    Allocate memory */
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    textBuff = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    messageBuffer = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    
    /*    Get current time/date */
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Add keyword to QC log file
    fprintf (midiQcLogPtr, "PRO.TYPE            \"%s\" \n", format->obsType);
    fprintf (midiQcLogPtr, "PRO.CATG            \"REDUCED_REFPIX\" \n");
    fprintf (midiQcLogPtr, "PRO.ARCFILE         \"%s\" \n", fileNames->archFileName);
    fprintf (midiQcLogPtr, "PRO.PIPEDATE        \"%s\" \n", currentTime);
    fprintf (midiQcLogPtr, "PRO.VERSION         \"%s\" \n", MIDI_PIPE_VERSION);
    fprintf (midiQcLogPtr, "PRO.PIPEFILE        \"%s\" \n", fileNames->pipeFileName);
    fprintf (midiQcLogPtr, "PRO.DID             \"%s\" \n", MIDI_QC_DIC_VERSION);

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
        free (messageBuffer);
        free (currentTime);
        free (cleanString);
        free (stringTemp);
        free (classification);
        free (textBuff);
        return;
    }

    /* Loop through the list of files and write into the QC log */
    i = 1;
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", messageBuffer, classification);
        
        /* If classification is not given get it from the raw file */
        if (strcmp (classification, "") == 0)
        {
            stringQfits = qfits_query_hdr (messageBuffer, "HIERARCH ESO DPR CATG");
            if (stringQfits == NULL) 
            {
                sprintf (classification, "%s", "UNKNOWN");
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Observation Category");
            }
            else
            {
                cleanUpString (stringQfits, cleanString);
                sprintf (classification, "%s", cleanString);
            }
        }
        removePathName (messageBuffer, midiReportPtr);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.NAME  \"%s\" \n", i, messageBuffer);
        fprintf (midiQcLogPtr, "PRO.REC1.RAW%d.CATG  \"%s\" \n", i++, classification);
    }
    fclose (inFitsBatchPtr);

    for (i = 0; i < refPix->numOfExposures; i++)
    {
        if (strcmp (refPix->exposure[i].beamCombiner, "OPEN") == 0)
        {
            if (strcmp (refPix->exposure[i].shutterId, "ABOPEN") == 0)
            {
                fprintf (midiQcLogPtr, "QC.PIX.OP.A.EX  %f \n", X1_REF_PIX_ABOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.OP.A.EY  %f \n", Y1_REF_PIX_ABOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.OP.A.ESI %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.OP.A.X   %f \n", refPix->exposure[i].centroid[0].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.OP.A.Y   %f \n", refPix->exposure[i].centroid[0].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.OP.A.SIZ %f \n", refPix->exposure[i].centroid[0].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.OP.B.EX  %f \n", X2_REF_PIX_ABOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.OP.B.EY  %f \n", Y2_REF_PIX_ABOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.OP.B.ESI %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.OP.B.X   %f \n", refPix->exposure[i].centroid[1].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.OP.B.Y   %f \n", refPix->exposure[i].centroid[1].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.OP.B.SIZ %f \n", refPix->exposure[i].centroid[1].size);
            }
        }
        else if (strcmp (refPix->exposure[i].beamCombiner, "HIGH_SENS") == 0)
        {
            if (strcmp (refPix->exposure[i].shutterId, "AOPEN") == 0)
            {
                fprintf (midiQcLogPtr, "QC.PIX.HS.A1.EX   %f \n", X1_REF_PIX_HS_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A1.EY   %f \n", Y1_REF_PIX_HS_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A1.ESI  %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.HS.A1.X    %f \n", refPix->exposure[i].centroid[0].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A1.Y    %f \n", refPix->exposure[i].centroid[0].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A1.SIZ  %f \n", refPix->exposure[i].centroid[0].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.HS.A2.EX   %f \n", X2_REF_PIX_HS_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A2.EY   %f \n", Y2_REF_PIX_HS_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A2.ESI  %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.HS.A2.X    %f \n", refPix->exposure[i].centroid[1].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A2.Y    %f \n", refPix->exposure[i].centroid[1].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.A2.SIZ  %f \n", refPix->exposure[i].centroid[1].size);
            }
            else if (strcmp (refPix->exposure[i].shutterId, "BOPEN") == 0)
            {
                fprintf (midiQcLogPtr, "QC.PIX.HS.B1.EX   %f \n", X1_REF_PIX_HS_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B1.EY   %f \n", Y1_REF_PIX_HS_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B1.ESI  %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.HS.B1.X    %f \n", refPix->exposure[i].centroid[0].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B1.Y    %f \n", refPix->exposure[i].centroid[0].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B1.SIZ  %f \n", refPix->exposure[i].centroid[0].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.HS.B2.EX   %f \n", X1_REF_PIX_HS_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B2.EY   %f \n", Y1_REF_PIX_HS_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B2.ESI  %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.HS.B2.X    %f \n", refPix->exposure[i].centroid[1].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B2.Y    %f \n", refPix->exposure[i].centroid[1].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.HS.B2.SIZ  %f \n", refPix->exposure[i].centroid[1].size);
            }
        }
        else if (strcmp (refPix->exposure[i].beamCombiner, "SCI_PHOT") == 0)
        {
            if (strcmp (refPix->exposure[i].shutterId, "AOPEN") == 0)
            {
                fprintf (midiQcLogPtr, "QC.PIX.SP.A1.EX   %f \n", X1_REF_PIX_SP_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A1.EY   %f \n", Y1_REF_PIX_SP_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A1.ESI  %f \n", SIZE_SEARCH_REF_PIX);

                fprintf (midiQcLogPtr, "QC.PIX.SP.A1.X    %f \n", refPix->exposure[i].centroid[0].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A1.Y    %f \n", refPix->exposure[i].centroid[0].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A1.SIZ  %f \n", refPix->exposure[i].centroid[0].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.SP.A2.EX   %f \n", X2_REF_PIX_SP_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A2.EY   %f \n", Y2_REF_PIX_SP_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A2.ESI  %f \n", SIZE_SEARCH_REF_PIX);

                fprintf (midiQcLogPtr, "QC.PIX.SP.A2.X    %f \n", refPix->exposure[i].centroid[1].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A2.Y    %f \n", refPix->exposure[i].centroid[1].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.A2.SIZ  %f \n", refPix->exposure[i].centroid[1].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.SP.PA.EX   %f \n", X3_REF_PIX_SP_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PA.EY   %f \n", Y3_REF_PIX_SP_AOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PA.ESI  %f \n", SIZE_SEARCH_REF_PIX);
                
                fprintf (midiQcLogPtr, "QC.PIX.SP.PA.X    %f \n", refPix->exposure[i].centroid[2].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PA.Y    %f \n", refPix->exposure[i].centroid[2].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PA.SIZ  %f \n", refPix->exposure[i].centroid[2].size);
            }
            else if (strcmp (refPix->exposure[i].shutterId, "BOPEN") == 0)
            {
                fprintf (midiQcLogPtr, "QC.PIX.SP.B1.EX    %f \n", X1_REF_PIX_SP_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B1.EY    %f \n", Y1_REF_PIX_SP_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B1.ESI   %f \n", SIZE_SEARCH_REF_PIX);

                fprintf (midiQcLogPtr, "QC.PIX.SP.B1.X     %f \n", refPix->exposure[i].centroid[0].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B1.Y     %f \n", refPix->exposure[i].centroid[0].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B1.SIZ   %f \n", refPix->exposure[i].centroid[0].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.SP.B2.EX    %f \n", X2_REF_PIX_SP_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B2.EY    %f \n", Y2_REF_PIX_SP_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B2.ESI   %f \n", SIZE_SEARCH_REF_PIX);

                fprintf (midiQcLogPtr, "QC.PIX.SP.B2.X     %f \n", refPix->exposure[i].centroid[1].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B2.Y     %f \n", refPix->exposure[i].centroid[1].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.B2.SIZ   %f \n", refPix->exposure[i].centroid[1].size);
                
                fprintf (midiQcLogPtr, "QC.PIX.SP.PB.EX    %f \n", X3_REF_PIX_SP_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PB.EY    %f \n", Y3_REF_PIX_SP_BOPEN);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PB.ESI   %f \n", SIZE_SEARCH_REF_PIX);

                fprintf (midiQcLogPtr, "QC.PIX.SP.PB.X     %f \n", refPix->exposure[i].centroid[2].xCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PB.Y     %f \n", refPix->exposure[i].centroid[2].yCoord);
                fprintf (midiQcLogPtr, "QC.PIX.SP.PB.SIZ   %f \n", refPix->exposure[i].centroid[2].size);
            }
        }
    }
    
    /*    release memory */
    free (messageBuffer);
    free (currentTime);
    free (textBuff);
    free (cleanString);
    free (stringTemp);
    free (classification);
        
    return;
}
/*****************************************************************************/


