/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*               VLTI Data Reduction Software
*
* File name:    diagnostics.c
* Description:  Contains the routines for diagnostic purposes
*
*
* History:
* 08-Jan-04     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>
#include <cpl.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "errorHandling.h"
#include "diagnostics.h"
#include "qfits.h"

/******************************************************************************
*   Global Variables
******************************************************************************/

/******************************************************************************
*   Prototypes
******************************************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreatePlotFileDouble3D
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a 3D plot file
*
* History:
* 08-Apr-05     (csabet)
******************************************************************************/
void midiCreatePlotFileDouble3D (
    const char    *plotFileName,
    const char    *title,
    const char    *xLabel,
    const char    *yLabel,
    const char    *zLabel,
    int        deletePlot,
    double    *image,
    int        sizeX,
    int        sizeY,
    const char    *style,
    const char    *lineType)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreatePlotFileDouble3D";
    int         i, j, frame, zero = 0;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");

    for (i = 0; i < sizeY; i++)
    {
        frame = i * sizeX;
        for (j = 0; j < sizeX; j++)
        {
            if (isnan (image[frame + j])) fprintf (dataFilePtr, "%d \n", zero);
            else fprintf (dataFilePtr, "%8.4f ", image[frame + j]);
        }

        fprintf (dataFilePtr, "\n");
    }
    fclose (dataFilePtr);

    //    Create the command file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.log");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "set zlabel \'%s\' \n", zLabel);
        fprintf (cmdFilePtr, "set xrange [0:%d] \n", sizeX);
        fprintf (cmdFilePtr, "set yrange [0:%d] \n", sizeY);
        fprintf (cmdFilePtr, "set ticslevel 0 \n");
        fprintf (cmdFilePtr, "splot \'%s\' matrix with %s %s \n", dataFilePath, style, lineType);
            
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);

        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 900x900+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreatePlotFile3D
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a 3D plot file
*
* History:
* 08-Apr-05     (csabet)
******************************************************************************/
void midiCreatePlotFile3D (
    const char    *plotFileName,
    const char    *title,
    const char    *xLabel,
    const char    *yLabel,
    const char    *zLabel,
    int        deletePlot,
    float    *image,
    int        sizeX,
    int        sizeY,
    const char    *style,
    const char    *lineType)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreatePlotFile3D";
    int         i, j, frame, zero = 0;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");

    for (i = 0; i < sizeY; i++)
    {
        frame = i * sizeX;
        for (j = 0; j < sizeX; j++)
        {
            if (isnan (image[frame + j])) fprintf (dataFilePtr, "%d \n", zero);
            else fprintf (dataFilePtr, "%8.4f ", image[frame + j]);
        }

        fprintf (dataFilePtr, "\n");
    }
    fclose (dataFilePtr);

    //    Create the command file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.log");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "set zlabel \'%s\' \n", zLabel);
        fprintf (cmdFilePtr, "set xrange [0:%d] \n", sizeX);
        fprintf (cmdFilePtr, "set yrange [0:%d] \n", sizeY);
        fprintf (cmdFilePtr, "set ticslevel 0 \n");
        fprintf (cmdFilePtr, "splot \'%s\' matrix with %s %s \n", dataFilePath, style, lineType);
            
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);

        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 900x900+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreatePlotFileDouble2D
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a plot file
*
* History:
* 14-Feb-05     (csabet)
******************************************************************************/
void midiCreatePlotFileDouble2D (
    const char    *plotFileName,
    const char    *title,
    const char    *xLabel,
    const char    *yLabel,
    int        deletePlot,
    double    *array,
    int        begin,
    int        end,
    int        interval)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreatePlotFileDouble2D";
    int         i, zero = 0;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");
    for (i = begin; i < end; i += interval)
    {
        if (isnan (array[i])) fprintf (dataFilePtr, "%d \n", zero);
        else fprintf (dataFilePtr, "%f \n", array[i]);
    }
    fclose (dataFilePtr);

    //    Display plot file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.grb");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "plot \'%s\' with lines 1\n", dataFilePath);
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);


        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 600x600+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
             cpl_msg_warning(cpl_func,"Failed to call the system command");
         }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreateReplotFileDouble2D2P
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a plot file for a 2 column table
*
* History:
* 14-Feb-05     (csabet)
******************************************************************************/
void midiCreateReplotFileDouble2D2P (
    char    *plotFileName,
    char    *title,
    char    *xLabel,
    char    *yLabel,
    int        deletePlot,
    double    *arrayX,    // In: Array X
    double    *arrayY,
    int        begin,        // In: Start of the arrayX
    int        end,        // In: End of arrayX
    int        interval,    // In: Interval for the arrayX
    char    *lineType)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreateReplotFileDouble2D2P";
    int         i, zero = 0;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");
    for (i = begin; i < end; i += interval)
    {
        if (isnan (arrayX[i]) || isnan (arrayY[i])) fprintf (dataFilePtr, "%d %d \n", zero, zero);
        else fprintf (dataFilePtr, "%f %f \n", arrayX[i], arrayY[i]);
    }
    fclose (dataFilePtr);

    //    Display plot file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.grb");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "replot \'%s\' with %s\n", dataFilePath, lineType);
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);


        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 600x600+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreatePlotFileDouble2D2P
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a plot file for a 2 column table
*
* History:
* 14-Feb-05     (csabet)
******************************************************************************/
void midiCreatePlotFileDouble2D2P (
    const char    *plotFileName,
    const char    *title,
    const char    *xLabel,
    const char    *yLabel,
    int        deletePlot,
    double    *arrayX,    // In: Array X
    double    *arrayY,
    int        begin,        // In: Start of the arrayX
    int        end,        // In: End of arrayX
    int        interval,    // In: Interval for the arrayX
    const char    *lineType)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreatePlotFileDouble2D2P";
    int         i, zero = 0;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");
    for (i = begin; i < end; i += interval)
    {
        if (isnan (arrayX[i]) || isnan (arrayY[i])) fprintf (dataFilePtr, "%d %d \n", zero, zero);
        else fprintf (dataFilePtr, "%f %f \n", arrayX[i], arrayY[i]);
    }
    fclose (dataFilePtr);

    //    Display plot file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.grb");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "plot \'%s\' with %s\n", dataFilePath, lineType);
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);


        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 600x600+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreatePlotFile2D2P
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a plot file for a 2 column table
*
* History:
* 14-Feb-05     (csabet)
******************************************************************************/
void midiCreatePlotFile2D2P (
    const char    *plotFileName,
    const char    *title,
    const char    *xLabel,
    const char    *yLabel,
    int        deletePlot,
    float    *arrayX,    // In: Array X
    float    *arrayY,
    int        begin,        // In: Start of the arrayX
    int        end,        // In: End of arrayX
    int        interval)    // In: Interval for the arrayX
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreatePlotFile2D2P";
    int         i, zero = 0;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");
    for (i = begin; i < end; i += interval)
    {
        if (isnan (arrayX[i]) || isnan (arrayY[i])) fprintf (dataFilePtr, "%d %d \n", zero, zero);
        else fprintf (dataFilePtr, "%f %f \n", arrayX[i], arrayY[i]);
    }
    fclose (dataFilePtr);

    //    Display plot file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.grb");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "plot \'%s\' with lines 1\n", dataFilePath);
        if (plotDuration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", plotDuration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", plotDuration);
        
        fclose (cmdFilePtr);


        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 600x600+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiCreatePlotFile2D
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a plot file
*
* History:
* 14-Feb-05     (csabet)
******************************************************************************/
void midiCreatePlotFile2D (
    const char    *plotFileName,
    const char    *title,
    const char    *xLabel,
    const char    *yLabel,
    int        deletePlot,
    float    *array,
    int        begin,
    int        end,
    int        interval,
    int        override)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "midiCreatePlotFile2D";
    int         i, zero = 0, duration;
    FILE        *dataFilePtr=NULL, *cmdFilePtr=NULL;
    char        *dataFilePath, *cmdFileName, *argument;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cmdFileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    argument = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Reset
    if (override) 
        duration = override;
    else 
        duration = plotDuration;
    
    //    Create the plot file
    sprintf (dataFilePath, "%s%s.%s.plt", outFileDir, outRootName, plotFileName);
    dataFilePtr = fopen (dataFilePath, "w");
    for (i = begin; i < end; i += interval)
    {
        if (isnan (array[i])) fprintf (dataFilePtr, "%d \n", zero);
        else fprintf (dataFilePtr, "%f \n", array[i]);
    }
    fclose (dataFilePtr);

    //    Display plot file
    if (plotFile > 1)
    {
        sprintf (cmdFileName, "plotCmdFile.grb");
        cmdFilePtr = fopen (cmdFileName, "w");
        fprintf (cmdFilePtr, "set title \'%s\' \n", title);
        fprintf (cmdFilePtr, "set xlabel \'%s\' \n", xLabel);
        fprintf (cmdFilePtr, "set ylabel \'%s\' \n", yLabel);
        fprintf (cmdFilePtr, "plot \'%s\' with lines 1\n", dataFilePath);
        if (duration == -1)
        {
           cpl_msg_info(cpl_func,"\n");
            fprintf (cmdFilePtr, "pause %d \"          ***** Hit Return to continue *****\" \n", duration);
        }
        else
            fprintf (cmdFilePtr, "pause %d \n", duration);
        
        fclose (cmdFilePtr);


        //    Run the command file
        sprintf (argument, "gnuplot -title \"MIDI DRS\" -geometry 600x600+0+0 %s", cmdFileName);
        int ret = system (argument);
        if (ret != 0) {
            cpl_msg_warning(cpl_func,"Failed to call the system command");
        }

        //    Remove temporary files
        remove (cmdFileName);
    }

    //    Delete plot file if display only
    if (deletePlot || plotFile == 3) remove (dataFilePath);

    //    Release memory
    free (dataFilePath);
    free (cmdFileName);
    free (argument);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  getVersions
* Input/Output: See function arguments to avoid duplication
* Description:  Displays versions of integrated software
*
* History:
* 14-Feb-05     (csabet)
******************************************************************************/
void getVersions (void)
{

    //    Local Declarations
    //    ------------------

    //    Algorithm
    //    ---------
   cpl_msg_info(cpl_func,"\nIntegrated Versions \n");
   cpl_msg_info(cpl_func,"------------------- \n");
   cpl_msg_info(cpl_func,"MIDI DRS Pipeline   = %s \n", MIDI_PIPE_VERSION);
   cpl_msg_info(cpl_func,"Iau Exchange        = %s \n", IAUEXCHANGE_VERSION);
   cpl_msg_info(cpl_func,"QC Dictionary       = %s \n", MIDI_QC_DIC_VERSION);
   cpl_msg_info(cpl_func,"\n");

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareWaterfallDisplay
* Input/Output: See function arguments to avoid duplication
* Description:  This routine creates the waterfall display.
*
* History:
* 14-Aug-03     (csabet) Created. Derived from pballest July 2003
******************************************************************************/
void prepareWaterfallDisplay (
    MidiFiles        *fileNames,        // In: Pointer to the file structure
    ImageFormat        *imageSize,        // In: Image format
    CompressedData    *compressed)    // In: Compressed data
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "prepareWaterfallDisplay";
    qfits_header    *pHeaderOut;
    FILE            *waterfallPtr;
    qfitsdumper     qdFringe;
    char            *sFramesPerScan, *sNumOfScans, *classification, *stringTemp, 
            *currentTime, *sNumOfChannels, *sNumOfFreqBins;
    FILE            *inFitsBatchPtr = NULL;
    struct stat     buf;
    char            *fileName, *title;
    int                f, fs, s, numOfScans, framesPerScan, numOfFrames, scanMax, frameStart, 
                    numOfFreqBins, frameEnd;
    double            *cleanArray, dcLevel, accum, fluxMax;
    time_t          now;
    struct tm       *newTime;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Waterfall Display Data for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"----------------------------------------- \n");
    fprintf (midiReportPtr, "\nCreating Waterfall Display Data for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "----------------------------------------- \n");

    //    Allocate memory
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sNumOfScans = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sFramesPerScan = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sNumOfFreqBins = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sNumOfChannels = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    cleanArray = (double *) calloc (imageSize->numOfFrames, sizeof (double));
    currentTime = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    //    Get current time/date
    now = time(NULL);
    newTime = gmtime (&now);
    strftime (currentTime, MIN_STRING_LENGTH, "%a %d %b %Y at %H:%M:%S", newTime);

    //    Create a clean iFringe
    numOfFrames = 0;
    for (f = 0; f < imageSize->numOfFrames; f++)
    {
        if (compressed->tarType[f] == 'T')
        {
            cleanArray[numOfFrames] = compressed->iFringe[f];
            numOfFrames++;
        }
    }

    if (strcmp (imageSize->beamCombiner, "SCI_PHOT") == 0)
    {
        numOfScans = countTransitions ('T', imageSize->numOfFrames, compressed->tarType) + 1;
        framesPerScan = numOfFrames / numOfScans;
    }
    else
    {
        numOfScans = imageSize->numOfScans;
        framesPerScan = imageSize->framesPerScan;
    }

    //    Remove DC level
    for (s = 0; s < numOfScans; s++)
    {
        accum = 0;
        for (fs = 0; fs < framesPerScan; fs++)
        {
            f = fs + s * framesPerScan;
            accum += cleanArray[f];
        }
        dcLevel = accum/framesPerScan;
        for (fs = 0; fs < framesPerScan; fs++)
        {
            f = fs + s * framesPerScan;
            cleanArray[f] -= dcLevel;
        }
    }

    //    Find the scan at which the fringe is maximum
    fluxMax = 0.0;
    scanMax = 0;
    for (f = 0; f < numOfFrames; f++)
    {
        if (fabs(cleanArray[f]) > fluxMax)
        {
            fluxMax = fabs(cleanArray[f]);
            scanMax = f;
        }
    }
    scanMax = scanMax / framesPerScan + 1;
    
    //    Write all NAXIS values as character strings
    numOfFreqBins = POST_FFT_SIZE/2;
    sprintf (sNumOfScans, "%d", numOfScans);
    sprintf (sFramesPerScan, "%d", framesPerScan);
    sprintf (sNumOfFreqBins, "%d", numOfFreqBins);
    sprintf (sNumOfChannels, "%d", imageSize->iXWidth);

    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
       cpl_msg_info(cpl_func,"\nCannot Create Waterfall Display Data for batch  %d \n", batchNumber);
        fprintf (midiReportPtr, "\nCannot Create Waterfall Display Data for batch  %d \n", batchNumber);
        free (sNumOfScans);
        free (sFramesPerScan);
        free (classification);
        free (stringTemp);
        free (cleanArray);
        free (sNumOfFreqBins);
        free (sNumOfChannels);
        return;
    }
        

    //    Read the name of the first file in the list and get it's full path name
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    //    Add primary Header
    if (diagnostic)cpl_msg_info(cpl_func,"Extracting primary header from input FITS file   %s \n", fileNames->inFitsName);
    if (diagnostic) fprintf(midiReportPtr, "Extracting primary header from input FITS file   %s \n", fileNames->inFitsName);
    pHeaderOut = qfits_header_read (fileNames->inFitsName);
    if (pHeaderOut == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot extract primary header from input FITS file");
       cpl_msg_info(cpl_func,"\nCannot Create Waterfall Display Data for batch  %d \n", batchNumber);
        fprintf (midiReportPtr, "\nCannot Create Waterfall Display Data for batch  %d \n", batchNumber);
        free (sNumOfScans);
        free (sFramesPerScan);
        free (classification);
        free (stringTemp);
        free (cleanArray);
        free (sNumOfFreqBins);
        free (sNumOfChannels);
        return;
    }

    //    Updates header for the Waterfall output file
    qfits_header_mod (pHeaderOut, "BITPIX","-64", "number of bits per pixel");
    qfits_header_mod (pHeaderOut, "NAXIS", "2", "number of data axes");
    qfits_header_add (pHeaderOut, "NAXIS1", sFramesPerScan, "", "");
    qfits_header_add (pHeaderOut, "NAXIS2", sNumOfScans, "", "");
    qfits_header_add (pHeaderOut, "FREQBIN", sNumOfFreqBins, "", "");
    qfits_header_add (pHeaderOut, "NUMCHAN", sNumOfChannels, "", "");
    qfits_header_mod (pHeaderOut, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );
    qfits_header_add (pHeaderOut, "HIERARCH ESO PRO CATG", "WATERFALL", "Pipeline product category", NULL);
    qfits_header_add (pHeaderOut, "HIERARCH ESO PRO ARCFILE", fileNames->archFileName, "Arcfile name of first raw file", "");
    qfits_header_add (pHeaderOut, "HIERARCH ESO PRO PIPEDATE", currentTime, "Pipeline run date", "");
    qfits_header_add (pHeaderOut, "HIERARCH ESO PRO VERSION", MIDI_PIPE_VERSION, "Pipeline version", "");
    qfits_header_add (pHeaderOut, "PIPEFILE", fileNames->pipeFileName, "Pipeline product file name", "");

    //    Create file and dump FITS Header for Waterfall output file. If the file exist delete it
    if (stat (fileNames->outWaterfallName, &buf) == 0) remove (fileNames->outWaterfallName);
    if ((waterfallPtr = fopen (fileNames->outWaterfallName, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Waterfall FITS file");
        free (sNumOfScans);
        free (sFramesPerScan);
        free (classification);
        free (stringTemp);
        qfits_header_destroy (pHeaderOut);
        free (cleanArray);
        free (sNumOfFreqBins);
        free (sNumOfChannels);
        return;
    }

    if (diagnostic)cpl_msg_info(cpl_func,"Created Waterfall FITS file: %s \n", fileNames->outWaterfallName);
    fprintf (midiReportPtr, "Created Waterfall FITS file: %s \n", fileNames->outWaterfallName);

    qfits_header_dump (pHeaderOut, waterfallPtr);
    fclose (waterfallPtr);

    //    Done writing header, close structure
    qfits_header_destroy (pHeaderOut);

    //    Dumps Fringe into the Waterfall file
    qdFringe.filename  = fileNames->outWaterfallName;
    qdFringe.npix      = framesPerScan * numOfScans;
    qdFringe.ptype     = PTYPE_DOUBLE;
    qdFringe.dbuf      = cleanArray;
    qdFringe.out_ptype = BPP_IEEE_DOUBLE;

    qfits_pixdump (&qdFringe);
    qfits_zeropad (fileNames->outWaterfallName);

    //    Close files and release memory
    fclose (inFitsBatchPtr);
    free (sNumOfScans);
    free (sFramesPerScan);
    free (sNumOfFreqBins);
    free (sNumOfChannels);
    free (classification);
    free (stringTemp);
    free (currentTime);

    //    Create plot file
    if (plotFile)
    {
        //    Allocate memory
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        //    Plot a few fringes around the maximum
        sprintf (fileName, "FringeAtScan%d", scanMax-1);
        sprintf (title, "Fringe at scan %d", scanMax-1);

        if(scanMax<2){
           frameStart = 0;
           frameEnd = framesPerScan;
        }
        else {
           frameStart = (scanMax - 2) * framesPerScan;
           frameEnd = (scanMax - 1) * framesPerScan;
        }
        midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux (dc removed)", 0, cleanArray, frameStart, frameEnd, 1);
 
        sprintf (fileName, "FringeStrongest");
        sprintf (title, "Strongest fringe found at scan %d", scanMax);
        frameStart = (scanMax - 1) * framesPerScan;
        frameEnd = scanMax * framesPerScan;
        midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux (dc removed)", 0, cleanArray, frameStart, frameEnd, 1);
 
        sprintf (fileName, "FringeAtScan%d", scanMax+1);
        sprintf (title, "Fringe at scan %d", scanMax+1);
        frameStart = scanMax * framesPerScan;
        frameEnd = (scanMax + 1) * framesPerScan;
        midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux (dc removed)", 0, cleanArray, frameStart, frameEnd, 1);
 
        free (fileName);
        free (title);
    }
    free (cleanArray);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  prepareWaterpowerDisplay
* Input/Output: See function arguments to avoid duplication
* Description:  This routine creates the waterpower display
*
* History:
* 21-Jul-03     (csabet) Created
******************************************************************************/
void prepareWaterpowerDisplay (
    MidiFiles    *fileNames,        // In: Pointer to file structure including target file name
    ImageFormat    *imageSize,        // In: Pointer to image structure
    float        *allSpectrum)    // In: Compressed spectrum for all regions
{
    /*  Local Declarations
    ---------------------- */
    const char      routine[] = "prepareWaterpowerDisplay";
    qfits_header    *pHeaderOut;
    FILE            *waterpowerPtr;
    qfitsdumper     qdEachSpectrum;
    char            *sFramesPerScan, *sNumOfScans, *classification, *stringTemp;
    FILE            *inFitsBatchPtr = NULL;
    struct stat     buf;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nCreating Waterpower Display Data for batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------------------ \n");
    fprintf (midiReportPtr, "\nCreating Waterpower Display Data for batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------------------ \n");

    /*  Allocate memory */
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sNumOfScans = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));
    sFramesPerScan = (char *) calloc (MIN_STRING_LENGTH, sizeof (char));

    /*  Write all NAXIS values as character strings */
    sprintf (sNumOfScans, "%d", imageSize->numOfScans); // imageSize->numOfScans);
    sprintf (sFramesPerScan, "%d", imageSize->fftsize/2);

    /*  Open the list of files */
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot open input FITS file list");
       cpl_msg_info(cpl_func,"\nCannot Create Waterpower Display Data for batch  %d \n", batchNumber);
        fprintf (midiReportPtr, "\nCannot Create Waterpower Display Data for batch  %d \n", batchNumber);
        free (sNumOfScans);
        free (sFramesPerScan);
        free (classification);
        free (stringTemp);
        return;
    }
    /*  Read the name of the first file in the list and get it's full path name */
    fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr);
    sprintf (classification, "%s", "");
    sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);

    /*  Add primary Header */
    if (diagnostic)cpl_msg_info(cpl_func,"Extracting primary header from input FITS file   %s \n", fileNames->inFitsName);
    if (diagnostic) fprintf(midiReportPtr, "Extracting primary header from input FITS file   %s \n", fileNames->inFitsName);
    pHeaderOut = qfits_header_read (fileNames->inFitsName);
    if (pHeaderOut == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot extract header from input FITS file");
       cpl_msg_info(cpl_func,"\nCannot Create Waterpower Display Data for batch  %d \n", batchNumber);
        fprintf (midiReportPtr, "\nCannot Create Waterpower Display Data for batch  %d \n", batchNumber);
        free (sNumOfScans);
        free (sFramesPerScan);
        free (classification);
        free (stringTemp);
        return;
    }

    /*  Updates header for the Waterpower output file */
    qfits_header_mod (pHeaderOut, "BITPIX","-64", "number of bits per pixel");
    qfits_header_mod (pHeaderOut, "NAXIS", "2", "number of data axes");
    qfits_header_add (pHeaderOut, "NAXIS1", sFramesPerScan, "", "");
    qfits_header_add (pHeaderOut, "NAXIS2", sNumOfScans, "", "");
    qfits_header_mod (pHeaderOut, "INSTRUME", "MIDI", "MIDI Raw Data Display FITS created by DRS pipeline" );

    //    Create file and dump FITS Header for Waterpower output file. If the file exist delete it
    if (stat (fileNames->outWaterpowerName, &buf) == 0) remove (fileNames->outWaterpowerName);
    if ((waterpowerPtr = fopen (fileNames->outWaterpowerName, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create Waterpower FITS file");
        free (sNumOfScans);
        free (sFramesPerScan);
        free (classification);
        free (stringTemp);
        qfits_header_destroy (pHeaderOut);
        return;
    }

    if (diagnostic)cpl_msg_info(cpl_func,"Created Waterpower FITS file: %s \n", fileNames->outWaterpowerName);
    fprintf (midiReportPtr, "Created Waterpower FITS file: %s \n", fileNames->outWaterpowerName);

    qfits_header_dump (pHeaderOut, waterpowerPtr);
    fclose (waterpowerPtr);

    /*  Done writing header, close structure */
    qfits_header_destroy (pHeaderOut);

    /*  Dumps allSpectrum into the Waterpower file */
    qdEachSpectrum.filename  = fileNames->outWaterpowerName;
    qdEachSpectrum.npix      = imageSize->allSpectrumLength;
    qdEachSpectrum.ptype     = PTYPE_FLOAT;
    qdEachSpectrum.fbuf      = allSpectrum;
    qdEachSpectrum.out_ptype = BPP_IEEE_FLOAT;

    qfits_pixdump (&qdEachSpectrum);
    qfits_zeropad (fileNames->outWaterpowerName);

    /*  Close files */
    fclose (inFitsBatchPtr);

    /*  Release memory */
    free (sNumOfScans);
    free (sFramesPerScan);
    free (classification);
    free (stringTemp);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  reportInterfChopping
* Input/Output: See function arguments to avoid duplication
* Description:  Checks chopping cycles
*
* History:
* 06-Dec-05     (csabet)
******************************************************************************/
void reportInterfChopping (
    const char            *dataKey,        // In: Whether Interferometry or Photometry etc.
    ImageFormat        *format,        // In: Image format
    CompressedData    *compressed)    // In: Compressed data
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "reportInterfChopping";
    int            f, s, u, n, fps;
    char        *fileName, *title, *dataFilePath;
    FILE        *dataFilePtr=NULL;
    double        *array;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset
    fps = format->framesPerScan;

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create a QC log file for Interferometry
    sprintf (dataFilePath, "%s%s.Compressed%s.qc.dummy", outFileDir, outRootName, dataKey);
    dataFilePtr = fopen (dataFilePath, "w");

    if ((strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0) ||
        (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0))
    {
        fprintf (dataFilePtr, "Frame  TarType  DATA1              DATA2              DATA1-DATA2 \n");
        fprintf (dataFilePtr, "================================================================= \n");
    }
    else
    {
        fprintf (dataFilePtr, "Frame  TarType  DATA2              DATA3              DATA2-DATA3 \n");
        fprintf (dataFilePtr, "================================================================= \n");
    }
    for (f = 0; f < format->numOfFrames; f++)
    {
        fprintf (dataFilePtr, "%5d  %c        %f    %f    %f\n", f, compressed->tarType[f], 
            compressed->iFringe1[f], compressed->iFringe2[f], compressed->iFringe[f]);
    }
    fclose (dataFilePtr);

    //    Create statistics and plot    
    s = u = compressed->numOfTargetChops = 0;
    for (f = 0; f < format->numOfFrames; f++)
    {
        if (compressed->tarType[f] == 'S') s++;
        else if (compressed->tarType[f] == 'T') (compressed->numOfTargetChops)++;
        else u++;
    }

   cpl_msg_info(cpl_func,"\n%s Chopping Statistics QCLOG \n", dataKey);
   cpl_msg_info(cpl_func,"---------------------------------    QCLOG \n");
   cpl_msg_info(cpl_func,"Number of Target chops    = %d       QCLOG \n", compressed->numOfTargetChops);
   cpl_msg_info(cpl_func,"Number of Sky chops       = %d       QCLOG \n", s);
   cpl_msg_info(cpl_func,"Number of Undefined chops = %d       QCLOG \n", u);
    fprintf (midiReportPtr, "\n%s Chopping Statistics QCLOG \n", dataKey);
    fprintf (midiReportPtr, "---------------------------------    QCLOG \n");
    fprintf (midiReportPtr, "Number of Target chops    = %d       QCLOG \n", compressed->numOfTargetChops);
    fprintf (midiReportPtr, "Number of Sky chops       = %d       QCLOG \n", s);
    fprintf (midiReportPtr, "Number of Undefined chops = %d       QCLOG \n", u);
    
    if (plotFile && diagnostic > 1)
    {
        array = (double *) calloc (format->numOfFrames, sizeof (double));
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        if ((strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0) ||
            (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0))
        {
            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                {
                    array[n] = compressed->iFringe1[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA1", dataKey);
            sprintf (title, "Compressed %s DATA1", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);

            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe2[f]))))
                {
                    array[n] = compressed->iFringe2[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA2", dataKey);
            sprintf (title, "Compressed %s DATA2", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);

            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe[f]))))
                {
                    array[n] = compressed->iFringe[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA1_DATA2", dataKey);
            sprintf (title, "Compressed %s DATA1-DATA2", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
        }
        else
        {
            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                {
                    array[n] = compressed->iFringe1[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA2", dataKey);
            sprintf (title, "Compressed %s DATA2", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);

            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe2[f]))))
                {
                    array[n] = compressed->iFringe2[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA3", dataKey);
            sprintf (title, "Compressed %s DATA3", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);

            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe[f]))))
                {
                    array[n] = compressed->iFringe[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA2_DATA3", dataKey);
            sprintf (title, "Compressed %s DATA2-DATA3", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
        }
        free (fileName);
        free (title);
        free (array);
    }

    //    Release memory
    free (dataFilePath);

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  displayInterfChoppingDisp
* Input/Output: See function arguments to avoid duplication
* Description:  Checks chopping cycles
*
* History:
* 06-Dec-05     (csabet)
******************************************************************************/
void displayInterfChoppingDisp (
    const char            *dataKey,        // In: Whether Interferometry or Photometry etc.
    ImageFormat        *format,        // In: Image format
    CompressedData    *compressed)    // In: Compressed data
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "displayInterfChoppingDisp";
    int            f, n, X, R;
    char        *fileName, *title;
    float        *arrayF;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Display for each channel
    if (plotFile && diagnostic > 4)
    {
        arrayF = (float *) calloc (format->numOfFrames, sizeof (float));
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        for (R = 0; R < format->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < format->iXWidth; X++)
            {
                if (badChannelList[X])
                    continue;
                    
                n = 0;
                for (f = 0; f < format->numOfFrames; f++)
                {
                    if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iDispFringe[R][X][f]))))
                    {
                        arrayF[n] = compressed->iDispFringe[R][X][f];
                        n++;
                    }
                }
                if ((strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0) ||
                    (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0))
                {
                    sprintf (fileName, "%sCompressedDATA%d_X%d", dataKey, R+1, X+1);
                    sprintf (title, "Compressed %s DATA%d for Channel %d", dataKey, R+1, X+1);
                }
                else
                {
                    sprintf (fileName, "%sCompressedDATA%d_X%d", dataKey, R+2, X+1);
                    sprintf (title, "Compressed %s DATA%d for Channel %d", dataKey, R+2, X+1);
                }
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 0, arrayF, 0, n, 1, 0);
            }
        }
        free (fileName);
        free (title);
        free (arrayF);
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  reportPhotomChopping
* Input/Output: See function arguments to avoid duplication
* Description:  Determines chopping coherency
*
* History:
* 06-Dec-05     (csabet)
******************************************************************************/
void reportPhotomChopping (
    const char            *dataKey,        // In: Whether Interferometry or Photometry etc.
    ImageFormat        *format,        // In: Image format
    CompressedData    *compressed)    // In: Compressed data
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "reportPhotomChopping";
    int            f, s, t, u, n, fps;
    char        *fileName, *title, *dataFilePath;
    FILE        *dataFilePtr=NULL;
    double        *array;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset
    fps = format->framesPerScan;

    //    Allocate memory
    dataFilePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Create a QC log file for Interferometry
    sprintf (dataFilePath, "%s%s.Compressed%s.qc.dummy", outFileDir, outRootName, dataKey);
    dataFilePtr = fopen (dataFilePath, "w");

    if ((strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0) ||
        (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0))
    {
        fprintf (dataFilePtr, "Frame  TarType  DATA1              DATA2 \n");
        fprintf (dataFilePtr, "======================================== \n");
        for (f = 0; f < format->numOfFrames; f++)
        {
            fprintf (dataFilePtr, "%5d  %c        %f    %f \n", f, compressed->tarType[f], 
                compressed->iFringe1[f], compressed->iFringe2[f]);
        }
    }
    else
    {
        if (strcmp (format->shutterId, "AOPEN") == 0)
        {
            fprintf (dataFilePtr, "Frame  TarType  DATA1 \n");
            fprintf (dataFilePtr, "===================== \n");
            for (f = 0; f < format->numOfFrames; f++)
            {
                fprintf (dataFilePtr, "%5d  %c        %f \n", f, compressed->tarType[f], 
                    compressed->iFringe1[f]);
            }
        }
        else if (strcmp (format->shutterId, "BOPEN") == 0)
        {
            fprintf (dataFilePtr, "Frame  TarType  DATA4 \n");
            fprintf (dataFilePtr, "===================== \n");
            for (f = 0; f < format->numOfFrames; f++)
            {
                fprintf (dataFilePtr, "%5d  %c        %f \n", f, compressed->tarType[f], 
                    compressed->iFringe1[f]);
            }
        }
        else if (strcmp (format->shutterId, "ABOPEN") == 0)
        {
            fprintf (dataFilePtr, "Frame  TarType  DATA1 or DATA4 \n");
            fprintf (dataFilePtr, "============================== \n");
            for (f = 0; f < format->numOfFrames; f++)
            {
                fprintf (dataFilePtr, "%5d  %c        %f \n", f, compressed->tarType[f], 
                    compressed->iFringe1[f]);
            }
        }
    }
    fclose (dataFilePtr);

    //    Create statistics and plot    
    s = t = u = 0;
    for (f = 0; f < format->numOfFrames; f++)
    {
        if (compressed->tarType[f] == 'S') s++;
        else if (compressed->tarType[f] == 'T') t++;
        else u++;
    }

   cpl_msg_info(cpl_func,"\n%s Chopping Statistics QCLOG \n", dataKey);
   cpl_msg_info(cpl_func,"---------------------------------    QCLOG \n");
   cpl_msg_info(cpl_func,"Number of Target chops    = %d       QCLOG \n", t);
   cpl_msg_info(cpl_func,"Number of Sky chops       = %d       QCLOG \n", s);
   cpl_msg_info(cpl_func,"Number of Undefined chops = %d       QCLOG \n", u);
    fprintf (midiReportPtr, "\n%s Chopping Statistics QCLOG \n", dataKey);
    fprintf (midiReportPtr, "---------------------------------    QCLOG \n");
    fprintf (midiReportPtr, "Number of Target chops    = %d       QCLOG \n", t);
    fprintf (midiReportPtr, "Number of Sky chops       = %d       QCLOG \n", s);
    fprintf (midiReportPtr, "Number of Undefined chops = %d       QCLOG \n", u);
    
    if (plotFile && diagnostic > 1)
    {
        array = (double *) calloc (format->numOfFrames, sizeof (double));
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        if ((strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0) ||
            (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0))
        {
            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                {
                    array[n] = compressed->iFringe1[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA1", dataKey);
            sprintf (title, "Compressed %s DATA1", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);

            n = 0;
            for (f = 0; f < format->numOfFrames; f++)
            {
                if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe2[f]))))
                {
                    array[n] = compressed->iFringe2[f];
                    n++;
                }
            }
            sprintf (fileName, "%sCompressedDATA2", dataKey);
            sprintf (title, "Compressed %s DATA2", dataKey);
            midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
        }
        else
        {
            if (strcmp (format->shutterId, "AOPEN") == 0)
            {
                n = 0;
                for (f = 0; f < format->numOfFrames; f++)
                {
                    if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                    {
                        array[n] = compressed->iFringe1[f];
                        n++;
                    }
                }
                sprintf (fileName, "%sCompressedDATA1", dataKey);
                sprintf (title, "Compressed %s DATA1", dataKey);
                midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
            }
            else if (strcmp (format->shutterId, "BOPEN") == 0)
            {
                n = 0;
                for (f = 0; f < format->numOfFrames; f++)
                {
                    if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                    {
                        array[n] = compressed->iFringe1[f];
                        n++;
                    }
                }
                sprintf (fileName, "%sCompressedDATA4", dataKey);
                sprintf (title, "Compressed %s DATA4", dataKey);
                midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
            }
            else
            {
                if (strcmp (dataKey, "PHOTOMA") == 0)
                {
                    n = 0;
                    for (f = 0; f < format->numOfFrames; f++)
                    {
                        if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                        {
                            array[n] = compressed->iFringe1[f];
                            n++;
                        }
                    }
                    sprintf (fileName, "%sCompressedDATA1", dataKey);
                    sprintf (title, "Compressed %s DATA1", dataKey);
                    midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
                }
                else
                {
                    n = 0;
                    for (f = 0; f < format->numOfFrames; f++)
                    {
                        if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iFringe1[f]))))
                        {
                            array[n] = compressed->iFringe1[f];
                            n++;
                        }
                    }
                    sprintf (fileName, "%sCompressedDATA4", dataKey);
                    sprintf (title, "Compressed %s DATA4", dataKey);
                    midiCreatePlotFileDouble2D (fileName, title, "Frame", "Flux", 0, array, 0, n, 1);
                }
            }
        }
        free (fileName);
        free (title);
        free (array);
    }
    
    //    Release memory
    free (dataFilePath);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  displayPhotomChoppingDisp
* Input/Output: See function arguments to avoid duplication
* Description:  Determines chopping coherency
*
* History:
* 06-Dec-05     (csabet)
******************************************************************************/
void displayPhotomChoppingDisp (
    const char            *dataKey,        // In: Whether Interferometry or Photometry etc.
    ImageFormat        *format,        // In: Image format
    CompressedData    *compressed)    // In: Compressed data
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "displayPhotomChoppingDisp";
    int            f, n, X, R;
    char        *fileName, *title;
    float        *arrayF;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);


    //    Display for each channel
    if (plotFile && diagnostic > 4)
    {
        arrayF = (float *) calloc (format->numOfFrames, sizeof (float));
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        
        for (R = 0; R < format->numOfRegionsToProcess; R++)
        {
            for (X = 0; X < format->iXWidth; X++)
            {
                n = 0;
                for (f = 0; f < format->numOfFrames; f++)
                {
                    if ((compressed->tarType[f] != 'U') && (!(isnan (compressed->iDispFringe[R][X][f]))))
                    {
                        arrayF[n] = compressed->iDispFringe[R][X][f];
                        n++;
                    }
                }
                if ((strcmp (batchTemplate, "HIGH_SENS_CALIB") == 0) ||
                    (strcmp (batchTemplate, "HIGH_SENS_SCIENCE") == 0))
                {
                    sprintf (fileName, "%sCompressedDATA%d_X%d", dataKey, R+1, X+1);
                    sprintf (title, "Compressed %s DATA%d for Channel %d", dataKey, R+1, X+1);
                }
                else
                {
                    if (strcmp (dataKey, "PHOTOMA") == 0)
                    {
                        sprintf (fileName, "%sCompressedDATA%d_X%d", dataKey, R+1, X+1);
                        sprintf (title, "Compressed %s DATA%d for Channel %d", dataKey, R+1, X+1);
                    }
                    else
                    {
                        sprintf (fileName, "%sCompressedDATA%d_X%d", dataKey, R+4, X+1);
                        sprintf (title, "Compressed %s DATA%d for Channel %d", dataKey, R+4, X+1);
                    }
                }
                midiCreatePlotFile2D (fileName, title, "Frame", "Flux", 0, arrayF, 0, n, 1, 0);
            }
        }
        free (fileName);
        free (title);
        free (arrayF);
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  countTransitions
* Input/Output: See function arguments to avoid duplication
* Description:  Checks chopping transitions
*
* History:
* 08-Feb-06     (csabet) Created
******************************************************************************/
int countTransitions (    // Ou: Chopping transitions
    char    key,        // In: Given TarTyp
    int        length,        // In: Length of array
    char    *tarType)    // In: Array of tarType

{
    //    Local Declarations
    //    ------------------
    const char        routine[] = "countTransitions";
    int                i, transitions, sighting;
    char            lastTarTyp;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Initialise
    transitions = 0;
    sighting = 0;
    lastTarTyp = tarType[0];

    //    Count transitions
    if (lastTarTyp == key) sighting++;
    for (i = 1; i < length; i++)
    {
        if (tarType[i] == key)
        {
            if (sighting && tarType[i] != lastTarTyp) 
                transitions++;

            sighting++;
        }
        
        lastTarTyp = tarType[i];
    }
    if (!sighting) transitions = - 1;

    return (transitions);
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  midiReportResultsFringe
* Input/Output: See function arguments to avoid duplication
* Description:  Reports all the results
*
* History:
* 23-Feb-05     (csabet) Created
******************************************************************************/
void midiReportResultsFringe (
    float           *freqCal,        // IO: Calibrated frequency in THz
    ImageFormat        *format,        // In: Image size of interferometry data
    DispersedResult    *dispResult)    // In: Result data structure
{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "midiReportResultsFringe";
    int            X;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);


   cpl_msg_info(cpl_func,"\nReporting the results for batch %d \n", batchNumber);
   cpl_msg_info(cpl_func,"------------------------------- \n");
    fprintf (midiReportPtr, "\nReporting the results for batch %d \n", batchNumber);
    fprintf (midiReportPtr, "------------------------------- \n");

    //    Recalculate number of channels rejected
    format->numOfChannelsProcessed = 0;
    for (X = 0; X < format->iXWidth; X++)
    {
        if (!(badChannelList[X]))
            (format->numOfChannelsProcessed)++;
    }
    format->numOfChannelsRejected = format->iXWidth - format->numOfChannelsProcessed;
    
   cpl_msg_info(cpl_func,"Observation Category         = %s \n", format->obsCatg);
   cpl_msg_info(cpl_func,"\nResult Abbreviation \n");
   cpl_msg_info(cpl_func,"------------------- \n");
   cpl_msg_info(cpl_func,"X      = Channel Number \n");
   cpl_msg_info(cpl_func,"F      = Channel Frequency in THz \n");
   cpl_msg_info(cpl_func,"W      = Channel Wavelength in microns \n");
   cpl_msg_info(cpl_func,"RVis   = Raw Visibility \n");
   cpl_msg_info(cpl_func,"RVisEr = Raw Visibility Error (Standard deviation) \n");
   cpl_msg_info(cpl_func,"NVis   = Normalised Visibility \n");
   cpl_msg_info(cpl_func,"NVisEr = Normalised Visibility Error (Standard deviation) \n");
   cpl_msg_info(cpl_func,"PhNe   = \'Net\' Photometry \n");
   cpl_msg_info(cpl_func,"PhNeEr = \'Net\' Photometry Error (Standard deviation) \n");
   cpl_msg_info(cpl_func,"TF     = Transfer Function \n");
   cpl_msg_info(cpl_func,"TFEr   = Transfer Function Error (Standard deviation) \n");
   cpl_msg_info(cpl_func,"CVis   = Calibrated Visibility \n");
   cpl_msg_info(cpl_func,"CVisEr = Calibrated Visibility Error (Standard deviation) \n");

    fprintf (midiReportPtr, "Observation Category      = %s                               QCLOG \n", format->obsCatg);
    fprintf (midiReportPtr, "\nResult Abbreviation                                        QCLOG \n");
    fprintf (midiReportPtr, "-------------------                                          QCLOG \n");
    fprintf (midiReportPtr, "X      = Channel Number                                      QCLOG \n");
    fprintf (midiReportPtr, "F      = Channel Frequency in THz                            QCLOG \n");
    fprintf (midiReportPtr, "W      = Channel Wavelength in microns                       QCLOG \n");
    fprintf (midiReportPtr, "RVis   = Raw Visibility                                      QCLOG \n");
    fprintf (midiReportPtr, "RVisEr = Raw Visibility Error (Standard deviation)           QCLOG \n");
    fprintf (midiReportPtr, "NVis   = Normalised Visibility                               QCLOG \n");
    fprintf (midiReportPtr, "NVisEr = Normalised Visibility Error (Standard deviation)    QCLOG \n");
    fprintf (midiReportPtr, "PhNe   = \'Net\' Photometry                                  QCLOG \n");
    fprintf (midiReportPtr, "PhNeEr = \'Net\' Photometry Error (Standard deviation)       QCLOG \n");
    fprintf (midiReportPtr, "TF     = Transfer Function                                   QCLOG \n");
    fprintf (midiReportPtr, "TFEr   = Transfer Function Error (Standard deviation)        QCLOG \n");
    fprintf (midiReportPtr, "CVis   = Calibrated Visibility                               QCLOG \n");
    fprintf (midiReportPtr, "CVisEr = Calibrated Visibility Error (Standard deviation)    QCLOG \n");

   cpl_msg_info(cpl_func,"\nX    W      F        RVis          RVisEr  NVis    NVisEr      ");
   cpl_msg_info(cpl_func,"PhNe      PhNeEr   TF      TFEr     CVis    CVisEr \n");
    fprintf (midiReportPtr, "\nX    W      F        RVis          RVisEr  NVis    NVisEr      ");
    fprintf (midiReportPtr, "PhNe      PhNeEr   TF      TFEr     CVis    CVisEr    QCLOG \n");

    for (X = 0; X < format->iXWidth; X++)
    {
        if (badChannelList[X])
        {
           cpl_msg_info(cpl_func,"%3d  %5.2f  %5.2f          <-- %s --> \n", 
                X, freqCal[X], SPEED_OF_LIGHT/freqCal[X], UNAV); 
             fprintf (midiReportPtr, "%3d  %5.2f  %5.2f          <-- %s -->  QCLOG \n", 
                X, freqCal[X], SPEED_OF_LIGHT/freqCal[X], UNAV); 
            continue;
        }
    
       cpl_msg_info(cpl_func,"%3d  %5.2f  %5.2f  %11.5f  %9.4f  %6.4f  %8.4f  %9.1f %7.2f   %6.4f   %6.4f   %5.3f    %5.3f \n", 
            X, freqCal[X], SPEED_OF_LIGHT/freqCal[X], 
            dispResult->rawVis2[X], dispResult->rawVis2Err[X], 
            dispResult->normVis2[X], dispResult->normVis2Err[X], 
            dispResult->photomNet2[X], dispResult->photomNet2Err[X], 
            dispResult->trf[X], dispResult->trfErr[X], 
            dispResult->calibVis2[X], dispResult->calibVis2Err[X]);
 
        fprintf (midiReportPtr, "%3d  %5.2f  %5.2f  %11.4f  %9.4f  %6.4f  %8.4f  %9.1f %7.2f   %6.4f   %6.4f   %5.3f    %5.3f     QCLOG\n", 
            X, freqCal[X], SPEED_OF_LIGHT/freqCal[X], 
            dispResult->rawVis2[X], dispResult->rawVis2Err[X], 
            dispResult->normVis2[X], dispResult->normVis2Err[X], 
            dispResult->photomNet2[X], dispResult->photomNet2Err[X], 
            dispResult->trf[X], dispResult->trfErr[X], 
            dispResult->calibVis2[X], dispResult->calibVis2Err[X]);
    }

    return;
}
/*****************************************************************************/

/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  reportRejectList
* Input/Output: See function arguments to avoid duplication
* Description:    Final result of the rejection list for dispersed processing
*
* History:
* 14-Mar-06     (csabet) Created
******************************************************************************/
void reportRejectList (
    ImageFormat        *format,        // In: Points to the image format
    CompressedData    *compressed)    // In: Compressed data
{
    //    Local Declarations
    //--------------------
    const char  routine[] = "reportRejectList";
    int            f, x, numOfFramesProcessed, numOfFramesRejected, frameFailureId;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine '%s'  \n", routine);

   cpl_msg_info(cpl_func,"\nFinalising Scan, Frame and Channel rejection \n");
   cpl_msg_info(cpl_func,"-------------------------------------------- \n");
    fprintf (midiReportPtr, "\nFinalising Scan, Frame and Channel rejection \n");
    fprintf (midiReportPtr, "-------------------------------------------- \n");

    if (diagnostic)cpl_msg_info(cpl_func,"\nChannel   Number of Frames   Number of Frames   Frame Failure ID   Channel Failure ID \n");
    if (diagnostic)cpl_msg_info(cpl_func,"              Processed          Rejected \n");
    if (diagnostic)cpl_msg_info(cpl_func,"------------------------------------------------------------------------------------- \n");
    fprintf (midiReportPtr, "\nChannel   Number of Frames   Number of Frames   Frame Failure ID   Channel Failure ID \n");
    fprintf (midiReportPtr, "              Processed          Rejected \n");
    fprintf (midiReportPtr, "------------------------------------------------------------------------------------- \n");

    format->numOfChannelsRejected = 0;
    for (x = 0; x < format->iXWidth; x++)
    {
        frameFailureId = 0;
        numOfFramesProcessed = 0;
        
        //    If channel was masked no frames were processed
        for (f = 0; f < format->numOfFrames; f++)
        {
            if (!(compressed->rejectList[x][f]))
                numOfFramesProcessed++;

            frameFailureId |= compressed->rejectList[x][f];
        }

        //    Count rejected channels
        if (!numOfFramesProcessed || (badChannelList[x]))
            (format->numOfChannelsRejected)++;
        
        //    Count rejected frames
        numOfFramesRejected = format->numOfFrames - numOfFramesProcessed;
        
        if (badChannelList[x])
        {
            if (diagnostic)cpl_msg_info(cpl_func,
                "%3d       %4d                %4d                %4d                %4d                REJECTED\n", 
                x, numOfFramesProcessed, numOfFramesRejected, frameFailureId, badChannelList[x]);
            fprintf (midiReportPtr, 
                "%3d       %4d                %4d                %4d                %4d                REJECTED  QCLOG \n", 
                x, numOfFramesProcessed, numOfFramesRejected, frameFailureId, badChannelList[x]);
        }
        else
        {
            if (diagnostic)cpl_msg_info(cpl_func,
                "%3d       %4d                %4d                %4d                %4d \n", 
                x, numOfFramesProcessed, numOfFramesRejected, frameFailureId, badChannelList[x]);
            fprintf (midiReportPtr, 
                "%3d       %4d                %4d                %4d                %4d                          QCLOG \n", 
                x, numOfFramesProcessed, numOfFramesRejected, frameFailureId, badChannelList[x]);
        }
    }

    //    Count number of channels to process
    format->numOfChannelsProcessed = format->iXWidth - format->numOfChannelsRejected;

   cpl_msg_info(cpl_func,"Number of channels processed  = %3d \n", format->numOfChannelsProcessed);
   cpl_msg_info(cpl_func,"Number of channels rejected   = %3d \n", format->numOfChannelsRejected);
    fprintf (midiReportPtr, "Number of channels processed  = %3d QCLOG \n", format->numOfChannelsProcessed);
    fprintf (midiReportPtr, "Number of channels rejected   = %3d QCLOG \n", format->numOfChannelsRejected);

    if (diagnostic)
    {
       cpl_msg_info(cpl_func,"Failure ID \n");
       cpl_msg_info(cpl_func,"---------- \n");
       cpl_msg_info(cpl_func,"   1 = Bit  1, indicates a TIME error \n");
       cpl_msg_info(cpl_func,"   2 = Bit  2, indicates an LOCALOPD error \n");
       cpl_msg_info(cpl_func,"   4 = Bit  3, indicates an OPD error \n");
       cpl_msg_info(cpl_func,"   8 = Bit  4, indicates a Bad DATA \n");
       cpl_msg_info(cpl_func,"  16 = Bit  5, indicates a chopping problem (TARTYPE2) \n");
       cpl_msg_info(cpl_func,"  32 = Bit  6, indicates a Delay Line Jump \n");
       cpl_msg_info(cpl_func,"  64 = Bit  7, indicates an Unwanted Region \n");
       cpl_msg_info(cpl_func," 128 = Bit  8, indicates a Weak Signal/Noise \n");
       cpl_msg_info(cpl_func," 256 = Bit  9, indicates a SKY or Undefined \n");
       cpl_msg_info(cpl_func," 512 = Bit 10, indicates a TARTYPE Cross error. e.g. region 0 different to region 1 \n");
       cpl_msg_info(cpl_func,"1024 = Bit 11, indicates a Masked Channel \n");
       cpl_msg_info(cpl_func,"\nHence a number such as 168 indicates bits 4, 6 and 8 were set \n\n");
    }    

    fprintf (midiReportPtr, "Failure ID QCLOG \n");
    fprintf (midiReportPtr, "---------- QCLOG \n");
    fprintf (midiReportPtr, "   1 = Bit  1, indicates a TIME error QCLOG \n");
    fprintf (midiReportPtr, "   2 = Bit  2, indicates an LOCALOPD error QCLOG \n");
    fprintf (midiReportPtr, "   4 = Bit  3, indicates an OPD error QCLOG \n");
    fprintf (midiReportPtr, "   8 = Bit  4, indicates a Bad DATA QCLOG \n");
    fprintf (midiReportPtr, "  16 = Bit  5, indicates a chopping problem (TARTYPE2) QCLOG \n");
    fprintf (midiReportPtr, "  32 = Bit  6, indicates a Delay Line Jump QCLOG \n");
    fprintf (midiReportPtr, "  64 = Bit  7, indicates an Unwanted Region QCLOG \n");
    fprintf (midiReportPtr, " 128 = Bit  8, indicates a Weak Signal/Noise QCLOG \n");
    fprintf (midiReportPtr, " 256 = Bit  9, indicates a SKY or Undefined QCLOG \n");
    fprintf (midiReportPtr, " 512 = Bit 10, indicates a TARTYPE Cross error. e.g. region 0 different to region 1 QCLOG \n");
    fprintf (midiReportPtr, "1024 = Bit 11, indicates a Masked Channel QCLOG \n");
    fprintf (midiReportPtr, "\nHence a number such as 168 indicates bits 4, 6 and 8 were set QCLOG \n\n");

    return;
}
/*****************************************************************************/
