/******************************************************************************
*******************************************************************************
*                European Southern Observatory
*                VLTI Data Reduction Software
*
* File name:    errorHandling.c
* Description:    Contains the routines for systematic error and warning reports
*
* History:        
* 22-Jul-03        (csabet) Created 
*******************************************************************************
******************************************************************************/

/******************************************************************************
*    Compiler directives
******************************************************************************/

/******************************************************************************
*    Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include "midiGlobal.h"
#include "errorHandling.h"
#include "midiLib.h"

/******************************************************************************
*    Global Variables
******************************************************************************/

/******************************************************************************
*    Prototypes
******************************************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  midiReportTbd
* Input/Output: See function arguments to avoid duplication
* Description:  Diagnostic report
*
* History:
* 17-Jun-05     (csabet)
******************************************************************************/
void midiReportTbd (
    FILE        *filePtr,
    const char  routineName[],
    const char        *fileName,
    int            lineNumber,
    const char        *message)
{
    
   cpl_msg_info(cpl_func,"\n<<TBD>>: %s \n", message);
   cpl_msg_info(cpl_func,"         Routine Name: %s \n", routineName);
   cpl_msg_info(cpl_func,"         File Name:    %s \n", fileName);
   cpl_msg_info(cpl_func,"         Line Number:  %d \n\n", lineNumber);

    fprintf (filePtr, "\n<<TBD>>: %s \n", message);
    fprintf (filePtr, "         Routine Name: %s \n", routineName);
    fprintf (filePtr, "         File Name:    %s \n", fileName);
    fprintf (filePtr, "         Line Number:  %d \n\n", lineNumber);

    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    midiReportError
* Input/Output:    See function arguments to avoid duplication
* Description:    Exception Handling
*
* History:        
* 03-Sep-03        (csabet)
******************************************************************************/
void midiReportError(
    FILE        *filePtr,
    const char  routineName[],
    const char        *fileName,
    int            lineNumber,
    const char        *message)
{
   
   cpl_msg_info(cpl_func,"\n<<ERROR>>: %s \n", message);
   cpl_msg_info(cpl_func,"           Routine Name: %s \n", routineName);
   cpl_msg_info(cpl_func,"           File Name:    %s \n", fileName);
   cpl_msg_info(cpl_func,"           Line Number:  %d \n\n", lineNumber);

    fprintf (filePtr, "\n<<ERROR>>: %s \n", message);
    fprintf (filePtr, "           Routine Name: %s \n", routineName);
    fprintf (filePtr, "           File Name:    %s \n", fileName);
    fprintf (filePtr, "           Line Number:  %d \n\n", lineNumber);
    
    fclose (filePtr);
    exit (2);    
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    midiReportInfo
* Input/Output:    See function arguments to avoid duplication
* Description:    Exception Handling for Information
*
* History:        
* 18-Nov-03        (csabet)
******************************************************************************/
void midiReportInfo(
    FILE        *filePtr,
    const char  routineName[],
    const char        *fileName,
    int            lineNumber,
    const char        *message)
{

    if (diagnostic > 4)
    {
       cpl_msg_info(cpl_func,"\n%s \n", message);
       cpl_msg_info(cpl_func,"Routine Name: %s \n", routineName);
       cpl_msg_info(cpl_func,"File Name:    %s \n", fileName);
       cpl_msg_info(cpl_func,"Line Number:  %d \n", lineNumber);

        fprintf (filePtr, "\n%s \n", message);
        fprintf (filePtr, "Routine Name: %s \n", routineName);
        fprintf (filePtr, "File Name:    %s \n", fileName);
        fprintf (filePtr, "Line Number:  %d \n", lineNumber);
    }
    else
    {
       cpl_msg_info(cpl_func,"%s \n", message);
        fprintf (filePtr, "%s \n", message);
    }

    return;
}
/*****************************************************************************/


/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    midiReportError
* Input/Output:    See function arguments to avoid duplication
* Description:    Exception Handling for warnings
*
* History:        
* 18-Nov-03        (csabet)
******************************************************************************/
void midiReportWarning(
    FILE        *filePtr,
    const char  routineName[],
    const char        *fileName,
    int            lineNumber,
    const char        *message)
{
   
    if (diagnostic > 4)
    {
       cpl_msg_info(cpl_func,"\n<<WARNING>>: %s \n", message);
       cpl_msg_info(cpl_func,"             Routine Name: %s \n", routineName);
       cpl_msg_info(cpl_func,"             File Name:    %s \n", fileName);
       cpl_msg_info(cpl_func,"             Line Number:  %d \n\n", lineNumber);

        fprintf (filePtr, "\n<<WARNING>>: %s \n", message);
        fprintf (filePtr, "             Routine Name:  %s \n", routineName);
        fprintf (filePtr, "             File Name:     %s \n", fileName);
        fprintf (filePtr, "             Line Number:   %d \n\n", lineNumber);
    }
    else
    {
       cpl_msg_info(cpl_func,"<<WARNING>>: %s \n", message);
        fprintf (filePtr, "<<WARNING>>: %s \n", message);
    }
    
    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  Trace
* Input/Output: See function arguments to avoid duplication
* Description:    
*
* History:
* 21-Jul-03     (JM)
* 20-Jan-05        (csabet) Integrated
******************************************************************************/
void Trace (
    int    n)
{
   if (!diagnostic) return;
  cpl_msg_info(cpl_func,"\nTrace %d", n);
  cpl_msg_info(cpl_func,"\n");
}
/*****************************************************************************/



/******************************************************************************
*                European Southern Observatory
*              VLTI MIDI Data Reduction Software
*
* Module name:    abortIfError
* Input/Output:    See function arguments to avoid duplication
* Description:    Exception Handling
*
* History:        
* 24-Feb-03        (jmeisner)    Leiden. Created
* 03-Mar-03        (csabet)     Revised
******************************************************************************/
void abortIfError(
    int        errcode, 
    char    *message)
{
    if(!errcode) return;
   cpl_msg_info(cpl_func,"\n\n <<ERROR>: %s, code %d.\n\n", message, errcode);
    exit(errcode);
}
/*****************************************************************************/
