/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  estimationFrgHSUndisp.c
* Description:  Contains routines for all data preprocessing algorithms
*
* History:
* 12-Jul-03     (csabet) Created. Derived from (jmeisner 03-Feb-03)
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "midiGlobal.h"
#include "fft.h"
#include "complex_midi.h"
#include "statistics.h"
#include "diagnostics.h"
#include "errorHandling.h"
#include "midiLib.h"
#include "estimationFrgHSUndisp.h"
#include "photometry.h"
#include "visibility.h"
#include "calibration.h"

/**********************************************************
*   Constant definitions
**********************************************************/
 
/*============================ C O D E    A R E A ===========================*/
 

/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  estimateFrgHSUndisp
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 10-Jan-04     (csabet) Added several routines and made many modifications
* 09-Dec-03     (csabet) Modified the interface to accomodate development of findGoodScanRange
* 17-Sep-03     (jmeisner) Completed the code
* 21-Jul-03     (csabet) Created routine template
******************************************************************************/
void estimateFrgHSUndisp (
    CompressedData        *compressedInterf,    // In: Compressed interferometry data
    CompressedData        *compressedPhotA,    // In: Compressed photometry data
    CompressedData        *compressedPhotB,    // In: Compressed photometry data
    FilterData            *filterInfo,        // In: Filter information
    ImageFormat            *formatInterf,        // In: Points to the image format
    ImageFormat            *formatPhotomA,        // In: Points to the image format
    ImageFormat            *formatPhotomB,        // In: Points to the image format
    RawVisibility        *rawVis,            // Ou: Result of visibilities
    PhotometryResult    *photometry)        // Ou: Result of photometry. Added (csabet 16-Jan-04)
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "estimateFrgHSUndisp";
    float        *accumPS, *accumPS1, *accumPS2;
    float        photomA[2], photomB[2], *photomErrA, *photomErrB;
    float        photomChop2ChopA[2];    // RMS of chop to chop variations.
    float        photomChop2ChopB[2];     // RMS of chop to chop variations.
    float        fractionalBandwidth;
    int            lofreq, hifreq;
    float        ratio_photA0_to_photA1, ratio_errA0_to_photA0, ratio_errA1_to_photA1, ratio_photB0_to_photB1,
                ratio_errB0_to_photB0, ratio_errB1_to_photB1;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'\n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s'\n", routine);

   cpl_msg_info(cpl_func,"\nComputing Normalized visibilities from batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"-------------------------------------------- \n");
    fprintf(midiReportPtr, "\nComputing Normalized visibilities from batch  %d \n", batchNumber);
    fprintf(midiReportPtr, "-------------------------------------------- \n");

    /*  Allocate memory */
    photomErrA = (float *) calloc (2, sizeof (float));
    photomErrB = (float *) calloc (2, sizeof (float));

    if (diagnostic) cpl_msg_info(cpl_func,"Running photometric files...\n" );
    fprintf (midiReportPtr, "Running photometric files...\n" );
    estimatePhotomUndisp (formatPhotomA, compressedPhotA, photomErrA, photomChop2ChopA, photomA);
    estimatePhotomUndisp (formatPhotomB, compressedPhotB, photomErrB, photomChop2ChopB, photomB);

    //    Singularity checks.    (Added, csabet 12-Jan-04)
    if (photomA[1] != 0.)
    {
        ratio_photA0_to_photA1 = photomA[0]/photomA[1];
        ratio_errA1_to_photA1 = 100.F*photomErrA[1]/photomA[1];
    }
    else
    {
        ratio_photA0_to_photA1 = NOT_A_NUMBER;
        ratio_errA1_to_photA1 = NOT_A_NUMBER;
    }

    if (photomA[0] != 0.)
        ratio_errA0_to_photA0 = 100.F*photomErrA[0]/photomA[0];
    else
        ratio_errA0_to_photA0 = NOT_A_NUMBER;

    if (photomB[1] != 0.)
    {
        ratio_photB0_to_photB1 = photomB[0]/photomB[1];
        ratio_errB1_to_photB1 = 100.F*photomErrB[1]/photomB[1];
    }
    else
    {
        ratio_photB0_to_photB1 = NOT_A_NUMBER;
        ratio_errB1_to_photB1 = NOT_A_NUMBER;
    }

    //    Singularity checks.    (Added, csabet 12-Jan-04)
    if (photomB[0] != 0.)
        ratio_errB0_to_photB0 = 100.F*photomErrB[0]/photomB[0];
    else
        ratio_errB0_to_photB0 = NOT_A_NUMBER;

    if (diagnostic)cpl_msg_info(cpl_func,
        "\nPhotometric results \n"
        "------------------- \n"
        "          region0    region1   ratio        RMS chop-to-chop    net photometric err \n"
        "Aopen   %8.0f   %8.0f    %5.4f        %4.1f%%   %4.1f%%       %4.1f%%   %4.1f%% \n"
        "Bopen   %8.0f   %8.0f    %5.4f        %4.1f%%   %4.1f%%       %4.1f%%   %4.1f%% \n",
        photomA[0], photomA[1], ratio_photA0_to_photA1, 100.F*photomChop2ChopA[0], 100.F*photomChop2ChopA[1],
        ratio_errA0_to_photA0,  ratio_errA1_to_photA1,
        photomB[0], photomB[1], ratio_photB0_to_photB1, 100.F*photomChop2ChopB[0], 100.F*photomChop2ChopB[1],
        ratio_errB0_to_photB0, ratio_errB1_to_photB1);

    fprintf(midiReportPtr,
        "\nPhotometric results                                                                  QCLOG \n"
        "-------------------                                                                  QCLOG \n"
        "          region0    region1   ratio        RMS chop-to-chop    net photometric err  QCLOG \n"
        "Aopen   %8.0f   %8.0f    %5.4f        %4.1f%%   %4.1f%%       %4.1f%%   %4.1f%%      QCLOG \n"
        "Bopen   %8.0f   %8.0f    %5.4f        %4.1f%%   %4.1f%%       %4.1f%%   %4.1f%%      QCLOG \n",
        photomA[0], photomA[1], ratio_photA0_to_photA1, 100.F*photomChop2ChopA[0], 100.F*photomChop2ChopA[1],
        ratio_errA0_to_photA0,  ratio_errA1_to_photA1,
        photomB[0], photomB[1], ratio_photB0_to_photB1, 100.F*photomChop2ChopB[0], 100.F*photomChop2ChopB[1],
        ratio_errB0_to_photB0, ratio_errB1_to_photB1);

    //    Temporarily hand over results and clean up later. Added. csabet 16-Jan-04
    photometry->photomA[0] = photomA[0];
    photometry->photomAErr[0] = photomErrA[0];
    photometry->photomA[1] = photomA[1];
    photometry->photomAErr[1] = photomErrA[1];

    photometry->photomB[0] = photomB[0];
    photometry->photomBErr[0] = photomErrB[0];
    photometry->photomB[1] = photomB[1];
    photometry->photomBErr[1] = photomErrB[1];

    photometry->ratioPhotomA0toA1 = ratio_photA0_to_photA1;
    photometry->ratioPhotomB0toB1 = ratio_photB0_to_photB1;

    photometry->chop2ChopAErr[0] = photomChop2ChopA[0];
    photometry->chop2ChopAErr[1] = photomChop2ChopA[1];
    photometry->chop2ChopBErr[0] = photomChop2ChopB[0];
    photometry->chop2ChopBErr[1] = photomChop2ChopB[1];

    //    Added. csabet 12-Jan-04
    if (photomA[1] == 0. || photomA[0] == 0. || photomB[1] == 0. || photomB[0] == 0.)
    {
        sprintf (midiMessage, "A value of  %.4f  is a reject result. QCLOG", (float)NOT_A_NUMBER);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
 
    //    Preset for the fft algorithm
    if (diagnostic) cpl_msg_info(cpl_func,"FFT size = %d\n",formatInterf->fftsize); 
    fprintf (midiReportPtr, "FFT size = %d\n",formatInterf->fftsize); 
    SetFFTsize (formatInterf->fftsize);
    
    //    Allocate 3 arrays for the (differenced) power spectrum accumulated from N scans
    accumPS = (float *) calloc (formatInterf->fftsize/2, sizeof(float));
    accumPS1 = (float *) calloc (formatInterf->fftsize/2, sizeof(float));
    accumPS2 = (float *) calloc (formatInterf->fftsize/2, sizeof(float));
    
    // Figure out the bandwidth from the filter description:
    fractionalBandwidth = 2.0F * (filterInfo->optFreqHi - filterInfo->optFreqLo)/formatInterf->optFreqCal;
    lofreq = (int) ((0.9F * ((float) formatInterf->fftsize) * filterInfo->optFreqLo)/formatInterf->optFreqCal);
    hifreq = (int) ((1.1F * ((float) formatInterf->fftsize) * filterInfo->optFreqHi)/formatInterf->optFreqCal);

    if (hifreq >= (formatInterf->fftsize/2))
        hifreq = formatInterf->fftsize/2 - 1;

    if (lofreq >= hifreq)
        lofreq = hifreq - 1;

    if (diagnostic)cpl_msg_info(cpl_func,
        "\nBandwidth used \n"
        "-------------- \n"
        "Specified (Min, Max) THz = %6.3f, %6.3f \n"
        "Integer   (Min, Max)     = %d,  %d \n"
        "Fractional THz           = %6.3f \n"
        "Folding frequency THz    = %6.3f at %2d \n\n",
        filterInfo->optFreqLo, filterInfo->optFreqHi, lofreq, hifreq,
        fractionalBandwidth, .5F*formatInterf->optFreqCal, formatInterf->fftsize/2);
    fprintf (midiReportPtr,
        "\nBandwidth used                             QCLOG \n"
        "--------------                             QCLOG \n"
        "Specified (Min, Max) THz = %6.3f, %6.3f    QCLOG \n"
        "Integer   (Min, Max)     = %d, %d          QCLOG \n"
        "Fractional THz           = %6.3f           QCLOG \n"
        "Folding frequency THz    = %6.3f at %2d    QCLOG \n\n",
        filterInfo->optFreqLo, filterInfo->optFreqHi, lofreq, hifreq,
        fractionalBandwidth, .5F*formatInterf->optFreqCal, formatInterf->fftsize/2);
 
    accumulatePowerSpectra (0, formatInterf, compressedInterf->allSpectrum, compressedInterf->badScanList,
        formatInterf->numOfScansProcessed, accumPS);

    rawVis->visSqrd = estimateCorrelatedFlux2 (formatInterf, accumPS, fractionalBandwidth,
        lofreq, hifreq, 0.F);

    rawVis->vis = estimateCorrelatedFlux (formatInterf, accumPS, lofreq, hifreq, 0.F);

    accumulatePowerSpectra (1, formatInterf, compressedInterf->allSpectrum, compressedInterf->badScanList,
        formatInterf->numOfScansProcessed, accumPS1);
        
    rawVis->visSqrd1= estimateCorrelatedFlux2 (formatInterf, accumPS1, fractionalBandwidth,
        lofreq, hifreq, 0.F);
        
    rawVis->vis1 = estimateCorrelatedFlux (formatInterf, accumPS1, lofreq, hifreq, 0.F);

    accumulatePowerSpectra (2, formatInterf, compressedInterf->allSpectrum, compressedInterf->badScanList,
        formatInterf->numOfScansProcessed, accumPS2);
        
    rawVis->visSqrd2 = estimateCorrelatedFlux2 (formatInterf, accumPS2, fractionalBandwidth,
        lofreq, hifreq, 0.F);
        
    rawVis->vis2 = estimateCorrelatedFlux (formatInterf, accumPS2, lofreq, hifreq, 0.F);

    //    This requires all the photometric processing, as well as vis, vis1, vis2
    normalizeVis (rawVis, photomA, photomB);

    free (photomErrA);
    free (photomErrB);
    free (accumPS);
    free (accumPS1);
    free (accumPS2);

    return;
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  accumulatePowerSpectra
* Input/Output: See function arguments to avoid duplication
* Description:
*
* History:
* 07-Jan-04     (csabet) Created. Derived from (jmeisner)
******************************************************************************/
void accumulatePowerSpectra (
    int            region,                    // In: region to be processed
    ImageFormat    *imageformat,            // In: Points to the image format
    float        *allSpectrum,            // In: Pointer to the entire spectra
    int            *badScanList,            // In: Scan range mask
    int            numOfScansProcessed,    // In: Number of scans processed
    float        *accumPS)                // Ou: Points to accumulated power spectra
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "accumulatePowerSpectra";
    int         i, j, fftSizeHalf;
    float       *floatPtr, norm, plotfreqcal;
 

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'\n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s'\n", routine);
    
    //    Initialize
    fftSizeHalf = 0.5 * imageformat->fftsize;
    plotfreqcal = imageformat->optFreqCal/((float)imageformat->fftsize);
    if (imageformat->optFreqCal == 0.F)    plotfreqcal = 1.F;    // So axis will just say 0,1,2,3....

    //    Accumulate scans. First make signal point to the correct region
    floatPtr = allSpectrum + (region * imageformat->numOfScans * fftSizeHalf);
    for (j = 0; j < imageformat->numOfScans; j++)
    {
        if (badScanList[j] == 0)    // For each good scan
        {
            for (i = 0; i < fftSizeHalf; i++)    // And for each element of the power spectra
              accumPS[i] += *floatPtr++;        // Add in the element i from spectrum
        }
        else
        {
            for (i = 0; i < fftSizeHalf; i++)
              floatPtr++;
        }
    }

    //    Normalize (to get average PS)
    if (numOfScansProcessed > 0)    // Check singularity
        norm = 1.0 / ((float) numOfScansProcessed);
    else
        norm = 1.0;

    for (i = 0; i < fftSizeHalf; i++)
        accumPS[i] *= norm;

    //    Tabular output in report
    fprintf (midiReportPtr, "\n--------------------------------------------------------\n"
                            "Region %d incoh power spectral amplitude, including noise\n\n", region);
    for (i = 0; i < fftSizeHalf; i++)
        fprintf (midiReportPtr, "%4.1f      %8.1f\n", plotfreqcal * ((float)i), sqrtp(accumPS[i]));

    fprintf (midiReportPtr, "\n--------------------------------------------------------\n\n");

    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  estimateCorrelatedFlux
* Input/Output: See function arguments to avoid duplication
* Description:  This is the incoherent visibility AMPLITUDE estimator. Currently
*               does NOT use a correction for weak channels. (which doesn't work well)
* History:
* 07-Jan-04     (csabet) Created. Derived from (jmeisner)
******************************************************************************/
float estimateCorrelatedFlux (      /*  Ou: Visibility */
    ImageFormat *imageFormat,       /*  In: Points to the image format */
    float       *accumPS,           /*  In: Points to accumulated power spectra */
    int         loFreq,             /*  In: Lower limit limit of the bandwidth */
    int         hiFreq,             /*  In: Higher limit of the bandwidth */
    float       noiselev)           /*  In: If noiselev ==0, will decide using spectral range above hifreq */
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "estimateCorrelatedFlux";
    int         i, fftSizeHalf;
    float       Accum = 0.0, result;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'  \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s'  \n", routine);    

    /*  Initialize */
    fftSizeHalf = 0.5 * imageFormat->fftsize;

    if (noiselev == 0.0)
    {
        for (i = hiFreq; i < fftSizeHalf; i++)
            Accum += accumPS[i];

        if ((fftSizeHalf - hiFreq) != 0)    /*  Check singularity */
            noiselev = Accum/((float) (fftSizeHalf - hiFreq));
        else
            noiselev = Accum/VERY_SMALL_FLOAT;

       cpl_msg_info(cpl_func,"Estimated noise power level = %f\n", noiselev);
        fprintf (midiReportPtr, "Estimated noise power level = %f\n", noiselev);
    }

    /*  Now find the amplitude within specified bandlimits */
    Accum = 0.0;
    for (i = loFreq; i <= hiFreq; i++)  /*  How do you ensure that this range is within the allocated memory for  accumPS?  csabet 10-Nov-03 */
        Accum += sqrtp (accumPS[i] - noiselev);

    result = Accum;
   cpl_msg_info(cpl_func,"Unnormalized vis amplitude  = %7f  \n\n", result);
    fprintf (midiReportPtr, "Unnormalized vis amplitude  = %7f  \n\n", result);

    return (result);
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  estimateCorrelatedFlux2
* Input/Output: See function arguments to avoid duplication
* Description:
*
*
* History:
* 07-Jan-04     (csabet) Created. Derived from (jmeisner)
******************************************************************************/
float estimateCorrelatedFlux2 (    // Ou: Visibility squared
    ImageFormat *imageFormat,    // In: Points to the image format
    float       *accumPS,        // In: Points to accumulated power spectra
    float       bandwidth,        // In: Best estimate of effective fractional bandwidth
    int         loFreq,            // In: Lower limit limit of the bandwidth
    int         hiFreq,            // In: Higher limit of the bandwidth
    float       noiselev)        // In: If noiselev ==0, will decide using spectral range above hifreq
{

    /*  Local Declarations
    --------------------*/
    const char  routine[] = "estimateCorrelatedFlux2";
    int         i, fftSizeHalf;
    float       Accum = 0.0, result;

    /*  Algorithm
    -----------*/
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s'  \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s'  \n", routine);    

    /*  Initialize */
    fftSizeHalf = 0.5 * imageFormat->fftsize;

    if(noiselev == 0.0)
    {
        for (i = hiFreq; i < fftSizeHalf; i++)
            Accum += accumPS[i];

        if ((fftSizeHalf - hiFreq) != 0)    /*  Check singularity */
            noiselev = Accum/((float) (fftSizeHalf - hiFreq));
        else
            noiselev = Accum/VERY_SMALL_FLOAT;

       cpl_msg_info(cpl_func,"Estimated noise power level = %f\n", noiselev);
        fprintf (midiReportPtr, "Estimated noise power level = %f\n", noiselev);
    }

    /* Now find the power within specified bandlimits */
    Accum = 0.0;
    for (i = loFreq; i <= hiFreq; i++)  /*  How do you ensure that this range is within the allocated memory region for  accumPS? csabet 10-Nov-03 */
        Accum += accumPS[i];

    result = 0.5 * ((float) imageFormat->fftsize) * bandwidth * (Accum - noiselev * (hiFreq - loFreq + 1));

   cpl_msg_info(cpl_func,"Unnormalized vis^2          = %7f  \n\n", result);
    fprintf (midiReportPtr, "Unnormalized vis^2          = %7f  \n\n", result);

    return (result);
}
/*****************************************************************************/

