/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*             VLTI MIDI Data Reduction Software
*
* Module name:  preProcKappa.c
* Description:  Contains routines for pre processing files related to
*                Kappa computation
*
* History:
* 05-Dec-05     (csabet) Created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/
#define SIMULATE_DATA_x    // Only used when proper data was not available

/******************************************************************************
*   Include files
******************************************************************************/
#include <stdio.h>
#include <cpl.h>
#include <math.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "diagnostics.h"
#include "preProcKappa.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables
**********************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  preProcKappa
* Input/Output: See function arguments to avoid duplication
* Description:  Prepares the data for the DISPERSED modes of processing. 
*                This is configured for SCI_PHOT Kappa computation
*
* History:
* 05-Dec-05     (csabet) Created
******************************************************************************/
void preProcKappa (
    UserOptions        *options,                // In: User parameters
    FilterData        *filterInfo,            // IO: Pointer to the filter data structure
    MidiFiles        *fileNames,                // In: Pointer to the MIDI file structure
    CompressedData    *compressedInterfA,        // Ou: Pointer to the compressed interferometry data structure
    CompressedData    *compressedInterfB,        // Ou: Pointer to the compressed interferometry data structure
    CompressedData    *compressedPhotomA,        // Ou: Pointer to the compressed photom A data structure
    CompressedData    *compressedPhotomB,        // Ou: Pointer to the compressed photom B data structure
    ImageFormat        *formatInterfA,            // In: Interf size parameters
    ImageFormat        *formatInterfB,            // In: Interf size parameters
    ImageFormat        *formatPhotomA,            // In: PhotA size parameters
    ImageFormat        *formatPhotomB,            // In: PhotB size parameters
    int                *error)                    // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char            routine[] = "preProcKappa";
    FILE                *inFitsBatchPtr=NULL;
    int                    newFile, extNumOfImagingDataFile, extNumOfImagingDataMask, R;
    char                *maskFile, *cleanString, *classification, *stringTemp, *fileName, *title;
    unsigned int        loopCount = 0;
    ImageFormat            *localFormat;
    enum ObsTechnique    obsTech;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

   cpl_msg_info(cpl_func,"\nReducing data of batch  %d \n", batchNumber);
   cpl_msg_info(cpl_func,"---------------------- \n");
    fprintf (midiReportPtr, "\nReducing data of batch  %d \n", batchNumber);
    fprintf (midiReportPtr, "---------------------- \n");

    //    Reset status
    *error = 0;
    compressedInterfA->exists = 0;
    compressedInterfB->exists = 0;
    compressedPhotomA->exists = 0;
    compressedPhotomB->exists = 0;
    newFile = 1;
    obsTech = UNKNOWN;

    //    Allocate memory
    stringTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    cleanString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    localFormat = callocImageFormat ();
    maskFile = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot open input FITS file list. No compression has been carried out for this batch");
        free (cleanString);
        free (localFormat);
        free (stringTemp);
        free (classification);
        freeImageFormat (localFormat);
        *error = 1;
        return;
    }

    //  Loop through the list of files and compress data
    while (fgets (stringTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (stringTemp, "%s%s", fileNames->inFitsName, classification);
       cpl_msg_info(cpl_func,"\nProcessing file   %s \n", fileNames->inFitsName);
        fprintf (midiReportPtr, "\nProcessing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error) break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, localFormat, error);
            if (*error) break;
        }
        else localFormat->hasData = 0;

        //    Get filter data and mask file name from the first FITS file
        if (loopCount == 0)
        {
            getFilterData (fileNames->inFitsName, filterInfo, error);
            if (*error) break;
            else fprintf (midiReportPtr, "First file of batch = %s (QCLOG)\n", fileNames->inFitsName);

            //    Select mask file
            selectMask (options->maskMode, fileNames, maskFile, error);
            if (*error) break;

            //    Get 'extNumOfImagingDataMask' extension number of IMAGING_DATA in mask file
            extNumOfImagingDataMask = findImagingDataExtension (maskFile, MASK_IMAGING_DATA, error);
            if (*error) break;

           cpl_msg_info(cpl_func,"Mask File is   %s\n", maskFile);
            fprintf( midiReportPtr, "Mask File is   %s\n", maskFile);
        }

        //    Check if the file has data
        if (localFormat->hasData)
        {
            //    Check file Category and Type. For kappa computation, only Photometry files are used
            if ((strcmp (localFormat->obsTech, "INTERFEROMETRY") == 0))
            {
                if (diagnostic > 1)    midiReportInfo (midiReportPtr, 
                    routine, __FILE__, __LINE__, "The above Interferometry file will not be processed");
                obsTech = INTERF;
            }
            else if ((strcmp (localFormat->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) &&
                (strcmp(localFormat->shutterId,"AOPEN") == 0))
            {
                //    Check previous TYPE
                if (obsTech != PHOTOMA) newFile = 1;
                else newFile = 0;
                
                //    Set the TYPE flag
                obsTech = PHOTOMA;
                
                //    Check consistency
                if ((formatPhotomA->numOfDetectorRegions != localFormat->numOfDetectorRegions) ||
                    (formatPhotomA->iXWidth != localFormat->iXWidth) || 
                    (formatPhotomA->iYWidth != localFormat->iYWidth))
                {
                    midiReportWarning (midiReportPtr, 
                        routine, __FILE__, __LINE__, "Expected format is incorrect");
                    *error = 1;
                    break;
                }

                /*    The Photometry files contribute to the computation of the 
                kappa matrix. Therefore here only formatPhotomA will be used */
                organiseKappa ("AOPEN", newFile, extNumOfImagingDataFile, fileNames, 
                    extNumOfImagingDataMask, maskFile, localFormat, formatInterfA,
                    compressedInterfA, compressedPhotomA, error);
                if (*error)    break;
                newFile = 0;    // So the next time through, will know not to start at frame 0
            }
            else if ((strcmp (localFormat->obsTech, "IMAGE,WINDOW,CHOPNOD") == 0) &&
                (strcmp(localFormat->shutterId,"BOPEN") == 0))
            {
                //    Check previous TYPE
                if (obsTech != PHOTOMB) newFile = 1;
                else newFile = 0;
                
                //    Set the TYPE flag
                obsTech = PHOTOMB;
                
                //    Check consistency
                if ((formatPhotomA->numOfDetectorRegions != localFormat->numOfDetectorRegions) ||
                    (formatPhotomA->iXWidth != localFormat->iXWidth) || 
                    (formatPhotomA->iYWidth != localFormat->iYWidth))
                {
                    midiReportWarning (midiReportPtr, 
                        routine, __FILE__, __LINE__, "Expected format is incorrect");
                    *error = 1;
                    break;
                }

                /*    The Photometry files contribute to the computation of the 
                kappa matrix. Therefore here only formatPhotomB will be used */
                organiseKappa ("BOPEN", newFile, extNumOfImagingDataFile, fileNames, 
                    extNumOfImagingDataMask, maskFile, localFormat, formatInterfB, 
                    compressedInterfB, compressedPhotomB, error);
                if (*error)    break;
                newFile = 0;    // So the next time through, will know not to start at frame 0
            }
            else
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Catg, Type or Tech");
                *error = 1;
                break;
            }
        }
        else
        {
            sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
            midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        }
        loopCount++;
    }

    //    Close files and release memory
    fclose (inFitsBatchPtr);
    free (cleanString);
    free (stringTemp);
    free (classification);
    freeImageFormat (localFormat);
    free (maskFile);

    //    Check if reduction has been carried out
   cpl_msg_info(cpl_func,"\nCompression status \n");
   cpl_msg_info(cpl_func,"------------------ \n");
    if (compressedInterfA->exists)cpl_msg_info(cpl_func,"Created Compressed Interferometry A Data\n");
    if (compressedInterfB->exists)cpl_msg_info(cpl_func,"Created Compressed Interferometry B Data\n");
    if (compressedPhotomA->exists)cpl_msg_info(cpl_func,"Created Compressed Photometry A Data\n");
    if (compressedPhotomB->exists)cpl_msg_info(cpl_func,"Created Compressed Photometry B Data\n");
   cpl_msg_info(cpl_func,"\n");

    fprintf (midiReportPtr, "\nCompression status \n");
    fprintf (midiReportPtr, "------------------ \n");
    if (compressedInterfA->exists) fprintf (midiReportPtr, "Created Compressed Interferometry A Data\n");
    if (compressedInterfA->exists) fprintf (midiReportPtr, "Created Compressed Interferometry B Data\n");
    if (compressedPhotomA->exists) fprintf (midiReportPtr, "Created Compressed Photometry A Data\n");
    if (compressedPhotomB->exists) fprintf (midiReportPtr, "Created Compressed Photometry B Data\n");
    fprintf (midiReportPtr, "\n");

    if (!(compressedInterfA->exists) || !(compressedInterfB->exists) || 
        !(compressedPhotomA->exists) || !(compressedPhotomB->exists) || *error)
    {
        *error = 1;
        sprintf (midiMessage, 
            "Cannot continue. Need the following compressed data: InterfA, InterfB, PhotomA, PhotomB");
        midiReportWarning (midiReportPtr, 
            routine, __FILE__, __LINE__, midiMessage);
        return;
    }
    
    //    Display images
    if (diagnostic > 1 && plotFile)
    {
        fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

        for (R = 0; R < formatInterfA->numOfRegionsToProcess; R++)
        {
            sprintf (fileName, "3dInterfADATA%d", R+2);
            sprintf (title, "Interferometry A DATA %d (Masked and Sky removed)", R+2);
            midiCreatePlotFile3D (fileName, title, "X", "Y", "Flux", 0, 
                compressedInterfA->image[R], formatInterfA->iXWidth, formatInterfA->iYWidth, "lines", "3");
        }

        for (R = 0; R < formatInterfB->numOfRegionsToProcess; R++)
        {
            sprintf (fileName, "3dInterfBDATA%d", R+2);
            sprintf (title, "Interferometry B DATA %d (Masked and Sky removed)", R+2);
            midiCreatePlotFile3D (fileName, title, "X", "Y", "Flux", 0, 
                compressedInterfB->image[R], formatInterfB->iXWidth, formatInterfB->iYWidth, "lines", "3");
        }

        for (R = 0; R < formatPhotomA->numOfRegionsToProcess; R++)
        {
            sprintf (fileName, "3dPhotomADATA%d", R+1);
            sprintf (title, "Photometry A DATA %d (Masked and Sky removed)", R+1);
            midiCreatePlotFile3D (fileName, title, "X", "Y", "Flux", 0, 
                compressedPhotomA->image[R], formatPhotomA->iXWidth, formatPhotomA->iYWidth, "lines", "3");
        }

        for (R = 0; R < formatPhotomB->numOfRegionsToProcess; R++)
        {
            sprintf (fileName, "3dPhotomBDATA%d", R+4);
            sprintf (title, "Photometry B DATA %d (Masked and Sky removed)", R+4);
            midiCreatePlotFile3D (fileName, title, "X", "Y", "Flux", 0, 
                compressedPhotomB->image[R], formatPhotomB->iXWidth, formatPhotomB->iYWidth, "lines", "3");
        }

        free (fileName);
        free (title);
    }

    return;
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:  organiseKappa
* Input/Output: See function arguments to avoid duplication
* Description:  Prepares SCI_PHOT Kappa files for compression. All files may be
*                split.
*
* History:
* 05-Dec-05     (csabet) Created
******************************************************************************/
void organiseKappa (
    const char                *shutter,                    // In: Whether AOPEN or BOPEN
    int                    newFile,                    // In: Flag indicating continuation
    int                    extNumOfImagingDataFile,    // In: Extention number
    MidiFiles            *fileNames,                    // In: Pointer to the MIDI file structure
    int                    extNumOfImagingDataMask,    // In: Extention number
    char                *maskFile,                    // In: Mask file
    ImageFormat            *localFormat,                // In: Local format size
    ImageFormat            *formatInterf,                // In: Interf size parameters
    CompressedData        *compressedInterf,            // Ou: Pointer to the compressed interferometry data structure
    CompressedData        *compressedPhotom,            // Ou: Pointer to the compressed photom A data structure
    int                    *error)                        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char        routine[] = "organiseKappa";
    int                frame0;

    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf(midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Initialise
    *error = 0;

    //    Compress data
    frame0 = compressKappa (shutter, fileNames->inFitsName, maskFile, 
        extNumOfImagingDataFile, extNumOfImagingDataMask, compressedInterf, 
        compressedPhotom, newFile, localFormat, formatInterf->numOfFrames, error);
    if (*error) return;

    //    Report compression status
    if (diagnostic)
    {
        sprintf (midiMessage, "\nWrote %d frames into Interf and Photom, starting at frame %d\nThus now %d frames total,"
            " expecting %d altogether.  Error=%d \n\n", localFormat->numOfFrames, frame0, 
            localFormat->numOfFrames+frame0, formatInterf->numOfFrames, *error);
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }
    
    //    Set the flags
    if ((frame0 + localFormat->numOfFrames) == formatInterf->numOfFrames)  // ALL have been read in
    {
        compressedInterf->exists = 1;    // Mark it as completed
        compressedPhotom->exists = 1;    // Mark it as completed
    }
                    
    return;
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*            VLTI MIDI Data Reduction Software
*
* Module name:    compressKappa
* Input/Output:    See function arguments to avoid duplication
* Description:    This routine compresses data files for kappa computation
*               iDispFringe points to a 3D structure. One dimension is time (frame), 
*                the second dimension is wave (produced by collapsing each X, Y image 
*                into one line) and the third dimension is the number of detector regions. 
*
*                Note: 
*                if AOPEN
*                ========
*                DATA1 = PHOT A
*                DATA2 = INTERF
*                DATA3 = INTERF
*                DATA4 = 0
*
*                if BOPEN
*                ========
*                DATA1 = 0
*                DATA2 = INTERF
*                DATA3 = INTERF
*                DATA4 = PHOT B
*
*                All 4 data regions are chopped
*
* History:
* 05-Dec-05        (csabet) Created
******************************************************************************/
int compressKappa (                            // Ou: Status. Returns number of first frame compressed; -1 if error
    const char            *shutter,                // In: Whether AOPEN or BOPEN
    char            *inFitsFile,            // In: Name of the input FITS file
    char            *maskFile,                // In: Name of the mask file
    int                extNumOfImagingDataFile,// In: Extension number of the IMAGING_DATA in input file
    int                extNumOfImagingDataMask,// In: Extension number of the IMAGING_DATA in mask file
    CompressedData    *compressedInterf,        // IO: Compressed Interf data
    CompressedData    *compressedPhotom,        // IO: Compressed Photom data
    int                newSet,                    // In: Flag indicating the arrival of a new set of FITS files
    ImageFormat        *format,                // In: Split file format 
    int                numOfFramesMax,            // In: Maximum number of frames
    int                *error)                    // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char        routine[] = "compressKappa";
    qfits_table        *pTable  = NULL, *pMask = NULL;
    short int        **inData, *inSteppingPhase;
    float            **inMask, accum, current,
                    *normalization; // Normalization factor. Becomes an array in X for the normalization of the 
                                    // compressedInterf data, valid for each subRegion then overwritten
    char            **inTARTYP = NULL, *tempStr, fitsColumnString[10], *dataName, *title=NULL, *fileString=NULL;
    double            *inTIME = NULL; 
    int                frame0 = - 1;     // Gets set to a valid number UNLESS routine terminates in error
    double            (*inLOCALOPD)[2] = NULL, (*inOPD)[2] = NULL; 
    static int        aprioriSteppingPhase, iF, channelSelected = 0;
    static double    zeroTime;
    int                i, k, *foundData, foundSteppingPhase = 0, indexSteppingPhase, scalingOffset, *indexData, 
                    *indexMask, maskWidthX, maskWidthY,    maskSubWindow, F, X, Y, R, 
                    *indexTARTYP, *foundTARTYP, startframe=0, frameOffset, indexOPD= -1, indexLOCALOPD= -1, 
                    indexTIME= -1, maxstep,
                    tartypMult=2;    // Signifies that there are 2 (instead of 1!) characters per tartyp value (only 
                                    // first is useful). In future, if the problem producing the MIDI files is solved, 
                                    // can be changed to 1 (only for newer files) or made variable (with some way to 
                                    // determine whether it is an "old" type file). Note: this problem really has 
                                    // nothing to do with Qfits, but with the (unintended) way the files are written.
    int                startRegion, endRegion, i2, fst, snd, found;
    
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    if (format->numOfDetectorRegions != 4)
    {
        *error = 1;
        sprintf (midiMessage, "Incorrect number of regions. Expected 4, found %d", format->numOfDetectorRegions);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        return (-1);
    }

    //    Determine region's start and end. See description
    if (strcmp (shutter, "AOPEN") == 0) 
    {
        startRegion = 0;
        endRegion = format->numOfDetectorRegions - 1;
    }
    else
    {
        startRegion = 1;
        endRegion = format->numOfDetectorRegions;
    }

    //    Allocate memory
    inData = (short int **) calloc (format->numOfDetectorRegions, sizeof (short int *));
    inTARTYP = (char **) calloc (format->numOfDetectorRegions, sizeof (char *));
    inMask = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));  // Now has each OUTPUT region allocated
    foundData = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    indexData = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    foundTARTYP = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    indexTARTYP = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    indexMask = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    normalization = (float *) calloc (format->iXWidth, sizeof (float));
    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Initialise stepping phase for a new set of FITS files
    if (newSet)
    {
        aprioriSteppingPhase = 0;
        iF = 0;
    }

    //    Make sure the sum of the split-files numOfFrames is not more than the allowed maximum
    if ((iF + format->numOfFrames) > numOfFramesMax)
        format->numOfFrames = numOfFramesMax - iF;

    //    Open IMAGING_DATA = INDEX 2
    pMask = qfits_table_open (maskFile, extNumOfImagingDataMask);
    if (!pMask)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load Mask's IMAGING_DATA");
        free (inData);
        free (inTARTYP);
        free (inMask);
        free (foundData);
        free (indexData);
        free (foundTARTYP);
        free (indexTARTYP);
        free (indexMask);
        free (normalization);
        free (dataName);
        *error = 1;
        return (-1);
    }
    pTable = qfits_table_open (inFitsFile, extNumOfImagingDataFile);
    if (!pTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load Data's IMAGING_DATA");
        qfits_table_close (pMask);
        free (inData);
        free (inTARTYP);
        free (inMask);
        free (foundData);
        free (indexData);
        free (foundTARTYP);
        free (indexTARTYP);
        free (indexMask);
        free (normalization);
        free (dataName);
        *error = 1;
        return (-1);
    }

    //    Get data table information
    for (R = startRegion; R < endRegion; R++)
    {
        foundData[R] = 0;
        indexData[R] = 0;
    }
    for (i = 0; i < pTable->nc; i++)
    {
        for (R = startRegion; R < endRegion; R++)
        {
            sprintf (dataName, "DATA%d", R+1);
            if (strcmp (pTable->col[i].tlabel, dataName) == 0)
            {
                foundData[R] = 1;
                indexData[R] = i;
                if (diagnostic) 
                {
                   cpl_msg_info(cpl_func,"Found 'DATA%d' at column %d in data file %s \n", R+1, i+1, inFitsFile);
                    fprintf(midiReportPtr, "Found 'DATA%d' at column %d in data file %s \n", R+1, i+1, inFitsFile);
                }
            }
            
            sprintf (dataName, "TARTYP%d", R+1);
            if (strcmp (pTable->col[i].tlabel, dataName) == 0)
            {
                foundTARTYP[R] = 1;
                indexTARTYP[R] = i;
                if (diagnostic) 
                {
                   cpl_msg_info(cpl_func,"Found 'TARTYP%d' at column %d in data file %s \n", R+1, i+1, inFitsFile);
                    fprintf(midiReportPtr, "Found 'TARTYP%d' at column %d in data file %s \n", R+1, i+1, inFitsFile);
                }
            }
        }
        if (strcmp (pTable->col[i].tlabel, "STEPPING_PHASE") == 0)
        {
            foundSteppingPhase = 1;
            indexSteppingPhase = i;
        }
        if (strcmp (pTable->col[i].tlabel, "OPD") == 0)
        {
            indexOPD = i;
        }
        if (strcmp (pTable->col[i].tlabel, "LOCALOPD") == 0)
        {
            indexLOCALOPD = i;
        }
        if (strcmp (pTable->col[i].tlabel, "TIME") == 0)
        {
            indexTIME = i;
        }
    }

    //    Now issue warnings
    if (foundSteppingPhase == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find STEPPING_PHASE in data FITS file");
        *error = 1;
    }
    if (indexOPD < 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find column for OPD in data FITS file");
        *error = 1;
    }
    if (indexLOCALOPD < 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find column for LOCALOPD in data FITS file");
        *error = 1;
    }
    if (indexTIME < 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find column for TIME in data FITS file");
        *error = 1;
    }
    for (R = startRegion; R < endRegion; R++)
    {
        if (foundData[R] == 0)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                "Cannot find requested DATA column in data FITS file");
            *error = 1;
        }
        if (foundTARTYP[R] == 0)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                "Cannot find requested TARTYP column in data FITS file");
            *error = 1;
        }
    }
    
    //    Get mask data table information
    for (R = startRegion; R < endRegion; R++)
    {
        foundData[R] = 0;
        indexMask[R] = 0;
    }
    for (i = 0; i < pMask->nc; i++)
    {
        for (R = startRegion; R < endRegion; R++)
        {
            sprintf (dataName, "DATA%d", R+1);
            if (strcmp (pMask->col[i].tlabel, dataName) == 0)
            {
                foundData[R] = 1;
                indexMask[R] = i;
                if (diagnostic) 
                {
                   cpl_msg_info(cpl_func,"Found 'DATA%d' at column %d in mask file %s \n", R+1, i+1, maskFile);
                    fprintf(midiReportPtr, "Found 'DATA%d' at column %d in mask file %s \n", R+1, i+1, maskFile);
                }
            }
        }
    }

    //    Now issue warnings
    for (R = startRegion; R < endRegion; R++)
    {
        if (foundData[R] == 0)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
                "Cannot find requested DATA column in mask FITS file");
           *error = 1;
        }
    }

    //    Get subwindow format for the mask file
    sprintf (fitsColumnString, "TDIM%d", indexMask[1]+1);    // Point it to the column of a valid region
    tempStr = qfits_query_ext (maskFile, fitsColumnString, extNumOfImagingDataMask);
    sscanf (tempStr, "'(%d,%d) '", &maskWidthX, &maskWidthY);
    maskSubWindow = maskWidthX * maskWidthY;
    if (diagnostic)cpl_msg_info(cpl_func,"Mask sub-window size = %d\n", maskSubWindow);
    if (diagnostic)cpl_msg_info(cpl_func,"Data sub-window size = %d\n", format->subWindowSize);
    fprintf (midiReportPtr, "Mask sub-window size = %d\n", maskSubWindow);
    fprintf (midiReportPtr, "Data sub-window size = %d\n", format->subWindowSize);
    if (maskSubWindow != format->subWindowSize)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Mask has incompatible sub window size");
        *error = 1;
    }

    //    Get MAXSTEP value
    tempStr = qfits_query_ext (inFitsFile, "MAXSTEP", extNumOfImagingDataFile);
    if (tempStr != NULL)
    { 
        if (diagnostic)cpl_msg_info(cpl_func,"MAXSTEP = %s\n", tempStr);
        fprintf( midiReportPtr, "MAXSTEP = %s\n", tempStr);
        sscanf (tempStr, "%d", &maxstep);
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read MAXSTEP");
        *error = 1;
    }

    //    Read column COL_STEPPING_PHASE and check compatibility
    inSteppingPhase = (short int*)qfits_query_column (pTable, indexSteppingPhase, NULL);
    if (inSteppingPhase[0] != ((aprioriSteppingPhase % format->framesPerScan) + 1))
    {
        sprintf (midiMessage, "Incorrect Stepping Phase. Expected %d to %d. Instead found %d to %d", 
            aprioriSteppingPhase+1, maxstep, inSteppingPhase[0], maxstep-1);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        *error = 1;
    }

    //    Release memory and exit this module if there has been a warning. No point going any further
    if (*error)
    {
        qfits_table_close (pTable);
        qfits_table_close (pMask);
        free (inData);
        free (inTARTYP);
        free (inMask);
        free (foundData);
        free (indexData);
        free (foundTARTYP);
        free (indexTARTYP);
        free (indexMask);
        free (inSteppingPhase);
        free (normalization);
        free (dataName);
        return (-1);
    }

    //    Read DATA and TARTYP for all regions from input files
    for (R = startRegion; R < endRegion; R++)
    {
        inData[R] = (short int*) qfits_query_column (pTable, indexData[R], NULL);
        inTARTYP[R] = (char *) qfits_query_column (pTable, indexTARTYP[R], NULL);
                
        found = 0;
        for (F = 0; F < format->numOfFrames; F++)
        {
            //    Correct the initial anomalies
            if (newSet)
            {
                if (inTARTYP[R][F*tartypMult] == 'U')
                    found = 1;
                if (!found)
                    inTARTYP[R][F*tartypMult] = 'U';
            }
            
            //    Load data and correct tartype problem
            if (strcmp (shutter, "AOPEN") == 0)
            {
                if (R == 0) 
                    (compressedPhotom->tarType)[iF+F] = inTARTYP[R][F*tartypMult];
                else if (R == 1) 
                    (compressedInterf->tarType)[iF+F] = inTARTYP[R][F*tartypMult];
            }
            else
            {
                if (R == 1) 
                    (compressedInterf->tarType)[iF+F] = inTARTYP[R][F*tartypMult];
                else if (R == 3)
                    (compressedPhotom->tarType)[iF+F] = inTARTYP[R][F*tartypMult];
            }
        }
    }

    //    Get TIME
    if(indexTIME >= 0)    // Valid ...
    {
        startframe = 0;
        inTIME = (double *) qfits_query_column (pTable, indexTIME, NULL);
        //    Stuff it into time, subtracting off the zero time
        if (iF == 0) zeroTime = inTIME[0];

        //    Fix a bug found in the data:
        if (iF == 0)
        {
            for (startframe = 0; startframe < ARB_NUM_OF_FRAMES; startframe++)
            {
                if ((zeroTime = inTIME[startframe]) >  1.0)    // Should be more like 53000 or something!
                break;
            }
        }
        if (startframe)
        {
            if (diagnostic) 
            {
               cpl_msg_info(cpl_func,"\nLOOK: frames 0 - %d had ZERO for their time field!!\n", startframe-1);
                fprintf(midiReportPtr, "\nLOOK: frames 0 - %d had ZERO for their time field!!\n", startframe-1);
            }
        }

        for (F = startframe; F < format->numOfFrames; F++)
        {
            compressedInterf->time[iF+F] = (float)(inTIME[F] - zeroTime);
            compressedPhotom->time[iF+F] = (float)(inTIME[F] - zeroTime);
            if (isnan (compressedInterf->time[iF+F]) || 
                isnan (compressedPhotom->time[iF+F]))
            {
                //    All channels and regions are affected
                for (i = 0; i < format->iXWidth; i++)
                {
                    compressedInterf->rejectList[i][iF+F] |= BSL_TIME_ERROR;
                    compressedPhotom->rejectList[i][iF+F] |= BSL_TIME_ERROR;
                }
                sprintf (midiMessage, "inTIME has an INVALID value at frame %d", iF+F);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Get LOCALOPD
    if (indexLOCALOPD >= 0)    // Valid ...
    {
        inLOCALOPD = (double (*)[2]) qfits_query_column (pTable, indexLOCALOPD, NULL);
        //    Add up the 2 delay lines, and stuff it into localOPD:
        for (F = 0; F < format->numOfFrames; F++)
        {
            compressedInterf->localOPD[iF+F] = (float)(inLOCALOPD[F][0] + inLOCALOPD[F][1]);
            compressedPhotom->localOPD[iF+F] = (float)(inLOCALOPD[F][0] + inLOCALOPD[F][1]);
            if (isnan (compressedInterf->localOPD[iF+F]) ||
                isnan (compressedPhotom->localOPD[iF+F]))
            {
                //    All channels and regions are affected
                for (i = 0; i < format->iXWidth; i++)
                {
                    compressedInterf->rejectList[i][iF+F] |= BSL_LOCALOPD_ERROR;
                    compressedPhotom->rejectList[i][iF+F] |= BSL_LOCALOPD_ERROR;
                }
                sprintf (midiMessage, "localOPD has an INVALID value at frame %d", iF+F);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Get OPD
    if (indexOPD >= 0)    // Valid ...
    {
        inOPD = (double (*)[2]) qfits_query_column (pTable, indexOPD, NULL);
        //    Add up the 2 delay lines, and stuff it into bigDL:
        for (F = 0; F < format->numOfFrames; F++)
        {
            (compressedInterf->bigDL)[iF+F] = (float)(inOPD[F][0] + inOPD[F][1]);
            (compressedPhotom->bigDL)[iF+F] = (float)(inOPD[F][0] + inOPD[F][1]);
            if (isnan (compressedInterf->bigDL[iF+F]) ||
                isnan (compressedPhotom->bigDL[iF+F]))
            {
                //    All channels and regions are affected
                for (i = 0; i < format->iXWidth; i++)
                {
                    compressedInterf->rejectList[i][iF+F] |= BSL_OPD_ERROR;
                    compressedPhotom->rejectList[i][iF+F] |= BSL_OPD_ERROR;
                }
                sprintf (midiMessage, "bigDL has an INVALID value at frame %d", iF+F); 
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Read all subregions from mask file
    for (R = startRegion; R < endRegion; R++)
    {
        inMask[R] = (float*) qfits_query_column (pMask, indexMask[R], NULL);

        //    Display mask data
        if (diagnostic && plotFile && newSet)
        {
            fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (fileString, "3dMaskDATA%d", R+1);
            sprintf (title, "3D Mask DATA %d", R+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                inMask[R], format->iXWidth, format->iYWidth, "lines", "3");
            free (fileString);
            free (title);
        }
    }

    //    Select valid channels
    if (newSet || !channelSelected)
    {
        selectChannels (startRegion, endRegion, format, inMask);
        channelSelected = 1;
    }

    //    Get the scaling offset
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (inFitsFile, dataName, extNumOfImagingDataFile);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            if (diagnostic) fprintf (midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }
    
    //    ACTUAL COMPRESSION OF DATA HERE
    for (R = startRegion; R < endRegion; R++)
    {
        //    Determine the normalization for each X (DISPERSED mode)
        for (X = 0; X < format->iXWidth; X++)
        {
            accum = 0.0F;
            for (Y = 0; Y < format->iYWidth; Y++)
                accum +=  inMask[R][Y * format->iXWidth + X];

            if (accum > 0.0)
                normalization[X] = 1.F/accum;
            else
                normalization[X] = 1.F;
        }

        for (F = 0; F < format->numOfFrames; F++)
        {
            frameOffset = F * format->subWindowSize;
            for (X = 0; X < format->iXWidth; X++)
            {
                accum = 0.0F;
                for (Y = 0; Y < format->iYWidth; Y++)
                {
                    k = Y * format->iXWidth + X;
                    i = frameOffset + k;

#ifdef SIMULATE_DATA                    
                    //    If suitable data is not available
                    if ((compressedInterf->tarType[F] == 'T') || (compressedPhotom->tarType[F] == 'T'))
                    {
                        if ((R == 1) || (R == 2))
                            inData[R][i] = 20000;
                        else
                            inData[R][i] = 30000;
                    }
                    else
                    {
                        if ((R == 1) || (R == 2))
                            inData[R][i] = 7000;
                        else
                            inData[R][i] = 10000;
                    }
#endif
                    //    If data is bad reject it
/*
                    if (isnan (inData[R][i]))
                    {
                        compressedInterf->rejectList[X][iF+F] |= BSL_DATA_ERROR;
                        compressedPhotom->rejectList[X][iF+F] |= BSL_DATA_ERROR;
                        sprintf (midiMessage, "inData has an INVALID value at frame %d", (iF+F));
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    }
                    else 
*/
                    {
                        current = (inData[R][i] + scalingOffset) * inMask[R][k];
                        accum += current;
                    }
                }
            
                //    Accumulate further for kappa computation
                if ((R == 0) && (strcmp (shutter, "AOPEN") == 0))
                {
                    (((compressedPhotom->iDispFringe)[R])[X])[iF+F] = 
                        accum * normalization[X];
                    (compressedPhotom->iFringe1)[iF+F] += accum;
                }
                if (R == 1)
                {
                    (((compressedInterf->iDispFringe)[R-1])[X])[iF+F] = 
                        accum * normalization[X];
                    (compressedInterf->iFringe1)[iF+F] += accum;
                    (compressedInterf->iFringe)[iF+F] += accum;
                }
                if (R == 2) 
                {
                    (((compressedInterf->iDispFringe)[R-1])[X])[iF+F] = 
                        accum * normalization[X];
                    (compressedInterf->iFringe2)[iF+F] += accum;
                    (compressedInterf->iFringe)[iF+F] -= accum;
                }
                if ((R == 3) && (strcmp (shutter, "BOPEN") == 0))
                {
                    (((compressedPhotom->iDispFringe)[R-3])[X])[iF+F] = 
                        accum * normalization[X];
                    (compressedPhotom->iFringe1)[iF+F] += accum;
                }
            }
        }

        //    Diagnostic
        if (diagnostic > 1)
        {
            for (F = 0; F < format->numOfFrames-maxstep; F++)
            {
                fst = F * format->subWindowSize;
                snd = (F+maxstep) * format->subWindowSize;
                if (compressedInterf->tarType[F] != 'U')
                {
                    for (Y = 0; Y < format->iYWidth; Y++)
                    {
                        for (X = 0; X < format->iXWidth; X++)
                        {
                            k = X * format->iYWidth + Y;
                            i = fst + k;
                            i2 = snd + k;
                            if ((compressedInterf->tarType[F] == 'T') && (compressedInterf->tarType[F+maxstep] == 'S'))
                                current = (inData[R][i] + scalingOffset) - (inData[R][i2] + scalingOffset);
                            else if ((compressedInterf->tarType[F] == 'S') && (compressedInterf->tarType[F+maxstep] == 'T'))
                                current = (inData[R][i2] + scalingOffset) - (inData[R][i] + scalingOffset);
                        
                            //    Load
                            if ((R == 0) && (strcmp (shutter, "AOPEN") == 0))
                                compressedPhotom->image[R][k] += (current * inMask[R][k]);
                            if (R == 1 || R == 2)
                                compressedInterf->image[R-1][k] += (current * inMask[R][k]);
                            if ((R == 3) && (strcmp (shutter, "BOPEN") == 0))
                                compressedPhotom->image[R-3][k] += (current * inMask[R][k]);
                        }
                    }
                }
            }
            F += maxstep;
        }
    }

    //    Save local parameters and indices
    aprioriSteppingPhase = inSteppingPhase[F-1];    // F-1 because of the above loop
    if (aprioriSteppingPhase == format->framesPerScan) aprioriSteppingPhase = 0;

    frame0 = iF;    // To output on return, below
    iF += format->numOfFrames;    // Set it correctly for the next call to this function

    //    Clean up now:
    for (R = 0; R < format->numOfDetectorRegions; R++) free (inData[R]);
    for (R = 0; R < format->numOfDetectorRegions; R++) free (inTARTYP[R]);
    for (R = 0; R < format->numOfDetectorRegions; R++) free (inMask[R]);
    if (pMask) qfits_table_close (pMask);
    if (pTable) qfits_table_close (pTable);
    if (inLOCALOPD) free(inLOCALOPD);
    if (inOPD) free(inOPD);
    if (inTIME) free(inTIME);
    free (inData);
    free (inTARTYP);
    free (inMask);
    free (inSteppingPhase);
    free (foundData);
    free (indexData);
    free (foundTARTYP);
    free (indexTARTYP);
    free (indexMask);
    free (normalization);
    free (dataName);

    return (frame0);
}
/*****************************************************************************/

