
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procDetRon.c
* Description:  Contains routines for processing the templates
*
* History:      
* 16-Jun-04     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <math.h>
#include <stdio.h>
#include <cpl.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "memoryHandling.h"
#include "statistics.h"
#include "createProdDetRon.h"
#include "cpl_image_stats.h"
#include "cpl_image_io.h"
#include "imageProcessing.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisTec.h"
#include "procDetRon.h"
#include "errorHandling.h"
#include "diagnostics.h"
#include "qfits.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procDetRon
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the detector read-out noise. This is the main routine for this
*                application.
*
* History:      
* 03-May-05     (csabet) Created
******************************************************************************/
void procDetRon (
    MidiFiles    *fileNames,    // In: Pointer to file names
    int            *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char      routine[] = "procDetRon";
    ImageFormat        *format=NULL;
    DetRonNoise        *noiseProfile=NULL;
    int                numOfFiles;
    FILE            *signaturePtr=NULL;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Write a signature
    signaturePtr = fopen ("MIDI_sig_ron.log", "w");
    fclose (signaturePtr);

    //    Reset status
    *error = 0;
    numOfFiles = 0;
    
    //    Allocate memory
    format = callocImageFormat ();
    
    analyseFitsDetRon (fileNames, format, &numOfFiles, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot analyse DETRON");
        freeImageFormat (format);
        return;
    }
    
    noiseProfile = callocDetRon (format);
    computeDetRon (numOfFiles, format, fileNames, noiseProfile, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot process DETRON");
        freeDetRon (noiseProfile);
        freeImageFormat (format);
        return;
    }
    
    createDetRonProd (fileNames, format, noiseProfile, error);
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create DETRON products");

    //    Release memory
    freeDetRon (noiseProfile);
    freeImageFormat (format);

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeDetRon
* Input/Output: See function arguments to avoid duplication
* Description:  Computes the detector read-out noise.
*                Compute the standard deviation pixel by pixel along the time sequence:
*                This method allows one to obtain a noise map and distribution of noise 
*                values. All    noise components contribute to the final value, including 
*                a frame to frame variation in the offset that is easily removed in the 
*                observations. To remove it, one can subtract from each frame the mean 
*                value, channel by channel:
*
*                IMPORTANT:
*                To obtain the noise in electrons, multiply the result in ADUs by the 
*                conversion factor (145 e-/ADU).
*
*                Summary: Find standard deviation of each pixel along the time (frame axis). 
*                Essentially you will end up with 320 X 240 number of standard deviations
*    
*
* History:      
* 23-Jun-04     (csabet) Created
******************************************************************************/
void computeDetRon (
    int            numOfFiles,        // In: Number of data files
    ImageFormat    *maxFormat,        // In: Pointer to the maximum image format
    MidiFiles    *fileNames,        // In: Pointer to the MIDI file structure
    DetRonNoise *noiseProfile,    // Ou: Pointer to the detector noise data structure
    int            *error)            // Ou: Error status
{
  
    // Local Declarations
    // ------------------
    const char  routine[] = "computeDetRon";
    char        *fileTemp, *classification, *firstFile;
    FILE           *inFitsBatchPtr;
    ImageFormat    *format;
    int            fileNumber, extNumOfImagingDataFile, subWindowSize;
    float        *arrayPixel;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    fileNumber = 0;
    noiseProfile->exists = 0;
    subWindowSize = maxFormat->iXWidth * maxFormat->iYWidth;

    //    Allocate memory
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    firstFile = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    format = callocImageFormat ();
    arrayPixel = (float *) calloc (subWindowSize * maxFormat->numOfFrames, sizeof (float));

    //    Initialise
    format->hasData = 0;
        
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot open input FITS file list\n                 No preprocessing has been carried out for this batch");
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        free (firstFile);
        free (arrayPixel);
        *error = 1;
        return;
    }

    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Processing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Processing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, error);
        if (*error)    break;

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, format, error);
            if (*error)    break;
        }
        else format->hasData = 0;

        //    Check if the file has data
        if (format->hasData)
        {
            //    Check Categ, Tech and Type and then compute the image size
            if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                (strcmp (format->obsTech, "IMAGE") == 0) &&
                (strcmp (format->obsType, "BIAS") == 0))
            {
                //    Increment file number but make sure it is not greater than the allocated
                fileNumber++;
                if (fileNumber > numOfFiles)
                {
                    *error = 1;
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Inconsistent file quantity");
                    break;
                }                

                //    Save the name of the first file
                if (fileNumber == 1) sprintf (firstFile, "%s", fileNames->inFitsName);
                
                //    Compress data
                compressDetRon (fileNumber, numOfFiles, maxFormat->numOfFrames, fileNames->inFitsName, 
                    extNumOfImagingDataFile, format, arrayPixel, noiseProfile, error);
                if (*error)
                {
                    sprintf (midiMessage, "Cannot compress data in %s", fileNames->inFitsName);
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    break;
                }
            }
            else
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "The above file is not suitable for this task");
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    
    //    Check if reduction has been carried out
    if (*error || (fileNumber != numOfFiles))
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create a noise pattern for this batch");
    else
    {
       cpl_msg_info(cpl_func,"\nDetector Readout Noise Inventry: \n");
       cpl_msg_info(cpl_func,"=============================== \n");
       cpl_msg_info(cpl_func,"   Expected number of data files  = %d\n", numOfFiles);
       cpl_msg_info(cpl_func,"   Number of data files processed = %d\n", fileNumber);
       cpl_msg_info(cpl_func,"\n");

        fprintf (midiReportPtr, "\nDetector Readout Noise Inventry: \n");
        fprintf (midiReportPtr, "=============================== \n");
        fprintf (midiReportPtr, "   Expected number of data files  = %d\n", numOfFiles);
        fprintf (midiReportPtr, "   Number of data files processed = %d\n", fileNumber);
        fprintf (midiReportPtr, "\n");
    }

    //    Close the file list
    fclose (inFitsBatchPtr);
    free (arrayPixel);

    //    Create Noise Pattern
    createNoisePattern (fileNumber, firstFile, maxFormat, noiseProfile, error);

    //    If no error set the exists flag
    if (!(*error)) noiseProfile->exists = 1;
    
    //    Release memory
    freeImageFormat (format);
    free (fileTemp);
    free (classification);    
    free (firstFile);
    
    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  compressDetRon
* Input/Output: See function arguments to avoid duplication
* Description:  Compresses the DETRON data
*
* History:      
* 25-Apr-06     (csabet) Created
******************************************************************************/
void compressDetRon (
    int            fileNumber,            // In: File number
    int            numOfFiles,            // In: Total number of files expected
    int            maxNumOfFrames,        // In: Maximum number of frames expected
    char        *fileName,            // In: Name of file to process
    int            extensionNumber,    // In: Extension number of the IMAGE_DATA
    ImageFormat    *format,            // In: Pointer to the image format
    float        *arrayPixel,        // In: Pointer to the accumulated data
    DetRonNoise    *noiseProfile,        // Ou: Pointer to the detector noise data structure
    int            *error)                // Ou: Error status

{
  
    //    Local Declarations
    //    ------------------
    const char  routine[] = "compressDetRon";
    qfits_table *pTable=NULL;
    short int   *inData;
    char        *tempStr, *dataName;
    int         i, k, frame, foundData = 0, indexData, pixel, subWindowSize, scalingOffset;
    static int    indexFrame;
    float        *arrayFrame, standDev, variance;

    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    subWindowSize = format->iXWidth * format->iYWidth;
    if (fileNumber == 1) indexFrame = 0;
    
    //    Open IMAGING_DATA = INDEX 2
    pTable = qfits_table_open (fileName, extensionNumber);
    if (!pTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load IMAGING_DATA");
        *error = 1;
        return;
    }
        
    //    Get data table information
    for (i = 0; i < pTable->nc; i++)
    {
        if (strcmp (pTable->col[i].tlabel, "DATA1") == 0)
        {
            foundData = 1;
            indexData = i;
        }
    }
    if (foundData == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find requested columns in data FITS file");
        qfits_table_close (pTable);
        *error = 1;
        return;
    }

    //    Read column DATA
    inData = (short int*) qfits_query_column (pTable, indexData, NULL); 

    //    Get the scaling offset
    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (fileName, dataName, extensionNumber);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            if (diagnostic) fprintf (midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }
    free (dataName);

    //    Now load or/and compress
    for (pixel = 0; pixel < subWindowSize; pixel++)
    {
        //    Along the time axis (frame)
        for (frame = 0; frame < format->numOfFrames; frame++)
        {
            //    Load each pixel data along the time axis
            i = frame * subWindowSize + pixel;
            k = (frame+indexFrame) * subWindowSize + pixel;
/*
            if (isnan (inData[i]))
            {
                sprintf (midiMessage, "inData has an INVALID value at frame %d", (indexFrame+frame));
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
*/
            noiseProfile->aveImage[pixel] += (float) (inData[i] + scalingOffset);
            arrayPixel[k] = (float) (inData[i] + scalingOffset);
        }
    }

    //    When all files have been read, compute an average 
    //    image and create an image from pixels standard deviation
    if (fileNumber == numOfFiles)
    {
        arrayFrame = (float *) calloc (maxNumOfFrames, sizeof (float));
        for (pixel = 0; pixel < subWindowSize; pixel++)
        {
            noiseProfile->aveImage[pixel] /= maxNumOfFrames;

            //    Along the time axis (frame)
            for (frame = 0; frame < maxNumOfFrames; frame++)
            {
                //    Load each pixel data along the time axis
                i = frame * subWindowSize + pixel;
                arrayFrame[frame] = arrayPixel[i];
            }

            //    Compute the standard deviation for the pixel array
            variance =  signalVariance (arrayFrame, 0, maxNumOfFrames, &standDev);
            noiseProfile->pattern[pixel] = standDev;
        }
        free (arrayFrame);
    }

    //    Set it correctly for the next call to this function
    indexFrame += format->numOfFrames;

    //    Release memory
    qfits_table_close (pTable);
    free (inData);

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createNoisePattern
* Input/Output: See function arguments to avoid duplication
* Description:  Creates a noise pattern made up of standard deviations of image pixels
*
* History:      
* 15-Jun-04     (csabet) Created
******************************************************************************/
void createNoisePattern (
    int            fileNumber,        // In: File number
    char        *inFile,        // In: Name of the input FITS file
    ImageFormat    *format,        // In: Pointer to the image format
    DetRonNoise    *noiseProfile,    // Ou: Pointer to the detector noise data structure
    int            *error)            // Ou: Error status

{
  
    //    Local Declarations
    //    ------------------
    const char  routine[] = "createNoisePattern";
    char        *title, *fileName;
    int         subWindowSize;
    cpl_image    *cplImage;
    
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    subWindowSize = format->iXWidth * format->iYWidth;
    
    //    Create an image FITS file
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (title, "AveImg");
    sprintf (fileName, "file %d", fileNumber);
    createFitsImage (fileName, title, inFile, format->iXWidth, format->iYWidth, noiseProfile->aveImage);
    free (fileName);
    free (title);

    //    Compute the median of the noise pattern
    cplImage = cpl_image_wrap_float (format->iXWidth, format->iYWidth, noiseProfile->pattern);
    noiseProfile->median = cpl_image_get_median (cplImage);
    cpl_image_unwrap (cplImage);

    if (diagnostic)cpl_msg_info(cpl_func,"   Median of the noise pattern  = %8.4f\n", noiseProfile->median);
    fprintf (midiReportPtr, "   Median of the noise pattern  = %8.4f\n", noiseProfile->median);

    //    Create plot file
    if (plotFile) midiCreatePlotFile3D ("3dSigmaMap", "Sigma Map", "X", "Y", "Sigma", 
        0, noiseProfile->pattern, format->iXWidth, format->iYWidth, "lines", "0");

    return; 
}
/*****************************************************************************/

