
/******************************************************************************
*******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procWaveCal.c
* Description:  Contains routines for processing the templates
*
* History:      
* 02-Jun-05     (csabet) created
*******************************************************************************
******************************************************************************/

/******************************************************************************
*   Compiler directives
******************************************************************************/

/******************************************************************************
*   Include files
******************************************************************************/
#include <math.h>
#include <stdio.h>
#include <cpl.h>
#include <errno.h>
#include <string.h>
#include "midiGlobal.h"
#include "midiLib.h"
#include "imageProcessing.h"
#include "memoryHandling.h"
#include "createProdWavCal.h"
#include "procWavCal.h"
#include "errorHandling.h"
#include "midiFitsUtility.h"
#include "fitsAnalysisTec.h"
#include "diagnostics.h"
#include "statistics.h"
#include "cpl_polynomial.h"
#include "qfits.h"
#include "midi_cplutils.h"
#include "midi_cplupgrade.h"

/**********************************************************
*   Constant definitions
**********************************************************/

/**********************************************************
*   Global Variables 
**********************************************************/

/*============================ C O D E    A R E A ===========================*/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  procWaveCal
* Input/Output: See function arguments to avoid duplication
* Description:  The main objective is to calibrate the channel against wavelength.
*                That is to assign the correct wavelength to the detector channels.
*
* History:      
* 02-Jun-05     (csabet) Created
******************************************************************************/
void procWaveCal (
    int            processing,    // In: Correlate = 1, Create = 2
    MidiFiles    *fileNames,    // In: Pointer to file names
    int            *error)        // Ou: Error status
{

    //    Local Declarations
    //    ------------------
    const char        routine[] = "procWaveCal";
    FILE            *signaturePtr=NULL, *filePtr=NULL;
    int                numOfFiles, numOfRecords;
    ImageFormat        *format=NULL;
    WaveCalibration    *waveCal=NULL;
    char            *fileString;
    float            record1, record2, record3, record4;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Write a signature
    signaturePtr = fopen ("MIDI_sig_wav.log", "w");
    fclose (signaturePtr);

    //    Reset status
    *error = 0;
    numOfFiles = 0;
    
    //    Allocate memory
    format = callocImageFormat ();
    
    analyseFitsWaveCal (fileNames, format, &numOfFiles, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot analyse WAVECAL");
        freeImageFormat (format);
        return;
    }

    //    If correlation is requested, check if database file exists
    if (processing == 1)
    {
        //    Check if correct frequency calibration file exists in the calibration database
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        sprintf (fileString , "%s_%s_%s.dat", fileNames->waveCalibName, format->grismId, format->beamCombiner);
        if ((filePtr = fopen (fileString, "r")) != NULL)    // It is in the database
        {
            //    Read from file and load into array
           cpl_msg_info(cpl_func,"\nChecking consistency of Wavelength Calibration file ... %s \n", fileString);
            fprintf (midiReportPtr, "\nChecking consistency of Wavelength Calibration file ... %s \n", fileString);
            numOfRecords = 0;
            if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
                while (fscanf (filePtr, "%f %f %f %f\n", &record1, &record2, &record3, &record4) != EOF) numOfRecords++;
            else
                while (fscanf (filePtr, "%f %f\n", &record1, &record2) != EOF) numOfRecords++;

            fclose (filePtr);
            if (numOfRecords == format->iXWidth)
            {
                *error = 0;
               cpl_msg_info(cpl_func,"Found a wavelength calibration file %s with %d pairs of floats \n", 
                    fileString, numOfRecords);
                fprintf (midiReportPtr, "Found a wavelength calibration file %s with %d pairs of floats \n", 
                    fileString, numOfRecords);
            }
            else
            {
                *error = 1;
                sprintf (midiMessage, "Inconsistent calibration data in %s. Nothing to correlate", fileString);
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
        else 
        {
            *error = 1;
            sprintf (midiMessage, "Cannot find calibration data file ... %s. Nothing to correlate", fileString);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        }
        free (fileString);
        
        if (*error)
        {
            freeImageFormat (format);
            return;
        }
    }
        
    //    Get the size of the Wavelength Calibration data
    numOfRecords = getWlCalib2Spectra (fileNames, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot get Wavelength Calibration data");
        freeImageFormat (format);
        return;
    }
    
    //    Get largest size
    if (numOfRecords < format->iXWidth) numOfRecords = format->iXWidth;
    
    //    Main task for wavelength calibration
    waveCal = callocWaveCal (numOfFiles, numOfRecords, format);
    computeWaveCal (processing, numOfFiles, fileNames, waveCal, error);
    if (*error)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot process WAVECAL");
        freeWaveCal (format, waveCal);
        freeImageFormat (format);
        return;
    }
    
    //    Create product files
    createWaveCalProd (processing, fileNames, format, waveCal, error);
    if (*error)    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create WAVECAL products");

    //    Release memory
    freeWaveCal (format, waveCal);
    freeImageFormat (format);


    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  computeWaveCal
* Input/Output: See function arguments to avoid duplication
* Description:  Performs Wavelength Calibration
*
* History:      
* 10-Jun-05     (csabet) Created
******************************************************************************/
void computeWaveCal (
    int                processing,    // In: Correlate = 1, Create = 2
    int                numOfFiles,    // In:    Number of data files
    MidiFiles        *fileNames,    // In: Pointer to the MIDI file structure
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    int                *error)        // Ou:    Error status
{

    //    Local Declarations
    //    ------------------
    const char  routine[] = "computeWaveCal";
    char                        *fileTemp, *classification;
    FILE                    *inFitsBatchPtr;
    ImageFormat             *format;
    int                     localError, fileNumber, extNumOfImagingDataFile;
    
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Allocate memory
    classification = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileTemp = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    format = callocImageFormat ();

    //    Reset status
    *error = 0;
    localError = 0;
    fileNumber = 0;
    waveCal->exists = 0;
    format->hasData = 0;
            
    //    Open the list of files
    if ((inFitsBatchPtr = fopen (fileNames->inFitsBatch, "r")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, 
            "Cannot open input FITS file list. No compression has been carried out for this batch");
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        *error = 1;
        return;
    }

    //    Loop through the files and analyse
    while (fgets (fileTemp, MAX_STRING_LENGTH, inFitsBatchPtr) != NULL)
    {
        sprintf (classification, "%s", "");
        sscanf (fileTemp, "%s%s", fileNames->inFitsName, classification);
        if (diagnostic)cpl_msg_info(cpl_func,"\n   Processing file   %s \n", fileNames->inFitsName);
        fprintf(midiReportPtr, "\n   Processing file   %s \n", fileNames->inFitsName);

        //    Get 'extNumOfImagingDataFile' extension number of IMAGING_DATA in input file
        extNumOfImagingDataFile  = findImagingDataExtension (fileNames->inFitsName, TAB_IMAGING_DATA, &localError);
        if (localError) 
        {
            *error = 1;
            break;
        }

        //    Get Image Format parameters
        if (extNumOfImagingDataFile > 0)
        {
            getImageFormat (fileNames->inFitsName, extNumOfImagingDataFile, format, &localError);
            if (localError) 
            {
                *error = 1;
                break;
            }
        }
        else format->hasData = 0;

        //    Check if the file has data
        if (format->hasData)
        {
            //    Check Categ, Tech and Type and then compute the image size
            if ((strcmp (format->obsCatg, "CALIB") == 0) &&
                (strcmp (format->obsTech, "SPECTRUM") == 0) &&
                (strcmp (format->obsType, "WAVE,SPECTEMPL") == 0))
            {
                //    Increment file number but make sure it is not greater than the allocated
                fileNumber++;
                if (fileNumber > numOfFiles)
                {
                    *error = 1;
                    sprintf (midiMessage, "Inconsistent number of files. Expected %d. Found %d", numOfFiles, fileNumber);
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    break;
                }
                
                //    Save file IDs
                sprintf (waveCal->shutterId[fileNumber-1], "%s", format->shutterId);
                sprintf (waveCal->filterName[fileNumber-1], "%s", format->filterName);
                
                //    Preprocess data
                compressWaveCal (fileNumber-1, fileNames->inFitsName, extNumOfImagingDataFile, format, waveCal, &localError);
                if (localError)
                {
                    *error = 1;
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot preprocess");
                    continue;
                }
            
                //    Validate data
                validateWaveCalData (fileNumber-1, fileNames->inFitsName, waveCal, format, &localError);
                if (localError)
                {
                    *error = 1;
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot validate data");
                    continue;
                }
            }
            else
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "The above file is not suitable for this task");
        }
        else
        {
            if (diagnostic)
            {
                sprintf (midiMessage, "No data tables in %s. Not processed", fileNames->inFitsName);
                midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            }
        }
    }

    //    Issue an interim error message
    if ((*error) || (fileNumber != numOfFiles))
    {
        sprintf (midiMessage, "Cannot do Wavelength Calibration. error=%d, numOfFiles=%d. Expected %d",
            *error, fileNumber, numOfFiles); 
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        fclose (inFitsBatchPtr);
        freeImageFormat (format);
        free (fileTemp);
        free (classification);
        return;
    }
    
    //    If so far all is well then carry out with the Calibration
    waveCal->exists = 1;
   cpl_msg_info(cpl_func,"\nWavelength Calibration Inventry: \n");
   cpl_msg_info(cpl_func,"=============================== \n");
   cpl_msg_info(cpl_func,"   Expected number of data files  = %d\n", numOfFiles);
   cpl_msg_info(cpl_func,"   Number of data files processed = %d\n", fileNumber);
   cpl_msg_info(cpl_func,"\n");
    fprintf (midiReportPtr, "\nWavelength Calibration Inventry: \n");
    fprintf (midiReportPtr, "=============================== \n");
    fprintf (midiReportPtr, "   Expected number of data files  = %d\n", numOfFiles);
    fprintf (midiReportPtr, "   Number of data files processed = %d\n", fileNumber);
    fprintf (midiReportPtr, "\n");

    //    Remove dark
    removeDark (format, waveCal, error);
        
    //    Perform Wavelength Calibration
    calibrateWaveChannels (processing, fileNames, waveCal, format, error);
    
    //    Close the file list
    fclose (inFitsBatchPtr);
    
    //    Release memory
    freeImageFormat (format);
    free (fileTemp);
    free (classification);    

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  compressWaveCal
* Input/Output: See function arguments to avoid duplication
* Description:  Compresses Wavelength Calibration raw data
*
* History:      
* 10-Jun-05     (csabet) Created
******************************************************************************/
void compressWaveCal (
    int                fileNumber,        // In: File number
    char            *inFitsName,    // In: Name of file to process
    int                extensionNumber,// In: Extension number of the IMAGE_DATA
    ImageFormat        *format,        // In: Pointer to the image format
    WaveCalibration    *waveCal,        // Ou: Pointer to the wavelength calibration
    int                *error)            // Ou: Error status

{
  
    //    Local Declarations
    //    ------------------
    const char  routine[] = "compressWaveCal";
    qfits_table    *pTable=NULL;
    short int    **inData;
    char        *inTarType=NULL, *tempStr, *dataName, lastTT;
    int            i, pixel, *foundData, scalingOffset, *indexData, numOfSkyFrames, numOfFramesUsed,
                frame, region, indexTarType, foundTarType, transitions, numOfUndefinedFrames,
                numOfBadFrames,    tartypMult;
       
    //    Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    foundTarType = 0;
    
    /*    Signifies that there are 2 (instead of 1!) characters per tartyp 
    value (only first is useful). In future, if the problem producing the MIDI 
    files is solved, can be changed to 1 (only for newer files) or made variable 
    (with some way to determine whether it is an "old" type file). Note: this problem 
    really has nothing to do with Qfits, but with the (unintended) way the files are written */
    tartypMult = 2;    
                    
    //    Allocate memory
    inData = (short int **) calloc (format->numOfDetectorRegions, sizeof (short int *));
    foundData = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    indexData = (int *) calloc (format->numOfDetectorRegions, sizeof (int));
    dataName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    
    //    Open IMAGING_DATA = INDEX 2
    pTable = qfits_table_open (inFitsName, extensionNumber);
    if (!pTable)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot load IMAGING_DATA");
        for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
        free (foundData);
        free (indexData);
        free (dataName);
        *error = 1;
        return;
    }
        
    //    Get data table information
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        foundData[region] = 0;
        indexData[region] = 0;
    }
    for (i = 0; i < pTable->nc; i++)
    {
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (dataName, "DATA%d", region+1);
            if (strcmp (pTable->col[i].tlabel, dataName) == 0)
            {
                foundData[region] = 1;
                indexData[region] = i;
                if (diagnostic)cpl_msg_info(cpl_func,"Found 'DATA%d' at column %d in data file %s \n", region+1, i+1, inFitsName);
                if (diagnostic) fprintf(midiReportPtr, "Found 'DATA%d' at column %d in data file %s \n", region+1, i+1, inFitsName);
            }
        }
        if (strcmp (pTable->col[i].tlabel, "TARTYP2") == 0)    // Now changed to TARTYP2, for all newer files
        {
            foundTarType = 1;
            indexTarType = i;
        }
    }

    //    Now issue warnings
    if (foundTarType == 0)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find TARTYP2 in data FITS file");
        *error = 1;
    }
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        if (foundData[region] == 0)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find requested DATA column in data FITS file");
            *error = 1;
        }
    }
    
    //    Release memory and exit this module if there has been a warning. No point going any further
    if (*error)
    {
        qfits_table_close (pTable);
        for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
        free (foundData);
        free (indexData);
        free (dataName);
        return;
    }

    //    Get TARTYP2
    inTarType = (char *) qfits_query_column (pTable, indexTarType, NULL);
    for (frame = 0; frame < format->numOfFrames; frame++)
        waveCal->tarType[frame] = inTarType[frame*tartypMult];     // New way around tartype problem (JM)

    //    Check if the data is chopped
    transitions = 0;
    lastTT = waveCal->tarType[0];
    for (frame = 0; frame < format->numOfFrames; frame++)
    {
        if (waveCal->tarType[frame] == lastTT) continue;
        transitions++;    // INCLUDES transitions to U etc.
        lastTT = waveCal->tarType[frame];
    }
    if ((format->chopped = (transitions > 10)))    // Apparently is a chopped file
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Wavelength Calibration data is chopped");
        *error = 1;
        qfits_table_close (pTable);
        for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
        free (foundData);
        free (indexData);
        free (dataName);
        return;
    }

    //    Read all regions from input files
    for (region = 0; region < format->numOfDetectorRegions; region++)
        inData[region] = (short int*) qfits_query_column (pTable, indexData[region], NULL);

    //    Get the scaling offset
    for (i = 14; i < 25; i++)
    {
        sprintf (dataName, "TZERO%d", i);
        tempStr = qfits_query_ext (inFitsName, dataName, extensionNumber);
        if (tempStr != NULL)
        {
            if (diagnostic)cpl_msg_info(cpl_func,"Scaling Offset = %s\n", tempStr);
            if (diagnostic) fprintf (midiReportPtr, "Scaling Offset = %s\n", tempStr);
            sscanf (tempStr, "%d", &scalingOffset);
            break;
        }
    }
    if (tempStr == NULL)
    {
        scalingOffset = 0;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot read Scaling Offset. It is set to 0");
    }

    //    Compression starts here
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        for (pixel = 0; pixel < format->subWindowSize; pixel++)
        {
            //    Along the time axis (frame)
            numOfUndefinedFrames = 0;
            numOfBadFrames = 0;
            numOfFramesUsed = 0;
            numOfSkyFrames = 0;
            if (strcmp (waveCal->filterName[fileNumber], "[ArIII]") == 0)
                (waveCal->ArIII->image)[region][pixel] = 0.0;
            else if (strcmp (waveCal->filterName[fileNumber], "[NeII]") == 0)
                (waveCal->NeII->image)[region][pixel] = 0.0;
            else if (strcmp (waveCal->filterName[fileNumber], "[SIV]") == 0)
                (waveCal->SIV->image)[region][pixel] = 0.0;
            else if (strcmp (waveCal->filterName[fileNumber], "WL-CALIB_2") == 0)
                (waveCal->foil->image)[region][pixel] = 0.0;
            else if (strcmp (waveCal->filterName[fileNumber], "OPEN") == 0)
                (waveCal->open->image)[region][pixel] = 0.0;
            else if (strcmp (waveCal->filterName[fileNumber], "CLOSED") == 0)
                (waveCal->dark->image)[region][pixel] = 0.0;
            else
            {
                midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Filter ID");
                *error = 1;
                qfits_table_close (pTable);
                for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
                free (foundData);
                free (indexData);
                free (dataName);
                return;
            }
            
            for (frame = 0; frame < format->numOfFrames; frame++)
            {
                //    Only consider Target, "T" and Sky, "S"
                if (waveCal->tarType[frame] == 'U')
                {
                    if (diagnostic && !pixel) 
                    {
                        sprintf (midiMessage, "Found undefined, 'U'  'TARTYP2' at frame %d of DATA%d in %s", 
                            frame+1, region+1, inFitsName);
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                    }
                    numOfUndefinedFrames++;
                    continue;
                }
                
                //    Load each pixel data along the time axis
                i = frame * format->subWindowSize + pixel;
/*
                if (isnan (inData[region][i]))
                {
                    numOfBadFrames++;
                    sprintf (midiMessage, "Found bad pixel %d of DATA%d in %s", i, region, inFitsName);
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
                }
                else
*/
                {
                    if (waveCal->tarType[frame] == 'S') numOfSkyFrames++;
                    numOfFramesUsed++;

                    if (strcmp (waveCal->filterName[fileNumber], "[ArIII]") == 0)
                        (waveCal->ArIII->image)[region][pixel] += (float) (inData[region][i]);
                    else if (strcmp (waveCal->filterName[fileNumber], "[NeII]") == 0)
                        (waveCal->NeII->image)[region][pixel] += (float) (inData[region][i]);
                    else if (strcmp (waveCal->filterName[fileNumber], "[SIV]") == 0)
                        (waveCal->SIV->image)[region][pixel] += (float) (inData[region][i]);
                    else if (strcmp (waveCal->filterName[fileNumber], "WL-CALIB_2") == 0)
                        (waveCal->foil->image)[region][pixel] += (float) (inData[region][i]);
                    else if (strcmp (waveCal->filterName[fileNumber], "OPEN") == 0)
                        (waveCal->open->image)[region][pixel] += (float) (inData[region][i]);
                    else if (strcmp (waveCal->filterName[fileNumber], "CLOSED") == 0)
                        (waveCal->dark->image)[region][pixel] += (float) (inData[region][i]);
                    else
                    {
                        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Filter ID");
                        *error = 1;
                        qfits_table_close (pTable);
                        for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
                        free (foundData);
                        free (indexData);
                        free (dataName);
                        return;
                    }
                }
            }

            //    Compute average image
            if (diagnostic && !pixel)
            {
               cpl_msg_info(cpl_func,"\nData Statistics \n");
               cpl_msg_info(cpl_func,"--------------- \n");
               cpl_msg_info(cpl_func,"Number of frames used in DATA%d      = %d \n", region+1, numOfFramesUsed);
               cpl_msg_info(cpl_func,"Number of Sky frames in DATA%d       = %d \n", region+1, numOfSkyFrames);
               cpl_msg_info(cpl_func,"Number of Bad frames in DATA%d       = %d \n", region+1, numOfBadFrames);
               cpl_msg_info(cpl_func,"Number of Undefined frames in DATA%d = %d \n", region+1, numOfUndefinedFrames);
                fprintf (midiReportPtr, "\nData Statistics \n");
                fprintf (midiReportPtr, "--------------- \n");
                fprintf (midiReportPtr, "Number of frames used in DATA%d      = %d \n", region+1, numOfFramesUsed);
                fprintf (midiReportPtr, "Number of Sky frames in DATA%d       = %d \n", region+1, numOfSkyFrames);
                fprintf (midiReportPtr, "Number of Bad frames in DATA%d       = %d \n", region+1, numOfBadFrames);
                fprintf (midiReportPtr, "Number of Undefined frames in DATA%d = %d \n", region+1, numOfUndefinedFrames);
            }
            if (numOfFramesUsed)
            {
                if (strcmp (waveCal->filterName[fileNumber], "[ArIII]") == 0)
                {
                    (waveCal->ArIII->image)[region][pixel] /= numOfFramesUsed;
                    (waveCal->ArIII->image)[region][pixel] += scalingOffset;
                }
                else if (strcmp (waveCal->filterName[fileNumber], "[NeII]") == 0)
                {
                    (waveCal->NeII->image)[region][pixel] /= numOfFramesUsed;
                    (waveCal->NeII->image)[region][pixel] += scalingOffset;
                }
                else if (strcmp (waveCal->filterName[fileNumber], "[SIV]") == 0)
                {
                    (waveCal->SIV->image)[region][pixel] /= numOfFramesUsed;
                    (waveCal->SIV->image)[region][pixel] += scalingOffset;
                }
                else if (strcmp (waveCal->filterName[fileNumber], "WL-CALIB_2") == 0)
                {
                    (waveCal->foil->image)[region][pixel] /= numOfFramesUsed;
                    (waveCal->foil->image)[region][pixel] += scalingOffset;
                }
                else if (strcmp (waveCal->filterName[fileNumber], "OPEN") == 0)
                {
                    (waveCal->open->image)[region][pixel] /= numOfFramesUsed;
                    (waveCal->open->image)[region][pixel] += scalingOffset;
                }
                else if (strcmp (waveCal->filterName[fileNumber], "CLOSED") == 0)
                {
                    (waveCal->dark->image)[region][pixel] /= numOfFramesUsed;
                    (waveCal->dark->image)[region][pixel] += scalingOffset;
                }
                else
                {
                    midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Filter ID");
                    *error = 1;
                    qfits_table_close (pTable);
                    for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
                    free (foundData);
                    free (indexData);
                    free (dataName);
                    return;
                }
            }
        }
    }

    //    Clean up now:
    for (region = 0; region < format->numOfDetectorRegions; region++) free (inData[region]);
    if (pTable) qfits_table_close (pTable);
    if (inTarType) free(inTarType);
    free (inData);
    free (foundData);
    free (indexData);
    free (dataName);

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  validateWaveCalData
* Input/Output: See function arguments to avoid duplication
* Description:  Validates the input data
*
* History:      
* 13-Jun-05     (csabet) Created
******************************************************************************/
void validateWaveCalData (
    int                fileNumber,        // In: File number
    char            *fileName,        // In: Input FITS name
    WaveCalibration    *compressed,    // In: Compressed data
    ImageFormat        *format,        // In: File format 
    int                *error)            // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char  routine[] = "validateWaveCalData";
    int            region;
    char        *title=NULL, *fileString=NULL;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    
    //    Allocate memory
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Analyse Raw Images
    if (strcmp (compressed->filterName[fileNumber], "[ArIII]") == 0)
    {
        //    Save file name
        sprintf (compressed->ArIII->fileName, "%s", fileName);

        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            
            //    Create Raw Image FITS files
            if (diagnostic)
            {
                sprintf (fileString , "region %d", region+1);
                sprintf (title , "AveRawImgArIIIDATA%d", region+1);
                createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                    (compressed->ArIII->image)[region]);
    
                if (plotFile)
                {
                    sprintf (fileString, "3dAveRawImgArIIIDATA%d", region+1);
                    sprintf (title, "[ArIII] Raw, averaged along time, Region %d", region+1);
                    midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                        (compressed->ArIII->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
                }
            }
        }
    }
    else if (strcmp (compressed->filterName[fileNumber], "[NeII]") == 0)
    {
        //    Save file name
        sprintf (compressed->NeII->fileName, "%s", fileName);

        for (region = 0; region < format->numOfDetectorRegions; region++)
        {            
            //    Create Raw Image FITS files
            if (diagnostic)
            {
                sprintf (fileString , "region %d", region+1);
                sprintf (title , "AveRawImgNeIIDATA%d", region+1);
                createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                    (compressed->NeII->image)[region]);
    
                if (plotFile)
                {
                    sprintf (fileString, "3dAveRawImgNeIIDATA%d", region+1);
                    sprintf (title, "[NeII] Raw, averaged along time, Region %d", region+1);
                    midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                        (compressed->NeII->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
                }
            }
        }
    }
    else if (strcmp (compressed->filterName[fileNumber], "[SIV]") == 0)
    {
        //    Save file name
        sprintf (compressed->SIV->fileName, "%s", fileName);
            
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            //    Create Raw Image FITS files
            if (diagnostic)
            {
                sprintf (fileString , "region %d", region+1);
                sprintf (title , "AveRawImgSIVDATA%d", region+1);
                createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                    (compressed->SIV->image)[region]);
    
                if (plotFile)
                {
                    sprintf (fileString, "3dAveRawImgSIVDATA%d", region+1);
                    sprintf (title, "[SIV] Raw, averaged along time, Region %d", region+1);
                    midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                        (compressed->SIV->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
                }
            }
        }
    }
    else if (strcmp (compressed->filterName[fileNumber], "WL-CALIB_2") == 0)
    {
        //    Save file name
        sprintf (compressed->foil->fileName, "%s", fileName);
            
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            //    Create Raw Image FITS files
            if (diagnostic)
            {
                sprintf (fileString , "region %d", region+1);
                sprintf (title , "AveRawImgFoilDATA%d", region+1);
                createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                    (compressed->foil->image)[region]);
    
                if (plotFile)
                {
                    sprintf (fileString, "3dAveRawImgFoilDATA%d", region+1);
                    sprintf (title, "Foil Raw, averaged along time, Region %d", region+1);
                    midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                        (compressed->foil->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
                }
            }
        }
    }
    else if (strcmp (compressed->filterName[fileNumber], "OPEN") == 0)
    {
        //    Save file name
        sprintf (compressed->open->fileName, "%s", fileName);
            
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            //    Create Raw Image FITS files
            if (diagnostic)
            {
                sprintf (fileString , "region %d", region+1);
                sprintf (title , "AveRawImgOpenDATA%d", region+1);
                createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                    (compressed->open->image)[region]);
    
                if (plotFile)
                {
                    sprintf (fileString, "3dAveRawImgOpenDATA%d", region+1);
                    sprintf (title, "Open Raw, averaged along, Region %d", region+1);
                    midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                        (compressed->open->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
                }
            }
        }
    }
    else if (strcmp (compressed->filterName[fileNumber], "CLOSED") == 0)
    {
        //    Save file name
        sprintf (compressed->dark->fileName, "%s", fileName);
            
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            //    Create Raw Image FITS files
            if (diagnostic)
            {
                sprintf (fileString , "region %d", region+1);
                sprintf (title , "AveRawImgDarkDATA%d", region+1);
                createFitsImage (fileString, title, fileName, format->iXWidth, format->iYWidth, 
                    (compressed->dark->image)[region]);
    
                if (plotFile)
                {
                    sprintf (fileString, "3dAveRawImgDarkDATA%d", region+1);
                    sprintf (title, "Dark Raw, averaged along time, Region %d", region+1);
                    midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                        (compressed->dark->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
                }
            }
        }
    }
    else
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Unknown Filter ID");
        *error = 1;
        free (title);
        free (fileString);
        return;
    }


    //    Release memory
    free (title);
    free (fileString);
    
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getWlCalib2Spectra
* Input/Output: See function arguments to avoid duplication
* Description:  Get WL-CALIB_2 spectra. Returns -1 if error
*
* History:      
* 20-Jun-05     (csabet) Created
******************************************************************************/
int getWlCalib2Spectra (
    MidiFiles    *fileNames,    // In: Pointer to file names
    int            *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char  routine[] = "getWlCalib2Spectra";
    FILE        *wlFilePtr=NULL;
    int            numOfRecords;
    char        *filePath, *tempString;
    float        elem1, elem2;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);
    
    //    Reset status
    *error = 0;
    numOfRecords = 0;

    //    Allocate memory
    filePath = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    tempString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Check the tracing record for WL-CALIB_2 foil
    //    --------------------------------------------
    sprintf (filePath, "%swl-calib_2.dat", fileNames->calibDbDir);
    if ((wlFilePtr = fopen (filePath, "r")) == NULL)
    {
        sprintf (midiMessage, "Cannot Open %s", filePath);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        free (filePath);
        free (tempString);
        *error = 1;
        return (-1);
    }
    //    Ignore first record which gives the title of the data file
    if (fgets (tempString, MAX_STRING_LENGTH, wlFilePtr) != NULL)
    sprintf (midiMessage, "Counting elements in %s", tempString);
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    while (fscanf (wlFilePtr, "%f %f\n", &elem1, &elem2) != EOF) 
        numOfRecords++;
    sprintf (midiMessage, "WL-CALIB_2 has %d pairs of 'float'(s)", numOfRecords);
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    fclose (wlFilePtr);

    //    Release memory
    free (filePath);
    free (tempString);
    
    return (numOfRecords); 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  removeDark
* Input/Output: See function arguments to avoid duplication
* Description:  Extracts all the necessary resources for calibration
*
* History:      
* 14-June-05    (csabet) Created
******************************************************************************/
void removeDark (
    ImageFormat        *format,    // In: File format 
    WaveCalibration    *waveCal,    // In: Compressed data
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "removeDark";
    int            region, pixel;
    char        *title=NULL, *fileString=NULL;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Allocate memory
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

    //    Get images with dark subtracted
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        for (pixel = 0; pixel < format->subWindowSize; pixel++)
        {
            (waveCal->ArIII->image)[region][pixel] -= (waveCal->dark->image)[region][pixel];
            (waveCal->NeII->image)[region][pixel] -= (waveCal->dark->image)[region][pixel];
            (waveCal->SIV->image)[region][pixel] -= (waveCal->dark->image)[region][pixel];
            (waveCal->foil->image)[region][pixel] -= (waveCal->dark->image)[region][pixel];
            (waveCal->open->image)[region][pixel] -= (waveCal->dark->image)[region][pixel];
        }
    }
        
    //    Create Image FITS files
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        sprintf (fileString , "region %d", region+1);
        sprintf (title , "AveImgArIIIDATA%d", region+1);
        createFitsImage (fileString, title, waveCal->ArIII->fileName, format->iXWidth, format->iYWidth, 
            (waveCal->ArIII->image)[region]);
    }

    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        sprintf (fileString , "region %d", region+1);
        sprintf (title , "AveImgNeIIDATA%d", region+1);
        createFitsImage (fileString, title, waveCal->NeII->fileName, format->iXWidth, format->iYWidth, 
            (waveCal->NeII->image)[region]);
    }

    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        sprintf (fileString , "region %d", region+1);
        sprintf (title , "AveImgSIVDATA%d", region+1);
        createFitsImage (fileString, title, waveCal->SIV->fileName, format->iXWidth, format->iYWidth, 
            (waveCal->SIV->image)[region]);
    }
    
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        sprintf (fileString , "region %d", region+1);
        sprintf (title , "AveImgFoilDATA%d", region+1);
        createFitsImage (fileString, title, waveCal->foil->fileName, format->iXWidth, format->iYWidth, 
            (waveCal->foil->image)[region]);
    }
    
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        sprintf (fileString , "region %d", region+1);
        sprintf (title , "AveImgOpenDATA%d", region+1);
        createFitsImage (fileString, title, waveCal->open->fileName, format->iXWidth, format->iYWidth, 
            (waveCal->open->image)[region]);
    }

    //    Create 3D plots
    if (plotFile)
    {
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (fileString, "3dAveImgArIIIDATA%d", region+1);
            sprintf (title, "[ArIII] Dark removed, averaged along time, Region %d", region+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                (waveCal->ArIII->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
        }

        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (fileString, "3dAveImgNeIIDATA%d", region+1);
            sprintf (title, "[NeII] Dark removed, averaged along time, Region %d", region+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                (waveCal->NeII->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
        }

        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (fileString, "3dAveImgSIVDATA%d", region+1);
            sprintf (title, "[SIV] Dark removed, averaged along time, Region %d", region+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                (waveCal->SIV->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
        }

        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (fileString, "3dAveImgFoilDATA%d", region+1);
            sprintf (title, "Foil Dark removed, averaged along time, Region %d", region+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                (waveCal->foil->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
        }

        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (fileString, "3dAveImgOpenDATA%d", region+1);
            sprintf (title, "Open Dark removed, averaged along time, Region %d", region+1);
            midiCreatePlotFile3D (fileString, title, "X", "Y", "Flux", 0, 
                (waveCal->open->image)[region], format->iXWidth, format->iYWidth, "lines", "3");
        }
    }
        
    //    Release memory
    free (title);
    free (fileString);

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  calibrateWaveChannels
* Input/Output: See function arguments to avoid duplication
* Description:  Perform the final wavelength calibration. 
*                1.    From the three filter files get three wavelengths for three channels.
*                    A linear fit gives the function (table) Wavelength [X], X = 0, N
*                    where N is the number of channels
*
*               Filter  Pk Freq.        Pk Wavelength
*                ======================================
*                NeII    2.34297e+13 Hz  12.7955  micron
*                SIV     2.85818e+13 Hz  10.489   micron
*                ArIII   3.33005e+13 Hz   9.00272 micron
*
*                2.     From the polycarbonate foil template, get the function (table) 
*                    Flux [x], x = 0, N,    where N is the number of channels.
*
*                3.    From the manufacturer's table get the function (table) 
*                    Flux [l], l = 0, W,    where W is the number of wavelength positions
*
* History:      
* 14-June-05    (csabet) 
******************************************************************************/
void calibrateWaveChannels (
    int                processing,    // In: Correlate = 1, Create = 2
    MidiFiles        *fileNames,    // In: Pointer to file names
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat        *format,    // In: Pointer to the image format
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "calibrateWaveChannels";

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Fitting of Filter Spectra
    fitFilterSpectra (waveCal, format, error);

    //    Get approximate channel assignment
    calibrateWaveFromNBF (waveCal, format, error);
    if (*error) return;
        
    //    Fitting of Foil Spectra frequency
    fitFoilSpectra (fileNames, waveCal, format, error);
    
    //    Fitting of Open Spectra
    fitOpenSpectra (error);
    
    //    Create frequency calibration database
    if (processing == 2) createWaveCalibDB (fileNames, waveCal, format, error);
    else correlateWaveCalibDB (fileNames, waveCal, format, error);
    
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  calibrateWaveFromNBF
* Input/Output: See function arguments to avoid duplication
* Description:  Creates an approximate wavelength calibration file based on the
*                Narrow Band Filter (NBF) spectra only
*
* History:      
* 08-Sep-05    (csabet)
******************************************************************************/
void calibrateWaveFromNBF (
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat        *format,    // In: Pointer to the image format
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "calibrateWaveFromNBF";
    int            region, *channel, chA, chN, chS, numOfNBF, order;
    float        *wave, *chFloat;
    char        *fileString, *title;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    numOfNBF = 3;

    //    Allocate memory
    channel = (int *) calloc (numOfNBF, sizeof (int));
    wave = (float *) calloc (numOfNBF, sizeof (float));
    
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        //    Sort channels
        channel[0] = (waveCal->ArIII->xCoord)[region];
        channel[1] = (waveCal->NeII->xCoord)[region];
        channel[2] = (waveCal->SIV->xCoord)[region];
        chA = (waveCal->ArIII->xCoord)[region];
        chN = (waveCal->NeII->xCoord)[region];
        chS = (waveCal->SIV->xCoord)[region];
        signalSortInt (channel, 0, numOfNBF);

        //    Correct alignment
        if (channel[0] == chA) wave[0] = (waveCal->ArIII->wavelength)[region];
        else if (channel[0] == chN) wave[0] = (waveCal->NeII->wavelength)[region];
        else wave[0] = (waveCal->SIV->wavelength)[region];
        
        if (channel[1] == chA) wave[1] = (waveCal->ArIII->wavelength)[region];
        else if (channel[1] == chN) wave[1] = (waveCal->NeII->wavelength)[region];
        else wave[1] = (waveCal->SIV->wavelength)[region];
        
        if (channel[2] == chA) wave[2] = (waveCal->ArIII->wavelength)[region];
        else if (channel[2] == chN) wave[2] = (waveCal->NeII->wavelength)[region];
        else wave[2] = (waveCal->SIV->wavelength)[region];
        
        if (plotFile)
        {
            chFloat = (float *) calloc (3, sizeof (float));
            chFloat[0] = (float) channel[0];
            chFloat[1] = (float) channel[1];
            chFloat[2] = (float) channel[2];
            fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (fileString , "NarrowBandFilterSectionalProfileDATA%d", region+1);
            sprintf (title , "Narrow Band Filter Sectional Profile Region %d", region+1);
            midiCreatePlotFile2D2P (fileString, title, "Detector Channel", "Wavelength in micron", 0, chFloat, wave, 0, 3, 1);
            free (fileString);
            free (title);
            free (chFloat);
        }

        //    Calibrate using a polynomial fit of order 
       cpl_msg_info(cpl_func,"\nCalibrating wavelength for region %d \n", region+1);
       cpl_msg_info(cpl_func,"--------------------------------- \n");
        fprintf (midiReportPtr, "\nCalibrating wavelength for region %d \n", region+1);
        fprintf (midiReportPtr, "--------------------------------- \n");
        order = 22;
        midiPolyFit (order, channel, wave, numOfNBF, format->iXWidth, waveCal->calibratedWave[region], error);
        if (*error)
        {
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot Calibrate");
            free (channel);
            free (wave);
            return;
        }
        
        if (plotFile)
        {
            fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (fileString , "CalibrationUsingPoly%dFitDATA%d", order/10, region+1);
            sprintf (title , "Calibration using a Polynomial fit of order %d for Region %d", order/10, region+1);
            midiCreatePlotFile2D (fileString, title, "Detector Channel", "Wavelength in micron", 
                0, waveCal->calibratedWave[region], 0, format->iXWidth, 1, 0);
            free (fileString);
            free (title);
        }
    }
    
    //    Release memory
    free (channel);
    free (wave);
    
    return; 
}
/*****************************************************************************/




/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  midiPolyFit
* Input/Output: See function arguments to avoid duplication
* Description:  A polynomial fit of various orders. We use two simple (fast)
*                but specific routines together with two general methods as follows:
*                10        Simple and fast first order (for diagnostics only)
*                20        Simple and fast 2nd order deterministic (for diagnostics only)
*                11        First order based on minimising chi-squared. Returns full set of statistics
*                22        Second order returning Mean Squared Error only
*
* History:      
* 14-June-05    (csabet)
******************************************************************************/
void midiPolyFit (
    int        order,        // In: Order (see above)
    int        *xCoord,    // In: Array of elements along X
    float    *yCoord,    // In: array of elements along Y
    int        numOfIn,    // In: Number of input points
    int        numOfOut,    // In: Number of output points
    float    *yOut,        // Ou: Pointer to the output result
    int        *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char        routine[] = "midiPolyFit";
    float            x1, x1s, x2, x2s, x3, x3s, y1, y2, y3, d, d1, d2, d3, 
                    coeff1, coeff2, coeff3, q, chi2, siga, sigb, sigc, sigData;
    cpl_polynomial  *coeffPoly;
    cpl_vector        *xPoly, *yPoly;
    int                i, degPoly, sigDataAvailable;
    cpl_size  power=0;
    double            *valuePloy,    *positionPoly, mse;


    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    coeff1=coeff2=coeff3=0.0;
    siga=sigb=sigc=chi2=q=mse=-1;

    //    Exception
    if (order == 10)
    {
       cpl_msg_info(cpl_func,"Fitting %d points to %d points using Fast-Simple 1st order polynomial \n", numOfIn, numOfOut);
        fprintf (midiReportPtr, "Fitting %d points to %d points using Fast-Simple 1st order polynomial \n", numOfIn, numOfOut);

        //    Assign short form
        x1 = xCoord[0];
        x2 = xCoord[1];
        x3 = xCoord[2];
        y1 = yCoord[0];
        y2 = yCoord[1];
        y3 = yCoord[2];
    
        //    Coefficients for a line going through extreme points are
        coeff1 = y1 - x1 * ((y3 - y1) / (x3 - x1));
        coeff2 = (y3 - y1) / (x3 - x1);
        if (diagnostic)cpl_msg_info(cpl_func,"Initial coefficients are: %f %f \n", coeff1, coeff2);
        if (diagnostic) fprintf (midiReportPtr, "Initial coefficients are: %f %f \n", coeff1, coeff2);
        
        //    Distance of middle point is
        d = fabs ((coeff2 * x2 - y2 + coeff1) / sqrt (coeff2 * coeff2 + 1));
        
        //    Update coeff1 for half the distance
        coeff1 = 0.5 * d * sqrt(coeff2 * coeff2 + 1) - coeff2 * x2 + y2;
    }
    else if (order == 20)
    {
       cpl_msg_info(cpl_func,"Fitting %d points to %d points using Fast-Simple 2nd order polynomial \n", numOfIn, numOfOut);
        fprintf (midiReportPtr, "Fitting %d points to %d points using Fast-Simple 2nd order polynomial \n", numOfIn, numOfOut);

        //    Assign short form
        x1 = xCoord[0];
        x2 = xCoord[1];
        x3 = xCoord[2];
        x1s = xCoord[0] * xCoord[0];
        x2s = xCoord[1] * xCoord[1];
        x3s = xCoord[2] * xCoord[2];
        y1 = yCoord[0];
        y2 = yCoord[1];
        y3 = yCoord[2];
    
        d = (x2 * x3s - x2s * x3) - x1 * (x3s - x2s) + x1s * (x3 - x2);
        d1 = y1 * (x2 * x3s - x2s * x3) - x1 * (y2 * x3s - x2s * y3) + x1s * (y2 * x3 - x2 * y3);
        d2 = (y2 * x3s - x2s * y3) - y1 * (x3s - x2s) + x1s * (y3 - y2);
        d3 = (x2 * y3 - y2 * x3) - x1 * (y3 - y2) + y1 * (x3 - x2);
    
        coeff1 = d1/d;
        coeff2 = d2/d;
        coeff3 = d3/d;
    }
    else if (order == 11)
    {
       cpl_msg_info(cpl_func,"Fitting %d points to %d points using 1st order polynomial \n", numOfIn, numOfOut);
        fprintf (midiReportPtr, "Fitting %d points to %d points using 1st order polynomial \n", numOfIn, numOfOut);

        //    Load input data
        valuePloy = (double *) calloc (numOfIn, sizeof (double));
        positionPoly = (double *) calloc (numOfIn, sizeof (double));
        for (i = 0; i < numOfIn; i++)
        {
            positionPoly[i] = (double) (xCoord[i]);
            valuePloy[i] = (double) (yCoord[i]);
        }

        //    We assume no knowledge of standard deviation on input data
        sigData = 0.0;
        sigDataAvailable = 0;
        midiGetLinearFit (positionPoly, valuePloy, numOfIn, sigData, sigDataAvailable, 
            &coeff1, &coeff2, &siga, &sigb, &chi2, &q, error);
        free (positionPoly);
        free (valuePloy);
        if (*error)
        {
            sprintf (midiMessage, "Cannot compute linear fit in routine '%s'", routine);
            midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
            return;
        }
    }
    else if (order == 22)
    {
       cpl_msg_info(cpl_func,"Fitting %d points to %d points using 2nd order polynomial \n", numOfIn, numOfOut);
        fprintf (midiReportPtr, "Fitting %d points to %d points using 2nd order polynomial \n", numOfIn, numOfOut);
    
        //    Load input data
        valuePloy = (double *) calloc (numOfIn, sizeof (double));
        positionPoly = (double *) calloc (numOfIn, sizeof (double));
        for (i = 0; i < numOfIn; i++)
        {
            positionPoly[i] = (double) (xCoord[i]);
            valuePloy[i] = (double) (yCoord[i]);
        }

        degPoly = 2;
        xPoly = cpl_vector_wrap (numOfIn, positionPoly);
        yPoly = cpl_vector_wrap (numOfIn, valuePloy);
        coeffPoly = cpl_polynomial_fit_1d_create (xPoly, yPoly, degPoly, &mse);
        power = 0;
        coeff1 = cpl_polynomial_get_coeff (coeffPoly, &power);
        power = 1;
        coeff2 = cpl_polynomial_get_coeff (coeffPoly, &power);
        power = 2;
        coeff3 = cpl_polynomial_get_coeff (coeffPoly, &power);

        //    Release memory
        cpl_vector_unwrap (xPoly);
        cpl_vector_unwrap (yPoly);
        cpl_polynomial_delete (coeffPoly);
        free (positionPoly);
        free (valuePloy);
    }
    else
    {
        *error = 1;
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Invalid polynomial order");
        return;
    }

   cpl_msg_info(cpl_func,"Coeffient 1, Uncertainty = (%f, %f) \n", coeff1, siga);
   cpl_msg_info(cpl_func,"Coeffient 2, Uncertainty = (%f, %f) \n", coeff2, sigb);
   cpl_msg_info(cpl_func,"Coeffient 3, Uncertainty = (%f, %f) \n", coeff3, sigc);
   cpl_msg_info(cpl_func,"Mean Squared Error       = %g \n", mse);
   cpl_msg_info(cpl_func,"Chi-squared              = %f \n", chi2);
   cpl_msg_info(cpl_func,"Goodness-of-fit          = %f \n", q);
    fprintf (midiReportPtr, "Coeffient 1, Uncertainty = (%f, %f) \n", coeff1, siga);
    fprintf (midiReportPtr, "Coeffient 2, Uncertainty = (%f, %f) \n", coeff2, sigb);
    fprintf (midiReportPtr, "Coeffient 3, Uncertainty = (%f, %f) \n", coeff3, sigc);
    fprintf (midiReportPtr, "Mean Squared Error       = %g \n", mse);
    fprintf (midiReportPtr, "Chi-squared              = %f \n", chi2);
    fprintf (midiReportPtr, "Goodness-of-fit          = %f \n", q);
    
   cpl_msg_info(cpl_func,"NB. negative values for the statistics indicate unavailability \n");
    fprintf (midiReportPtr, "NB. negative values for the statistics indicate unavailability \n");
    
    
    //    Fill in the output array using 
    for (i = 0; i < numOfOut; i++)
        yOut[i] = (coeff1) + (coeff2 * i) + (coeff3 * i * i);

    return; 
}
/*****************************************************************************/



/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  correlateWaveCalibDB
* Input/Output: See function arguments to avoid duplication
* Description:  Correlates the real-time template with that in the database
*
* History:      
* 08-Sep-05    (csabet)
******************************************************************************/
void correlateWaveCalibDB (
    MidiFiles        *fileNames,    // In: Pointer to file names
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat        *format,    // In: Pointer to the image format
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "correlateWaveCalibDB";
    int            i, region;
    float        *arrayError, **waveTable, standDev;
    char        *fileString, *title, *fileName;
    FILE        *filePtr=NULL;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

   cpl_msg_info(cpl_func,"\nCorrelating template with the database \n");
   cpl_msg_info(cpl_func,"-------------------------------------- \n");
    fprintf (midiReportPtr, "\nCorrelating template with the database \n");
    fprintf (midiReportPtr, "-------------------------------------- \n");

    //    Allocate memory
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    arrayError = (float *) calloc (format->iXWidth, sizeof (float));
    waveTable = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (region = 0; region < format->numOfDetectorRegions; region++)
        waveTable[region] = calloc (format->iXWidth, sizeof (float));

    //    We know that this file exists in the calibration database and it's consistent
    sprintf (fileString , "%s_%s_%s.dat", fileNames->waveCalibName, format->grismId, format->beamCombiner);
    if ((filePtr = fopen (fileString, "r")) != NULL)    // It is in the database
    {
        //    Read from file and load into array
       cpl_msg_info(cpl_func,"Reading Wavelength Calibration file ... %s \n", fileString);
        fprintf (midiReportPtr, "Reading Wavelength Calibration file ... %s \n", fileString);
        i = 0;
        //    Read calibrated frequency from the database in THz
        if (strcmp (format->beamCombiner, "SCI_PHOT") == 0)
        {
            while (fscanf (filePtr, "%f %f %f %f\n", &(waveTable[0][i]), &(waveTable[1][i]),
                &(waveTable[2][i]), &(waveTable[3][i])) != EOF) i++;
        }
        else
        {
            while (fscanf (filePtr, "%f %f \n", &(waveTable[0][i]), &(waveTable[1][i])) != EOF) i++;
        }

        fclose (filePtr);
       cpl_msg_info(cpl_func,"Read %d wavelength values in micron \n", i*format->numOfDetectorRegions);
        fprintf (midiReportPtr, "Read %d pairs of wavelengths in micron \n", i);

        //    Do correlation
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            //    Compute error series
            for (i = 0; i < format->iXWidth; i++)
                arrayError[i] = waveCal->calibratedWave[region][i] - waveTable[region][i];

            if (plotFile)
            {
                fileName = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));

                sprintf (fileName , "WavelengthCalibDataBaseDATA%d", region+1);
                sprintf (title , "Wavelength Calibration from Database, Region %d", region+1);
                midiCreatePlotFile2D (fileName, title, "Channel", "Wavelength in micro", 0, 
                    waveTable[region], 0, format->iXWidth, 1, 0);

                sprintf (fileName , "WavelengthCalibCurrentTemplateDATA%d", region+1);
                sprintf (title , "Wavelength Calibration for Current template, Region %d", region+1);
                midiCreatePlotFile2D (fileName, title, "Channel", "Wavelength in micron", 0, 
                    waveCal->calibratedWave[region], 0, format->iXWidth, 1, 0);

                sprintf (fileName , "CorrelationErrorDATA%d", region+1);
                sprintf (title , "Correlation Error of Region %d", region+1);
                midiCreatePlotFile2D (fileName, title, "Channel", "Wavelength in micron", 0, 
                    arrayError, 0, format->iXWidth, 1, 0);
                free (fileName);
                free (title);
            }
                
            //    Compute variance
            waveCal->variance[region] = signalVariance (arrayError,    0, format->iXWidth,    &standDev);
           cpl_msg_info(cpl_func,"Error Variance for region %d = %f \n", region+1, waveCal->variance[region]);
            fprintf (midiReportPtr, "Error Variance for region %d = %f \n", region+1, waveCal->variance[region]);
        }
    }
    else 
    {
        *error = 1;
        sprintf (midiMessage, "Cannot find calibration data file ... %s. Nothing to correlate", fileString);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    }

    //    Release memory
    free (fileString);
    free (arrayError);
    for (region = 0; region < format->numOfDetectorRegions; region++) free (waveTable[region]);
    free (waveTable);
        
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  createWaveCalibDB
* Input/Output: See function arguments to avoid duplication
* Description:  Creates an approximate wavelength calibration file based on the
*                Narrow band filter spectra only
*
* History:      
* 08-Sep-05    (csabet)
******************************************************************************/
void createWaveCalibDB (
    MidiFiles        *fileNames,    // In: Pointer to file names
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat        *format,    // In: Pointer to the image format
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "createWaveCalibDB";
    char        *fileString, *title;
    FILE        *filePtr=NULL;
    int            i, region;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    //    Plot for diagnostic
    if (plotFile && diagnostic)
    {
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        for (region = 0; region < format->numOfDetectorRegions; region++)
        {
            sprintf (fileString , "WavelengthCalib_%s_%sDATA%d", format->grismId, format->beamCombiner, region+1);
            sprintf (title , "Wavelength Calibration for %s %s, Region %d", format->grismId, format->beamCombiner, region+1);
            midiCreatePlotFile2D (fileString, title, "Channel", "Wavelength in micron", 0, 
                waveCal->calibratedWave[region], 0, format->iXWidth, 1, 0);
        }
        free (fileString);
        free (title);
    }
    
    //    Create database file
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (fileString , "%s_%s_%s.dat", fileNames->waveCalibName, format->grismId, format->beamCombiner);
    if ((filePtr = fopen (fileString, "w")) == NULL)
    {
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, "Cannot create wavelength calibration file");
        *error = 1;
        free (fileString);
        return;
    }
    
    //    Write the wavelength data into file
    for (i = 0; i < format->iXWidth; i++)
    {
        for (region = 0; region < format->numOfDetectorRegions; region++)
            fprintf (filePtr, "%3.10f ", waveCal->calibratedWave[region][i]);
        fprintf (filePtr, "\n");    
    }
    
   cpl_msg_info(cpl_func,"\nCreated wavelength calibration file ... %s \n", fileString);
    fprintf (midiReportPtr, "\nCreated wavelength calibration file ... %s \n", fileString);
    
    //    Close files and release memory
    free (fileString);    
    fclose (filePtr);

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  fitFilterSpectra
* Input/Output: See function arguments to avoid duplication
* Description:  Fitting of filter spectra. The peaks of the filters are:
*
*               Filter  Pk Freq.        Pk Wavelength
*                ======================================
*                NeII    2.34297e+13 Hz  12.7955  micron
*                SIV     2.85818e+13 Hz  10.489   micron
*                ArIII   3.33005e+13 Hz   9.00272 micron
*
*                Use corresponding image for each filter and, using the Gaussian Fit
*                method, find the positions of the centre-pinholes (there should be
*                3 for each filter image). From these computed positions and the above
*                given peak values, one can assign wavelength to the detector channels.
*                This facilitates a partial detector calibration.
*
* History:      
* 14-June-05    (csabet) Created
******************************************************************************/
void fitFilterSpectra (
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat        *format,    // In: Pointer to the image format
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "fitFilterSpectra";
    int            region;

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    
    //    Get the spectra for the NeII filter
    for (region = 0; region < format->numOfDetectorRegions; region++)
        getFilterSpectra (region, "NeII", (waveCal->NeII->image)[region], format, 
            &(waveCal->NeII->xCoord[region]), &(waveCal->NeII->yCoord[region]), error);
    
    //    Get the spectra for the SIV filter
    for (region = 0; region < format->numOfDetectorRegions; region++)
        getFilterSpectra (region, "SIV", (waveCal->SIV->image)[region], format, 
            &(waveCal->SIV->xCoord[region]), &(waveCal->SIV->yCoord[region]), error);
    
    //    Get the spectra for the ArIII filter
    for (region = 0; region < format->numOfDetectorRegions; region++)
        getFilterSpectra (region, "ArIII", (waveCal->ArIII->image)[region], format, 
            &(waveCal->ArIII->xCoord[region]), &(waveCal->ArIII->yCoord[region]), error);

    //    Computed centre pinholes
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
       cpl_msg_info(cpl_func,"\nComputed Centre Pinholes for region %d: \n", region+1);
       cpl_msg_info(cpl_func,"----------------------------------- \n");
       cpl_msg_info(cpl_func,"NeII Centre Pinhole  = (%f, %f) \n", (waveCal->NeII->xCoord)[region], (waveCal->NeII->yCoord)[region]);
       cpl_msg_info(cpl_func,"SIV Centre Pinhole   = (%f, %f) \n", (waveCal->SIV->xCoord)[region], (waveCal->SIV->yCoord)[region]);
       cpl_msg_info(cpl_func,"ArIII Centre Pinhole = (%f, %f) \n", (waveCal->ArIII->xCoord)[region], (waveCal->ArIII->yCoord)[region]);
    
        fprintf (midiReportPtr, "\nComputed Centre Pinholes for region %d: \n", region+1);
        fprintf (midiReportPtr, "----------------------------------- \n");
        fprintf (midiReportPtr, "NeII Centre Pinhole  = (%f, %f) \n", 
            (waveCal->NeII->xCoord)[region], (waveCal->NeII->yCoord)[region]);
        fprintf (midiReportPtr, "SIV Centre Pinhole   = (%f, %f) \n", 
            (waveCal->SIV->xCoord)[region], (waveCal->SIV->yCoord)[region]);
        fprintf (midiReportPtr, "ArIII Centre Pinhole = (%f, %f) \n", 
            (waveCal->ArIII->xCoord)[region], (waveCal->ArIII->yCoord)[region]);
    }
        
    //    Assign computed peak values to the peak values given by the manufacturer
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
       cpl_msg_info(cpl_func,"\nDetector Channel Assignment for region %d: \n", region+1);
       cpl_msg_info(cpl_func,"-------------------------------------- \n");
       cpl_msg_info(cpl_func,"Channel %3d = %f THz  %f micron \n", (int) ((waveCal->NeII->xCoord)[region]),
            FREQ_PEAK_NeII, WAVELENGTH_PEAK_NeII);
       cpl_msg_info(cpl_func,"Channel %3d = %f THz  %f micron \n", (int) ((waveCal->SIV->xCoord)[region]),
            FREQ_PEAK_SIV, WAVELENGTH_PEAK_SIV);
       cpl_msg_info(cpl_func,"Channel %3d = %f THz  %f micron \n", (int) ((waveCal->ArIII->xCoord)[region]),
            FREQ_PEAK_ArIII, WAVELENGTH_PEAK_ArIII);
    
        fprintf (midiReportPtr, "\nDetector Channel Assignment for region %d: \n", region+1);
        fprintf (midiReportPtr, "-------------------------------------- \n");
        fprintf (midiReportPtr, "Channel %3d = %f THz  %f micron \n", (int) ((waveCal->NeII->xCoord)[region]),
            FREQ_PEAK_NeII, WAVELENGTH_PEAK_NeII);
        fprintf (midiReportPtr, "Channel %3d = %f THz  %f micron \n", (int) ((waveCal->SIV->xCoord)[region]),
            FREQ_PEAK_SIV, WAVELENGTH_PEAK_SIV);
        fprintf (midiReportPtr, "Channel %3d = %f THz  %f micron \n", (int) ((waveCal->ArIII->xCoord)[region]),
            FREQ_PEAK_ArIII, WAVELENGTH_PEAK_ArIII);
    
        (waveCal->NeII->wavelength)[region] = WAVELENGTH_PEAK_NeII;
        (waveCal->SIV->wavelength)[region] = WAVELENGTH_PEAK_SIV;
        (waveCal->ArIII->wavelength)[region] = WAVELENGTH_PEAK_ArIII;
    }
    
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getFilterSpectra
* Input/Output: See function arguments to avoid duplication
* Description:  Determines the position of the centre-pinhole in the given image
*
* History:      
* 30-Aug-05    (csabet)
******************************************************************************/
void getFilterSpectra (
    int            region,        // In: Region of the image
    const char        *filter,    // In: Name of the filter
    float         *image,        // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat    *format,    // In: Pointer to the image format
    float        *xCPH,        // Ou: X coordinate of the centre pinhole
    float        *yCPH,        // Ou: Y coordinate of the centre pinhole
    int            *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "getFilterSpectra";
    int            i, j, frame, span, quartSpan;
    float        *array, *arrayPeak, maxFlux, peak, fluxMax;
    char        *fileString, *title;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    *xCPH = 0.0;
    *yCPH = 0.0;
    span = EXPECTED_PINHOLE_WIDTH;
    quartSpan = span / 4;


   cpl_msg_info(cpl_func,"\nComputing Coordinates for %s filter region %d \n", filter, region+1);
   cpl_msg_info(cpl_func,"-------------------------------------------- \n");
    fprintf (midiReportPtr, "\nComputing Coordinates for %s filter region %d\n", filter, region+1);
    fprintf (midiReportPtr, "-------------------------------------------- \n");
    
    //================================================
    //    This section computes the approximate X coordinate of the centre pinhole
    
    //    Allocate memory
    array = (float *) calloc (format->iXWidth, sizeof (float));
    
    //    Compress spectrum in the Y direction
    for (i = 0; i < format->iYWidth; i++)
    {
        frame = i * format->iXWidth;
        for (j = 0; j < format->iXWidth; j++)
            array[j] += image[frame + j];
    }
    
    //    Normalize array for a better correlation
    fluxMax = array[0];
    for (i = 0; i < format->iXWidth; i++)
        if (array[i] > fluxMax) fluxMax = array[i];
    for (i = 0; i < format->iXWidth; i++)
        array[i] /= fluxMax;

    if (plotFile && diagnostic)
    {
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                
        sprintf (fileString , "%sSpectraAveAlongYDATA%d", filter, region+1);
        sprintf (title , "%s Spectra averaged along Y region %d", filter, region+1);
        midiCreatePlotFile2D (fileString, title, "Channel", "Flux", 0, array, 0, format->iXWidth, 1, 0);
        free (fileString);
        free (title);
    }

    //    The peak of the above spectra gives the approximate x-coordinate of the centre-pinhole
    maxFlux = 0.0;
    peak = 0;
    for (i = 0; i < format->iXWidth; i++)
    {
        if (array[i] > maxFlux)
        {
            maxFlux = array[i];
            peak = i;
        }
    }

    //    Now smooth the array around the peak
    *xCPH = midiGaussianSmooth (array, format->iXWidth, peak, span, error);
   cpl_msg_info(cpl_func,"Found apprximate X at %f \n", *xCPH);
    fprintf (midiReportPtr, "Found apprximate X at %f \n", *xCPH);
    
    //    Release memory
    free (array);
    //================================================



    //================================================
    //    This section computes the exact Y coordinate of the centre pinhole
    
    //    Allocate memory
    array = (float *) calloc (format->iYWidth, sizeof (float));
    arrayPeak = (float *) calloc (3, sizeof (float));
    
    //    Compress spectrum in the X direction
    for (i = 0; i < format->iYWidth; i++)
    {
        frame = i * format->iXWidth;
        for (j = 0; j < format->iXWidth; j++)
            array[i] += image[frame + j];
    }

    //    Normalize array for a better correlation
    fluxMax = array[0];
    for (i = 0; i < format->iYWidth; i++)
        if (array[i] > fluxMax) fluxMax = array[i];
    for (i = 0; i < format->iYWidth; i++)
        array[i] /= fluxMax;

    if (plotFile && diagnostic)
    {
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                
        sprintf (fileString , "%sSpectraAveAlongXDATA%d", filter, region+1);
        sprintf (title , "%s Spectra averaged along X region %d", filter, region+1);
        midiCreatePlotFile2D (fileString, title, "Sub-frame", "Flux", 0, array, 0, format->iYWidth, 1, 0);
        free (fileString);
        free (title);
    }
    
    //    Here we expect three peaks
    //    Find the strongest peak
    maxFlux = 0.0;
    arrayPeak[0] = 0;
    for (i = 0; i < format->iYWidth; i++)
    {
        if (array[i] > maxFlux)
        {
            maxFlux = array[i];
            arrayPeak[0] = i;
        }
    }
   cpl_msg_info(cpl_func,"Found first Y at      %f \n", arrayPeak[0]);
    fprintf (midiReportPtr, "Found first Y at      %f \n", arrayPeak[0]);
    
    //    Find the second strongest peak
    maxFlux = 0.0;
    arrayPeak[1] = 0;
    for (i = 0; i < format->iYWidth; i++)
    {
        if ( !((i > arrayPeak[0]-quartSpan) && (i < arrayPeak[0]+quartSpan)) )
        {
            if (array[i] > maxFlux)
            {
                maxFlux = array[i];
                arrayPeak[1] = i;
            }
        }
    }
   cpl_msg_info(cpl_func,"Found second Y at     %f \n", arrayPeak[1]);
    fprintf (midiReportPtr, "Found second Y at     %f \n", arrayPeak[1]);

    //    Find the third strongest peak
    maxFlux = 0.0;
    arrayPeak[2] = 0;
    for (i = 0; i < format->iYWidth; i++)
    {
        if ( !((i > arrayPeak[0]-quartSpan) && (i < arrayPeak[0]+quartSpan)) && 
            !((i > arrayPeak[1]-quartSpan) && (i < arrayPeak[1]+quartSpan)) ) 
        {
            if (array[i] > maxFlux)
            {
                maxFlux = array[i];
                arrayPeak[2] = i;
            }
        }
    }
   cpl_msg_info(cpl_func,"Found third Y at      %f \n", arrayPeak[2]);
    fprintf (midiReportPtr, "Found third Y at      %f \n", arrayPeak[2]);

    //    Find the median of the above three peaks
    peak = signalMedian (arrayPeak, 0, 3);
    free (arrayPeak);
   cpl_msg_info(cpl_func,"Found median Y at     %f \n", peak);
    fprintf (midiReportPtr, "Found median Y at     %f \n", peak);
    
    //    Normalise around median
    if (peak == 0)
    {
        *error = 1;
        midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, "Cannot find median");
        free (array);
        return;
    }
    fluxMax = array[0];
    if (quartSpan > peak)
        quartSpan = peak;
        
    for (i = peak-quartSpan; i <= peak+quartSpan; i++)
        if (array[i] > fluxMax) fluxMax = array[i];
    for (i = peak-quartSpan; i <= peak+quartSpan; i++)
        array[i] /= fluxMax;

    if (plotFile && diagnostic)
    {
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                
        sprintf (fileString , "%sCentrePinholeAlongYDATA%d", filter, region+1);
        sprintf (title , "%s Centre Pinhole along Y region %d", filter, region+1);
        midiCreatePlotFile2D (fileString, title, "Sub-frame", "Flux", 0, array, peak-quartSpan, peak+quartSpan+1, 1, 0);
        free (fileString);
        free (title);
    }
    
    //    Now smooth the array around the median and find the exact Y coordinate
    *yCPH = midiGaussianSmooth (array, format->iYWidth, peak, quartSpan, error);
   cpl_msg_info(cpl_func,"Found exact Y at      %f \n", *yCPH);
    fprintf (midiReportPtr, "Found exact Y at      %f \n", *yCPH);
    free (array);
    //================================================


    
    //================================================
    //    This section computes the exact X coordinate of the centre pinhole
    
    //    Allocate memory
    array = (float *) calloc (format->iXWidth, sizeof (float));
    
    //    Compress spectrum in the Y direction around the exact Y coordinate
    for (i = *yCPH-quartSpan; i < *yCPH+quartSpan; i++)
    {
        frame = i * format->iXWidth;
        for (j = 0; j < format->iXWidth; j++)
            array[j] += image[frame + j];
    }
    
    //    Normalize array for a better correlation
    fluxMax = array[0];
    for (i = 0; i < format->iXWidth; i++)
        if (array[i] > fluxMax) fluxMax = array[i];
    for (i = 0; i < format->iXWidth; i++)
        array[i] /= fluxMax;

    if (plotFile && diagnostic)
    {
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                
        sprintf (fileString , "%sCentrePinholeAveAlongYDATA%d", filter, region+1);
        sprintf (title , "%s Centre pinhole averaged along Y region %d", filter, region+1);
        midiCreatePlotFile2D (fileString, title, "Channel", "Flux", 0, array, 0, format->iXWidth, 1, 0);
        free (fileString);
        free (title);
    }

    //    The peak of the above spectra gives the approximate x-coordinate of the centre-pinhole
    maxFlux = 0.0;
    peak = 0;
    for (i = 0; i < format->iXWidth; i++)
    {
        if (array[i] > maxFlux)
        {
            maxFlux = array[i];
            peak = i;
        }
    }

    //    Now smooth the array around the peak
    *xCPH = midiGaussianSmooth (array, format->iXWidth, peak, span, error);
   cpl_msg_info(cpl_func,"Found exact X at      %f \n", *xCPH);
    fprintf (midiReportPtr, "Found exact X at      %f \n", *xCPH);
    //================================================
    
    //    Release memory
    free (array);
    //================================================
    
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  fitFoilSpectra
* Input/Output: See function arguments to avoid duplication
* Description:  Fitting of Foil spectra
*
* History:      
* 14-June-05    (csabet)
******************************************************************************/
void fitFoilSpectra (
    MidiFiles        *fileNames,    // In: Pointer to file names
    WaveCalibration *waveCal,    // Ou: Pointer to the Wavelength Calibration data structure
    ImageFormat        *format,    // In: Pointer to the image format
    int                *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "fitFoilSpectra";
    float        **foilSpectra, **openSpectra, fluxMax;
    int            i, region, length;
    char        *fileString, *title;
    FILE        *wlFilePtr=NULL;
    
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;
    
    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire code to be designed");
    
    //    Allocate memory
    foilSpectra = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    openSpectra = (float **) calloc (format->numOfDetectorRegions, sizeof (float *));
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        foilSpectra[region] = (float *) calloc (format->iXWidth, sizeof (float));
        openSpectra[region] = (float *) calloc (format->iXWidth, sizeof (float));
    }
    
    //    For each region get foil spectra
    for (region = 0; region < format->numOfDetectorRegions; region++)
        getFoilSpectra (region, "Foil", waveCal->foil->image[region], format, foilSpectra[region], error);

    //    For each region get OPEN spectra
    for (region = 0; region < format->numOfDetectorRegions; region++)
        getFoilSpectra (region, "Open", waveCal->open->image[region], format, openSpectra[region], error);

    //    Divide foilSpectra by openSpectra
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        for (i = 0; i < format->iXWidth; i++)
            foilSpectra[region][i] /= openSpectra[region][i];

        //    Normalize array for a better correlation
        fluxMax = foilSpectra[region][0];
        for (i = 0; i < format->iXWidth; i++)
            if (foilSpectra[region][i] > fluxMax) fluxMax = foilSpectra[region][i];
        for (i = 0; i < format->iXWidth; i++)
            foilSpectra[region][i] /= fluxMax;

        //    Plot it
        if (plotFile && diagnostic)
        {
            fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
            sprintf (fileString , "FoilDivideOpenAveAlongYDATA%d", region+1);
            sprintf (title , "Foil divided by Open Averaged along Y region %d", region+1);
            midiCreatePlotFile2D (fileString, title, "Channel", "Flux", 0, foilSpectra[region], 0, format->iXWidth, 1, 0);
            free (fileString);
            free (title);
        }
    }

    //    Compare with published spectra. Load the WL-CALIB_2 spectra
    fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
    sprintf (fileString, "%swl-calib_2.dat", fileNames->calibDbDir);
    if ((wlFilePtr = fopen (fileString, "r")) == NULL)
    {
        sprintf (midiMessage, "Cannot Open %s", fileString);
        midiReportWarning (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
        free (fileString);
        free (title);
        *error = 1;
        return;
    }
    
    //    Ignore first record which gives the title of the data file
    if( fgets (title, MAX_STRING_LENGTH, wlFilePtr) != NULL)
    	sprintf (midiMessage, "Title %s", title);
    length = 0;
    //    First column in the file is the    wavenumber and the second is transmission
    while (fscanf (wlFilePtr, "%f %f\n", &(waveCal->foil->actualWavelength[length]), 
        &(waveCal->foil->actualTransmission[length])) != EOF) length++;

    //    Convert to microns
    for (i = 0; i < length; i++)
        waveCal->foil->actualWavelength[i] = 10000.0 * (1.0 / waveCal->foil->actualWavelength[i]);

    sprintf (midiMessage, "Read %d pairs of 'float'(s) from %s", length, fileString);
    midiReportInfo (midiReportPtr, routine, __FILE__, __LINE__, midiMessage);
    fclose (wlFilePtr);
    free (fileString);
    free (title);

    //    Plot manufacturer's Foil transmission
    midiCreatePlotFile2D2P ("FoilTransmission", "Foil Transmission Versus Wavelength", 
        "Wavelength in micron", "Transmission", 0, waveCal->foil->actualWavelength, waveCal->foil->actualTransmission, 0, length, 1);
    
    //    Release memory
    for (region = 0; region < format->numOfDetectorRegions; region++)
    {
        free (foilSpectra[region]); 
        free (openSpectra[region]); 
    }
    free (foilSpectra);
    free (openSpectra);

    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  getFoilSpectra
* Input/Output: See function arguments to avoid duplication
* Description:  Processing of the foil spectra
*
* History:      
* 02-Sep-05    (csabet)
******************************************************************************/
void getFoilSpectra (
    int            region,        // In: Region of the image
    const char        *filter,    // In: Name of the filter
    float        *image,        // In: pointer to the image
    ImageFormat    *format,    // In: Pointer to the image format
    float        *spectra,    // Ou: pointer to the resulting spectra
    int            *error)        // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "getFoilSpectra";
    char        *fileString, *title;
    int            i, j, frame;
    float        fluxMax;
        
    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire code to be designed");    

   cpl_msg_info(cpl_func,"\nObtaining %s Spectra for region %d \n", filter, region+1);
   cpl_msg_info(cpl_func,"------------------------------- \n");
    fprintf (midiReportPtr, "\nObtaining %s Spectra for region %d \n", filter, region+1);
    fprintf (midiReportPtr, "------------------------------- \n");
    
    //    Compress spectrum in the Y direction
    for (i = 0; i < format->iYWidth; i++)
    {
        frame = i * format->iXWidth;
        for (j = 0; j < format->iXWidth; j++)
            spectra[j] += image[frame + j];
    }

    //    Normalize array for a better correlation
    fluxMax = spectra[0];
    for (i = 0; i < format->iXWidth; i++)
        if (spectra[i] > fluxMax) fluxMax = spectra[i];
    for (i = 0; i < format->iXWidth; i++)
        spectra[i] /= fluxMax;

    //    Plot it
    if (plotFile && diagnostic)
    {
        fileString = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
        title = (char *) calloc (MAX_STRING_LENGTH, sizeof (char));
                
        sprintf (fileString , "%sSpectraAveAlongYDATA%d", filter, region+1);
        sprintf (title , "%s Spectra Averaged along Y region %d", filter, region+1);
        midiCreatePlotFile2D (fileString, title, "Channel", "Flux", 0, spectra, 0, format->iXWidth, 1, 0);
        free (fileString);
        free (title);
    }
        
    return; 
}
/*****************************************************************************/


/******************************************************************************
*               European Southern Observatory
*          VLTI MIDI Maintenance Templates Software
*
* Module name:  fitOpenSpectra
* Input/Output: See function arguments to avoid duplication
* Description:  Fitting of Open spectra
*
* History:      
* 14-June-05    (csabet)
******************************************************************************/
void fitOpenSpectra (
    int    *error)    // Ou: Error status

{

    //  Local Declarations
    //    ------------------
    const char    routine[] = "fitOpenSpectra";

    //  Algorithm
    //    ---------
    if (diagnostic > 4)cpl_msg_info(cpl_func,"Invoking      routine   '%s' \n", routine);
    if (diagnostic > 4) fprintf (midiReportPtr, "Invoking      routine   '%s' \n", routine);

    //    Reset status
    *error = 0;

    midiReportTbd (midiReportPtr, routine, __FILE__, __LINE__, "Entire code to be designed");    

    return; 
}
/*****************************************************************************/
