/* $Id: midi_raw_to_fitsimage.c,v 1.8 2010-05-28 09:16:01 agabasch Exp $
 *
 * This file is part of the MIDI Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: agabasch $
 * $Date: 2010-05-28 09:16:01 $
 * $Revision: 1.8 $
 * $Name: not supported by cvs2svn $
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/

#include <cpl.h>

#include "midi_utils.h"
#include "midi_pfits.h"
#include "midi_dfs.h"
#include "string.h"
#include "midiTableToFits.h"
/*-----------------------------------------------------------------------------
                            Private function prototypes
 -----------------------------------------------------------------------------*/

static int midi_raw_to_fitsimage_create(cpl_plugin *);
static int midi_raw_to_fitsimage_exec(cpl_plugin *);
static int midi_raw_to_fitsimage_destroy(cpl_plugin *);
static int midi_raw_to_fitsimage(cpl_frameset *, const cpl_parameterlist *);

/*-----------------------------------------------------------------------------
                            Static variables
 -----------------------------------------------------------------------------*/

static char midi_raw_to_fitsimage_description[] =
"The main purpose of this recipe is to convert the imaging sections of the\n"
"Midi raw files into fits-cubes or fits-images.  This recipe is able to\n"
"process all midi raw files with a DATA1/2/3/4 column in the\n"
"IMAGING_DATA extension, i.e. most of the midi raw files. Therefore no\n"
"special classification tag needs to be given to the SOF. Moreover, the SOF\n"
"should include only one midi raw file.\n"
" \n"
"As this recipe calculates up to 28 diagnostic product files the user\n"
"should refer to the pipeline manual for a detailed description.\n"
"\n";

/*-----------------------------------------------------------------------------
                                Function code
 -----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------*/
/**
 * @defgroup midi_raw_to_fitsimage    Raw to Fitsimage
 *
 */
/*----------------------------------------------------------------------------*/

/**@{*/

/*----------------------------------------------------------------------------*/
/**
  @brief    Build the list of available plugins, for this module. 
  @param    list    the plugin list
  @return   0 if everything is ok, 1 otherwise
  @note     Only this function is exported

  Create the recipe instance and make it available to the application using the 
  interface. 
 */
/*----------------------------------------------------------------------------*/
int cpl_plugin_get_info(cpl_pluginlist * list)
{
    cpl_recipe  *   recipe = cpl_calloc(1, sizeof *recipe );
    cpl_plugin  *   plugin = &recipe->interface;

    if (cpl_plugin_init(plugin,
                    CPL_PLUGIN_API,
                    MIDI_BINARY_VERSION,
                    CPL_PLUGIN_TYPE_RECIPE,
                    "midi_raw_to_fitsimage",
                    "MIDI raw data display",
                    midi_raw_to_fitsimage_description,
                    "Armin Gabasch",
                    PACKAGE_BUGREPORT,
                    midi_get_license(),
                    midi_raw_to_fitsimage_create,
                    midi_raw_to_fitsimage_exec,
                    midi_raw_to_fitsimage_destroy)) {    
        cpl_msg_error(cpl_func, "Plugin initialization failed");
        (void)cpl_error_set_where(cpl_func);                          
        return 1;                                               
    }                                                    

    if (cpl_pluginlist_append(list, plugin)) {                 
        cpl_msg_error(cpl_func, "Error adding plugin to list");
        (void)cpl_error_set_where(cpl_func);                         
        return 1;                                              
    }                                                          
    
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Setup the recipe options    
  @param    plugin  the plugin
  @return   0 if everything is ok

  Defining the command-line/configuration parameters for the recipe.
 */
/*----------------------------------------------------------------------------*/
static int midi_raw_to_fitsimage_create(cpl_plugin * plugin)
{
    cpl_recipe    * recipe;                                               
    cpl_parameter * p;
                                                                       
    /* Do not create the recipe if an error code is already set */     
    if (cpl_error_get_code() != CPL_ERROR_NONE) {                      
        cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",
                      cpl_func, __LINE__, cpl_error_get_where());      
        return (int)cpl_error_get_code();                              
    }                                                                  
                                                                       
    if (plugin == NULL) {                                              
        cpl_msg_error(cpl_func, "Null plugin");                        
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                 
    }                                                                  
                                                                       
    /* Verify plugin type */                                           
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {       
        cpl_msg_error(cpl_func, "Plugin is not a recipe");             
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);              
    }                                                                  
                                                                       
    /* Get the recipe */                                               
    recipe = (cpl_recipe *)plugin;                                     
                                                                       
    /* Create the parameters list in the cpl_recipe object */          
    recipe->parameters = cpl_parameterlist_new();                      
    if (recipe->parameters == NULL) {                                  
        cpl_msg_error(cpl_func, "Parameter list allocation failed");   
        cpl_ensure_code(0, (int)CPL_ERROR_ILLEGAL_OUTPUT);             
    }                                                                  

    /* Fill the parameters list */
    /* --stropt */
    p = cpl_parameter_new_value("midi.midi_raw_to_fitsimage.outputfilename", 
            CPL_TYPE_STRING, "Output Filename", "midi.midi_raw_to_fitsimage","cube.fits");
    cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "outfile");
    cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV);
    cpl_parameterlist_append(recipe->parameters, p);

/*     /\* --boolopt *\/ */
/*     p = cpl_parameter_new_value("midi.midi_raw_to_fitsimage.bool_option",  */
/*             CPL_TYPE_BOOL, "a flag", "midi.midi_raw_to_fitsimage", TRUE); */
/*     cpl_parameter_set_alias(p, CPL_PARAMETER_MODE_CLI, "boolopt"); */
/*     cpl_parameter_disable(p, CPL_PARAMETER_MODE_ENV); */
/*     cpl_parameterlist_append(recipe->parameters, p); */
 
    return 0;
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Execute the plugin instance given by the interface
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_raw_to_fitsimage_exec(cpl_plugin * plugin)
{

    cpl_recipe * recipe;                                                   
    int recipe_status;                                                     
    cpl_errorstate initial_errorstate = cpl_errorstate_get();              
                                                                           
    /* Return immediately if an error code is already set */               
    if (cpl_error_get_code() != CPL_ERROR_NONE) {                          
        cpl_msg_error(cpl_func, "%s():%d: An error is already set: %s",    
                      cpl_func, __LINE__, cpl_error_get_where());          
        return (int)cpl_error_get_code();                                  
    }                                                                      
                                                                           
    if (plugin == NULL) {                                                  
        cpl_msg_error(cpl_func, "Null plugin");                            
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
                                                                           
    /* Verify plugin type */                                               
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {           
        cpl_msg_error(cpl_func, "Plugin is not a recipe");                 
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);                  
    }                                                                      
                                                                           
    /* Get the recipe */                                                   
    recipe = (cpl_recipe *)plugin;                                         
                                                                           
    /* Verify parameter and frame lists */                                 
    if (recipe->parameters == NULL) {                                      
        cpl_msg_error(cpl_func, "Recipe invoked with NULL parameter list");
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
    if (recipe->frames == NULL) {                                          
        cpl_msg_error(cpl_func, "Recipe invoked with NULL frame set");     
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);                     
    }                                                                      
                                                                           
    /* Invoke the recipe */                                                
    recipe_status = midi_raw_to_fitsimage(recipe->frames, recipe->parameters);
                                                                           
    /* Ensure DFS-compliance of the products */                            
    if (cpl_dfs_update_product_header(recipe->frames)) {                   
        if (!recipe_status) recipe_status = (int)cpl_error_get_code();                         
    }                                                                      
                                                                           
    if (!cpl_errorstate_is_equal(initial_errorstate)) {                    
        /* Dump the error history since recipe execution start.            
           At this point the recipe cannot recover from the error */       
        cpl_errorstate_dump(initial_errorstate, CPL_FALSE, NULL);          
    }                                                                      
                                                                           
    return recipe_status;                                                  
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Destroy what has been created by the 'create' function
  @param    plugin  the plugin
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_raw_to_fitsimage_destroy(cpl_plugin * plugin)
{
    cpl_recipe * recipe;                                          
                                                                  
    if (plugin == NULL) {                                         
        cpl_msg_error(cpl_func, "Null plugin");                   
        cpl_ensure_code(0, (int)CPL_ERROR_NULL_INPUT);            
    }                                                             
                                                                  
    /* Verify plugin type */                                      
    if (cpl_plugin_get_type(plugin) != CPL_PLUGIN_TYPE_RECIPE) {  
        cpl_msg_error(cpl_func, "Plugin is not a recipe");        
        cpl_ensure_code(0, (int)CPL_ERROR_TYPE_MISMATCH);         
    }                                                             
                                                                  
    /* Get the recipe */                                          
    recipe = (cpl_recipe *)plugin;                                
                                                                  
    cpl_parameterlist_delete(recipe->parameters);             
                                                                  
    return 0;                                                    
}

/*----------------------------------------------------------------------------*/
/**
  @brief    Interpret the command line options and execute the data processing
  @param    frameset   the frames list
  @param    parlist    the parameters list
  @return   0 if everything is ok
 */
/*----------------------------------------------------------------------------*/
static int midi_raw_to_fitsimage(cpl_frameset            * frameset,
                    const cpl_parameterlist * parlist)
{

    const cpl_parameter *   param;
    const char          *   outputfilename;
    const cpl_frame     *   rawframe;
    cpl_table           *   table;
    int extnum;
    int i;
    cpl_propertylist  *  pHeader_tmp;
    int ext_imaging_data;
    char * current_extension;
    /* Use the errorstate to detect an error in a function that does not
       return an error code. */
    cpl_errorstate          prestate = cpl_errorstate_get();



    /* HOW TO RETRIEVE INPUT PARAMETERS */
    /* --stropt */
    param = cpl_parameterlist_find_const(parlist, "midi.midi_raw_to_fitsimage.outputfilename");
    outputfilename = cpl_parameter_get_string(param);


    if (!cpl_errorstate_is_equal(prestate)) {
        return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                          "Could not retrieve the input "
                                          "parameters");
    }

    /* Identify the RAW and CALIB frames in the input frameset */
    cpl_ensure_code(midi_dfs_set_groups(frameset) == CPL_ERROR_NONE,
                    cpl_error_get_code());
 
    /* HOW TO ACCESS INPUT DATA */
    /*  - A required file */
    rawframe = cpl_frameset_get_position(frameset, 0);
/*     rawframe = cpl_frameset_find_const(frameset, MIDI_ACQ); */
    if (rawframe == NULL) {
        /* cpl_frameset_find_const() does not set an error code, when a frame
           is not found, so we will set one here. */
        return (int)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
                                          "SOF does not have any file");
    }
    

    /* Check for a change in the CPL error state */
    /* - if it did change then propagate the error and return */
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
    
    /* NOW PERFORMING THE DATA REDUCTION */


    /* read the number of extensions from the rawfile */
    extnum=cpl_frame_get_nextensions(rawframe);
    cpl_msg_info(cpl_func, "Number of extensions found: %d",extnum); 


    /* Loop through the extension and find Extension "IMAGING_DATA" */

    ext_imaging_data=0;
/*     pHeader_tmp = cpl_propertylist_new();   */

    for (i=1; i<=extnum; i++){
       pHeader_tmp = cpl_propertylist_load(cpl_frame_get_filename(rawframe), i ); 
       current_extension=(char *)cpl_propertylist_get_string(pHeader_tmp,"EXTNAME");
       cpl_msg_info(cpl_func, "Loop through extensions: %s", current_extension);
       if (strcmp(current_extension,"IMAGING_DATA")==0){
          ext_imaging_data=i;
          cpl_msg_info(cpl_func, " Extension <IMAGING_DATA> found in extension %d",ext_imaging_data);
          cpl_propertylist_delete(pHeader_tmp); 
          break;
       }
       cpl_propertylist_delete(pHeader_tmp);
    }
    

    if (ext_imaging_data==0){
       cpl_msg_error(cpl_func, " Extension <IMAGING_DATA> NOT found");
       return -1;
    }
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());


    /* Load extension  Imaging Data  */ 
    table = cpl_table_load(cpl_frame_get_filename(rawframe), ext_imaging_data, 1);
    if (table == NULL) {
       return (int)cpl_error_set_message(cpl_func, cpl_error_get_code(),
                                         "Could not load the table");
    }
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());

    cpl_msg_info(cpl_func, "Scanning for DATA1 ...");
    if (cpl_table_has_column(table,"DATA1")){
    table_to_fitsimage(frameset, parlist, "DATA1",outputfilename,table);
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
    }
    else {
       cpl_msg_info(cpl_func, "DATA1 not found");
    }

    cpl_msg_info(cpl_func, "Scanning for DATA2 ...");
    if (cpl_table_has_column(table,"DATA2")){
    table_to_fitsimage(frameset, parlist, "DATA2",outputfilename,table);
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
    }
    else {
       cpl_msg_info(cpl_func, "DATA2 not found");
    }


    cpl_msg_info(cpl_func, "Scanning for DATA3 ...");
    if (cpl_table_has_column(table,"DATA3")){
    table_to_fitsimage(frameset, parlist, "DATA3",outputfilename,table);
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
    }
    else {
       cpl_msg_info(cpl_func, "DATA3 not found");
    }

    cpl_msg_info(cpl_func, "Scanning for DATA4 ...");
    if (cpl_table_has_column(table,"DATA4")){
    table_to_fitsimage(frameset, parlist, "DATA4",outputfilename,table);
    cpl_ensure_code(cpl_errorstate_is_equal(prestate), cpl_error_get_code());
    }
    else {
       cpl_msg_info(cpl_func, "DATA4 not found");
    }




/* Free the Memory */

/*      Not to be freed at the moment !!!!!!!*/
/*     cpl_imagelist_delete(imagelist_data1); */
/*     cpl_imagelist_delete(imagelist_data2); */

    cpl_table_delete(table);

    return (int)cpl_error_get_code();
}

/**@}*/
